@@ -1,5 +1,6 @@
 import datetime
 import typing
+from functools import lru_cache
 
 from flyteidl.admin import common_pb2 as _common_pb2
 from flyteidl.admin import execution_pb2 as _execution_pb2
@@ -10,13 +11,17 @@ from flyteidl.admin import project_domain_attributes_pb2 as _project_domain_attr
 from flyteidl.admin import project_pb2 as _project_pb2
 from flyteidl.admin import task_execution_pb2 as _task_execution_pb2
 from flyteidl.admin import task_pb2 as _task_pb2
+from flyteidl.admin import version_pb2 as _version_pb2
 from flyteidl.admin import workflow_attributes_pb2 as _workflow_attributes_pb2
 from flyteidl.admin import workflow_pb2 as _workflow_pb2
+from flyteidl.core import identifier_pb2 as _identifier_pb2
 from flyteidl.service import dataproxy_pb2 as _data_proxy_pb2
+from flyteidl.service.dataproxy_pb2 import ARTIFACT_TYPE_DECK
 from google.protobuf.duration_pb2 import Duration
 
 from flytekit.clients.raw import RawSynchronousFlyteClient as _RawSynchronousFlyteClient
 from flytekit.models import common as _common
+from flytekit.models import domain as _domain
 from flytekit.models import execution as _execution
 from flytekit.models import filters as _filters
 from flytekit.models import launch_plan as _launch_plan
@@ -33,11 +38,13 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
     """
     This is a low-level client that users can use to make direct gRPC service calls to the control plane. See the
     :std:doc:`service spec <idl:protos/docs/service/index>`. This is more user-friendly interface than the
-    :py:class:`raw client <flytekit.clients.raw.RawSynchronousFlyteClient>` so users should try to use this class
-    first. Create a client by ::
+    {{< py_class_ref flytekit.clients.raw.RawSynchronousFlyteClient >}} so users should try to use this class
+    first. Create a client by
 
-        SynchronousFlyteClient("your.domain:port", insecure=True)
-        # insecure should be True if your flyteadmin deployment doesn't have SSL enabled
+    ```python
+    SynchronousFlyteClient("your.domain:port", insecure=True)
+    # insecure should be True if your flyteadmin deployment doesn't have SSL enabled
+    ```
 
     """
 
@@ -60,11 +67,10 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This will create a task definition in the Admin database. Once successful, the task object can be
         retrieved via the client or viewed via the UI or command-line interfaces.
 
-        .. note ::
-
-            Overwrites are not supported so any request for a given project, domain, name, and version that exists in
-            the database must match the existing definition exactly. Furthermore, as long as the request
-            remains identical, calling this method multiple times will result in success.
+        > [!NOTE]
+        > Overwrites are not supported so any request for a given project, domain, name, and version that exists in
+          the database must match the existing definition exactly. Furthermore, as long as the request
+          remains identical, calling this method multiple times will result in success.
 
         :param flytekit.models.core.identifier.Identifier task_identifer: The identifier for this task.
         :param flytekit.models.task.TaskSpec task_spec: This is the actual definition of the task that
@@ -83,14 +89,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This returns a page of identifiers for the tasks for a given project and domain. Filters can also be
         specified.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param Text project: The namespace of the project to list.
@@ -124,14 +128,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This returns a page of task metadata for tasks in a given project and domain.  Optionally,
         specifying a name will limit the results to only tasks with that name in the given project and domain.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param flytekit.models.common.NamedEntityIdentifier identifier: NamedEntityIdentifier to list.
@@ -164,6 +166,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             str(task_list.token),
         )
 
+    @lru_cache
     def get_task(self, id):
         """
         This returns a single task for a given identifier.
@@ -187,9 +190,8 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This will create a workflow definition in the Admin database. Once successful, the workflow object can be
         retrieved via the client or viewed via the UI or command-line interfaces.
 
-        .. note ::
-
-            Overwrites are not supported so any request for a given project, domain, name, and version that exists in
+        > [!NOTE]
+        > Overwrites are not supported so any request for a given project, domain, name, and version that exists in
             the database must match the existing definition exactly. Furthermore, as long as the request
             remains identical, calling this method multiple times will result in success.
 
@@ -212,14 +214,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This returns a page of identifiers for the workflows for a given project and domain. Filters can also be
         specified.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param: Text project: The namespace of the project to list.
@@ -253,14 +253,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This returns a page of workflow meta-information for workflows in a given project and domain.  Optionally,
         specifying a name will limit the results to only workflows with that name in the given project and domain.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param flytekit.models.common.NamedEntityIdentifier identifier: NamedEntityIdentifier to list.
@@ -293,6 +291,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             str(wf_list.token),
         )
 
+    @lru_cache
     def get_workflow(self, id):
         """
         This returns a single workflow for a given ID.
@@ -316,9 +315,8 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This will create a launch plan definition in the Admin database.  Once successful, the launch plan object can be
         retrieved via the client or viewed via the UI or command-line interfaces.
 
-        .. note ::
-
-            Overwrites are not supported so any request for a given project, domain, name, and version that exists in
+        > [!NOTE]
+        > Overwrites are not supported so any request for a given project, domain, name, and version that exists in
             the database must match the existing definition exactly.  This also means that as long as the request
             remains identical, calling this method multiple times will result in success.
 
@@ -337,6 +335,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             )
         )
 
+    @lru_cache
     def get_launch_plan(self, id):
         """
         Retrieves a launch plan entity.
@@ -367,14 +366,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This returns a page of identifiers for the launch plans for a given project and domain. Filters can also be
         specified.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param: Text project: The namespace of the project to list.
@@ -408,14 +405,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         This returns a page of launch plan meta-information for launch plans in a given project and domain.  Optionally,
         specifying a name will limit the results to only workflows with that name in the given project and domain.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param flytekit.models.common.NamedEntityIdentifier identifier: NamedEntityIdentifier to list.
@@ -448,19 +443,19 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             str(lp_list.token),
         )
 
-    def list_active_launch_plans_paginated(self, project, domain, limit=100, token=None, sort_by=None):
+    def list_active_launch_plans_paginated(
+        self, project, domain, limit=100, token=None, sort_by=None
+    ) -> typing.Tuple[typing.List[_launch_plan.LaunchPlan], str]:
         """
         This returns a page of currently active launch plan meta-information for launch plans in a given project and
         domain.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param Text project:
@@ -601,14 +596,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         """
         This returns a page of executions in a given project and domain.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param Text project: Project in which to list executions.
@@ -662,6 +655,17 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             .id
         )
 
+    def get_execution_metrics(self, id, depth=10):
+        return (
+            super(SynchronousFlyteClient, self)
+            .get_execution_metrics(
+                get_execution_metrics_request=_execution_pb2.WorkflowExecutionGetMetricsRequest(
+                    id=id.to_flyte_idl(), depth=depth
+                )
+            )
+            .span
+        )
+
     ####################################################################################################################
     #
     #  Node Execution Endpoints
@@ -802,8 +806,8 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         :param flytekit.models.core.identifier.NodeExecutionIdentifier node_execution_identifier:
         :param int limit:
         :param Text token: [Optional] If specified, this specifies where in the rows of results to skip before reading.
-        If you previously retrieved a page response with token="foo" and you want the next page,
-        specify token="foo".
+            If you previously retrieved a page response with token="foo" and you want the next page,
+            specify token="foo".
         :param list[flytekit.models.filters.Filter] filters:
         :param flytekit.models.admin.common.Sort sort_by: [Optional] If provided, the results will be sorted.
         :rtype: (list[flytekit.models.admin.task_execution.TaskExecution], Text)
@@ -852,14 +856,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         """
         This returns a page of projects.
 
-        .. note ::
-
-            This is a paginated API.  Use the token field in the request to specify a page offset token.
+        > [!NOTE]
+        > This is a paginated API.  Use the token field in the request to specify a page offset token.
             The user of the API is responsible for providing this token.
 
-        .. note ::
-
-            If entries are added to the database between requests for different pages, it is possible to receive
+        > [!NOTE]
+        > If entries are added to the database between requests for different pages, it is possible to receive
             entries on the second page that also appeared on the first.
 
         :param int limit: [Optional] The maximum number of entries to return.  Must be greater than 0.  The maximum
@@ -887,6 +889,21 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             str(projects.token),
         )
 
+    ####################################################################################################################
+    #
+    #  Domain Endpoints
+    #
+    ####################################################################################################################
+
+    def get_domains(self):
+        """
+        This returns a list of domains.
+
+        :rtype: list[flytekit.models.Domain]
+        """
+        domains = super(SynchronousFlyteClient, self).get_domains()
+        return [_domain.Domain.from_flyte_idl(domain) for domain in domains.domains]
+
     ####################################################################################################################
     #
     #  Matching Attributes Endpoints
@@ -978,36 +995,52 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
 
     def get_upload_signed_url(
-        self, project: str, domain: str, content_md5: bytes, filename: str = None, expires_in: datetime.timedelta = None
+        self,
+        project: str,
+        domain: str,
+        content_md5: typing.Optional[bytes] = None,
+        filename: typing.Optional[str] = None,
+        expires_in: typing.Optional[datetime.timedelta] = None,
+        filename_root: typing.Optional[str] = None,
+        add_content_md5_metadata: bool = True,
     ) -> _data_proxy_pb2.CreateUploadLocationResponse:
         """
         Get a signed url to be used during fast registration
-        :param str project: Project to create the upload location for
-        :param str domain: Domain to create the upload location for
-        :param bytes content_md5: ContentMD5 restricts the upload location to the specific MD5 provided. The content_md5
+
+        :param project: Project to create the upload location for
+        :param domain: Domain to create the upload location for
+        :param content_md5: ContentMD5 restricts the upload location to the specific MD5 provided. The content_md5
             will also appear in the generated path.
-        :param str filename: [Optional] If provided this specifies a desired suffix for the generated location
-        :param datetime.timedelta expires_in: [Optional] If provided this defines a requested expiration duration for
+        :param filename: If provided this specifies a desired suffix for the generated location
+        :param expires_in: If provided this defines a requested expiration duration for
             the generated url
+        :param filename_root: If provided will be used as the root of the filename.  If not, Admin will use a hash
+          This option is useful when uploading a series of files that you want to be grouped together.
+        :param add_content_md5_metadata: If true, the content md5 will be added to the metadata in signed URL
         :rtype: flyteidl.service.dataproxy_pb2.CreateUploadLocationResponse
         """
-        expires_in_pb = None
-        if expires_in:
-            expires_in_pb = Duration()
-            expires_in_pb.FromTimedelta(expires_in)
-        return super(SynchronousFlyteClient, self).create_upload_location(
-            _data_proxy_pb2.CreateUploadLocationRequest(
-                project=project,
-                domain=domain,
-                content_md5=content_md5,
-                filename=filename,
-                expires_in=expires_in_pb,
+        try:
+            expires_in_pb = None
+            if expires_in:
+                expires_in_pb = Duration()
+                expires_in_pb.FromTimedelta(expires_in)
+            return super(SynchronousFlyteClient, self).create_upload_location(
+                _data_proxy_pb2.CreateUploadLocationRequest(
+                    project=project,
+                    domain=domain,
+                    content_md5=content_md5,
+                    filename=filename,
+                    expires_in=expires_in_pb,
+                    filename_root=filename_root,
+                    add_content_md5_metadata=add_content_md5_metadata,
+                )
             )
-        )
+        except Exception as e:
+            raise RuntimeError(f"Failed to get signed url for {filename}.") from e
 
     def get_download_signed_url(
         self, native_url: str, expires_in: datetime.timedelta = None
-    ) -> _data_proxy_pb2.CreateUploadLocationResponse:
+    ) -> _data_proxy_pb2.CreateDownloadLocationResponse:
         expires_in_pb = None
         if expires_in:
             expires_in_pb = Duration()
@@ -1018,3 +1051,61 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
                 expires_in=expires_in_pb,
             )
         )
+
+    def get_data(self, flyte_uri: str) -> _data_proxy_pb2.GetDataResponse:
+        req = _data_proxy_pb2.GetDataRequest(flyte_url=flyte_uri)
+
+        resp = self._dataproxy_stub.GetData(req, metadata=self._metadata)
+        return resp
+
+    def get_download_artifact_signed_url(
+        self,
+        node_id: str,
+        project: str,
+        domain: str,
+        name: str,
+        artifact_type: _data_proxy_pb2.ArtifactType = ARTIFACT_TYPE_DECK,
+        expires_in: datetime.timedelta = None,
+    ) -> _data_proxy_pb2.CreateDownloadLinkResponse:
+        """
+        Get a signed url for an artifact.
+
+        :param node_id: Node id associated with artifact
+        :param project: Name of the project the resource belongs to
+        :param domain: Name of the domain the resource belongs to
+        :param name: User or system provided value for the resource
+        :param artifact_type: ArtifactType of the artifact requested
+        :param expires_in: If provided this defines a requested expiration duration for the generated url
+        :rtype: flyteidl.service.dataproxy_pb2.CreateDownloadLinkResponse
+        """
+        expires_in_pb = None
+        if expires_in:
+            expires_in_pb = Duration()
+            expires_in_pb.FromTimedelta(expires_in)
+        return super(SynchronousFlyteClient, self).create_download_link(
+            _data_proxy_pb2.CreateDownloadLinkRequest(
+                artifact_type=artifact_type,
+                node_execution_id=_identifier_pb2.NodeExecutionIdentifier(
+                    node_id=node_id,
+                    execution_id=_identifier_pb2.WorkflowExecutionIdentifier(
+                        project=project,
+                        domain=domain,
+                        name=name,
+                    ),
+                ),
+                expires_in=expires_in_pb,
+            )
+        )
+
+    def get_control_plane_version(self) -> str:
+        """
+        Retrieve the Control Plane version from Flyteadmin.
+
+        This method calls Flyteadmin's GetVersion API to obtain the current version information of the control plane.
+        The retrieved version can be used to enable or disable specific features based on the Flyteadmin version.
+
+        Returns:
+            str: The version string of the control plane.
+        """
+        version_response = self._stub.GetVersion(_version_pb2.GetVersionRequest(), metadata=self._metadata)
+        return version_response.control_plane_version.Version
