@@ -1,40 +1,14 @@
+import sys
 from typing import Tuple, Union
 
-import six as _six
+import click
 from flyteidl.admin.launch_plan_pb2 import LaunchPlan
 from flyteidl.admin.task_pb2 import TaskSpec
 from flyteidl.admin.workflow_pb2 import WorkflowSpec
 from flyteidl.core import identifier_pb2 as _identifier_pb2
 from flyteidl.core import workflow_pb2 as _workflow_pb2
 
-from flytekit.common.types.helpers import get_sdk_type_from_literal_type as _get_sdk_type_from_literal_type
-from flytekit.models import literals as _literals
-
-
-def construct_literal_map_from_variable_map(variable_dict, text_args):
-    """
-    This function produces a map of Literals to use when creating an execution.  It reads the required values from
-    a Variable map (presumably obtained from a launch plan), and then fills in the necessary inputs
-    from the click args. Click args will be strings, which will be parsed into their SDK types with each
-    SDK type's parse string method.
-
-    :param dict[Text, flytekit.models.interface.Variable] variable_dict:
-    :param dict[Text, Text] text_args:
-    :rtype: flytekit.models.literals.LiteralMap
-    """
-    inputs = {}
-
-    for var_name, variable in _six.iteritems(variable_dict):
-        # Check to see if it's passed from click
-        # If this is an input that has a default from the LP, it should've already been parsed into a string,
-        # and inserted into the default for this option, so it should still be here.
-        if var_name in text_args and text_args[var_name] is not None:
-            # the SDK type is also available from the sdk workflow object's saved user inputs but let's
-            # derive it here from the interface to be more rigorous.
-            sdk_type = _get_sdk_type_from_literal_type(variable.type)
-            inputs[var_name] = sdk_type.from_string(text_args[var_name])
-
-    return _literals.LiteralMap(literals=inputs)
+from flytekit.configuration import DOMAIN_PLACEHOLDER, PROJECT_PLACEHOLDER, VERSION_PLACEHOLDER
 
 
 def parse_args_into_dict(input_arguments):
@@ -49,59 +23,31 @@ def parse_args_into_dict(input_arguments):
     return {split_arg[0]: split_arg[1] for split_arg in [input_arg.split("=", 1) for input_arg in input_arguments]}
 
 
-def construct_literal_map_from_parameter_map(parameter_map, text_args):
-    """
-    Take a dictionary of Text to Text and construct a literal map using a ParameterMap as guidance.
-    Required input parameters must have an entry in the text arguments given.
-    Parameters with defaults will have those defaults filled in if missing from the text arguments.
-
-    :param flytekit.models.interface.ParameterMap parameter_map:
-    :param dict[Text, Text] text_args:
-    :rtype: flytekit.models.literals.LiteralMap
-    """
-
-    # This function can be written by calling construct_literal_map_from_variable_map also, but not that much
-    # code is saved.
-    inputs = {}
-    for var_name, parameter in _six.iteritems(parameter_map.parameters):
-        sdk_type = _get_sdk_type_from_literal_type(parameter.var.type)
-        if parameter.required:
-            if var_name in text_args and text_args[var_name] is not None:
-                inputs[var_name] = sdk_type.from_string(text_args[var_name])
-            else:
-                raise Exception("Missing required parameter {}".format(var_name))
-        else:
-            if var_name in text_args and text_args[var_name] is not None:
-                inputs[var_name] = sdk_type.from_string(text_args[var_name])
-            else:
-                inputs[var_name] = parameter.default
-
-    return _literals.LiteralMap(literals=inputs)
-
-
 def str2bool(str):
     """
     bool('False') is True in Python, so we need to do some string parsing.  Use the same words in ConfigParser
     :param Text str:
     :rtype: bool
     """
-    return not str.lower() in ["false", "0", "off", "no"]
+    return str.lower() not in ["false", "0", "off", "no"]
 
 
+# TODO Deprecated delete after deleting flyte_cli register
 def _hydrate_identifier(
     project: str, domain: str, version: str, identifier: _identifier_pb2.Identifier
 ) -> _identifier_pb2.Identifier:
-    if not identifier.project or identifier.project == "{{ registration.project }}":
+    if not identifier.project or identifier.project == PROJECT_PLACEHOLDER:
         identifier.project = project
 
-    if not identifier.domain or identifier.domain == "{{ registration.domain }}":
+    if not identifier.domain or identifier.domain == DOMAIN_PLACEHOLDER:
         identifier.domain = domain
 
-    if not identifier.version or identifier.version == "{{ registration.version }}":
+    if not identifier.version or identifier.version == VERSION_PLACEHOLDER:
         identifier.version = version
     return identifier
 
 
+# TODO Deprecated delete after deleting flyte_cli register
 def _hydrate_node(project: str, domain: str, version: str, node: _workflow_pb2.Node) -> _workflow_pb2.Node:
     if node.HasField("task_node"):
         task_node = node.task_node
@@ -135,6 +81,7 @@ def _hydrate_node(project: str, domain: str, version: str, node: _workflow_pb2.N
     return node
 
 
+# TODO Deprecated delete after deleting flyte_cli register
 def _hydrate_workflow_template_nodes(
     project: str, domain: str, version: str, template: _workflow_pb2.WorkflowTemplate
 ) -> _workflow_pb2.WorkflowTemplate:
@@ -148,6 +95,7 @@ def _hydrate_workflow_template_nodes(
     return template
 
 
+# TODO Deprecated delete after deleting flyte_cli register
 def hydrate_registration_parameters(
     resource_type: int,
     project: str,
@@ -183,3 +131,9 @@ def hydrate_registration_parameters(
     del entity.sub_workflows[:]
     entity.sub_workflows.extend(refreshed_sub_workflows)
     return identifier, entity
+
+
+def display_help_with_error(ctx: click.Context, message: str):
+    click.echo(f"{ctx.get_help()}\n")
+    click.secho(message, fg="red")
+    sys.exit(1)
