@@ -1,17 +1,19 @@
 import typing
 from datetime import datetime, timedelta
 
-import click
+import rich_click as click
 
 from flytekit.clis.sdk_in_container.helpers import get_and_save_remote_with_click_context
-from flytekit.clis.sdk_in_container.run import DateTimeType, DurationParamType
+from flytekit.clis.sdk_in_container.utils import domain_option_dec, project_option_dec
+from flytekit.core.workflow import WorkflowFailurePolicy
+from flytekit.interaction.click_types import DateTimeType, DurationParamType
 
 _backfill_help = """
 The backfill command generates and registers a new workflow based on the input launchplan to run an
 automated backfill. The workflow can be managed using the Flyte UI and can be canceled, relaunched, and recovered.
 
-- launchplan refers to the name of the launchplan
-- launchplan_version is optional and should be a valid version for a launchplan version.
+    - ``launchplan`` refers to the name of the Launchplan
+    - ``launchplan_version`` is optional and should be a valid version for a Launchplan version.
 """
 
 
@@ -42,22 +44,8 @@ def resolve_backfill_window(
 
 
 @click.command("backfill", help=_backfill_help)
-@click.option(
-    "-p",
-    "--project",
-    required=False,
-    type=str,
-    default="flytesnacks",
-    help="Project to register and run this workflow in",
-)
-@click.option(
-    "-d",
-    "--domain",
-    required=False,
-    type=str,
-    default="development",
-    help="Domain to register and run this workflow in",
-)
+@project_option_dec
+@domain_option_dec
 @click.option(
     "-v",
     "--version",
@@ -90,8 +78,8 @@ def resolve_backfill_window(
     is_flag=True,
     default=False,
     show_default=True,
-    help="All backfill steps can be run in parallel (limited by max-parallelism), if using --parallel."
-    " Else all steps will be run sequentially [--serial].",
+    help="All backfill steps can be run in parallel (limited by max-parallelism), if using ``--parallel.``"
+    " Else all steps will be run sequentially [``--serial``].",
 )
 @click.option(
     "--execute/--do-not-execute",
@@ -125,6 +113,26 @@ def resolve_backfill_window(
     "backfills between. This is needed with from-date / to-date. Optional if both from-date and to-date are "
     "provided",
 )
+@click.option(
+    "--fail-fast/--no-fail-fast",
+    required=False,
+    type=bool,
+    is_flag=True,
+    default=True,
+    show_default=True,
+    help="If set to true, the backfill will fail immediately (WorkflowFailurePolicy.FAIL_IMMEDIATELY) if any of the "
+    "backfill steps fail. If set to false, the backfill will continue to run even if some of the backfill steps "
+    "fail (WorkflowFailurePolicy.FAIL_AFTER_EXECUTABLE_NODES_COMPLETE).",
+)
+@click.option(
+    "--overwrite-cache",
+    required=False,
+    type=bool,
+    is_flag=True,
+    default=False,
+    show_default=True,
+    help="Whether to overwrite the cache if it already exists.",
+)
 @click.argument(
     "launchplan",
     required=True,
@@ -151,6 +159,8 @@ def backfill(
     parallel: bool,
     execution_name: str,
     version: str,
+    fail_fast: bool,
+    overwrite_cache: bool,
 ):
     from_date, to_date = resolve_backfill_window(from_date, to_date, backfill_window)
     remote = get_and_save_remote_with_click_context(ctx, project, domain)
@@ -167,12 +177,17 @@ def backfill(
             dry_run=dry_run,
             execute=execute,
             parallel=parallel,
+            failure_policy=WorkflowFailurePolicy.FAIL_IMMEDIATELY
+            if fail_fast
+            else WorkflowFailurePolicy.FAIL_AFTER_EXECUTABLE_NODES_COMPLETE,
+            overwrite_cache=overwrite_cache,
         )
-        if entity:
-            console_url = remote.generate_console_url(entity)
-            if execute:
-                click.secho(f"\n Execution launched {console_url} to see execution in the console.", fg="green")
-                return
-            click.secho(f"\n Workflow registered at {console_url}", fg="green")
+        if dry_run:
+            return
+        console_url = remote.generate_console_url(entity)
+        if execute:
+            click.secho(f"\n Execution launched {console_url} to see execution in the console.", fg="green")
+            return
+        click.secho(f"\n Workflow registered at {console_url}", fg="green")
     except StopIteration as e:
         click.secho(f"{e.value}", fg="red")
