@@ -1,18 +1,23 @@
 from dataclasses import replace
 from typing import Optional
 
-import click
+import rich_click as click
 
 from flytekit.clis.sdk_in_container.constants import CTX_CONFIG_FILE
-from flytekit.configuration import Config, ImageConfig, get_config_file
-from flytekit.loggers import cli_logger
+from flytekit.configuration import ImageConfig
+from flytekit.configuration.plugin import get_plugin
+from flytekit.constants import CopyFileDetection
 from flytekit.remote.remote import FlyteRemote
 
 FLYTE_REMOTE_INSTANCE_KEY = "flyte_remote"
 
 
 def get_and_save_remote_with_click_context(
-    ctx: click.Context, project: str, domain: str, save: bool = True
+    ctx: click.Context,
+    project: str,
+    domain: str,
+    save: bool = True,
+    data_upload_location: Optional[str] = None,
 ) -> FlyteRemote:
     """
     NB: This function will by default mutate the click Context.obj dictionary, adding a remote key with value
@@ -22,19 +27,13 @@ def get_and_save_remote_with_click_context(
     :param project: default project for the remote instance
     :param domain: default domain
     :param save: If false, will not mutate the context.obj dict
+    :param data_upload_location: if specified, will set the data upload location for the remote instance
     :return: FlyteRemote instance
     """
+    if ctx.obj.get(FLYTE_REMOTE_INSTANCE_KEY) is not None:
+        return ctx.obj[FLYTE_REMOTE_INSTANCE_KEY]
     cfg_file_location = ctx.obj.get(CTX_CONFIG_FILE)
-    cfg_file = get_config_file(cfg_file_location)
-    if cfg_file is None:
-        cfg_obj = Config.for_sandbox()
-        cli_logger.info("No config files found, creating remote with sandbox config")
-    else:
-        cfg_obj = Config.auto(cfg_file_location)
-        cli_logger.info(
-            f"Creating remote with config {cfg_obj}" + (f" with file {cfg_file_location}" if cfg_file_location else "")
-        )
-    r = FlyteRemote(cfg_obj, default_project=project, default_domain=domain)
+    r = get_plugin().get_remote(cfg_file_location, project, domain, data_upload_location)
     if save:
         ctx.obj[FLYTE_REMOTE_INSTANCE_KEY] = r
     return r
@@ -63,3 +62,17 @@ def patch_image_config(config_file: Optional[str], image_config: ImageConfig) ->
             if addl.name not in additional_image_names:
                 new_additional_images.append(addl)
     return replace(image_config, default_image=new_default, images=new_additional_images)
+
+
+def parse_copy(ctx, param, value) -> Optional[CopyFileDetection]:
+    """Helper function to parse cmd line args into enum"""
+    if value == "auto":
+        copy_style = CopyFileDetection.LOADED_MODULES
+    elif value == "all":
+        copy_style = CopyFileDetection.ALL
+    elif value == "none":
+        copy_style = CopyFileDetection.NO_COPY
+    else:
+        copy_style = None
+
+    return copy_style
