@@ -1,79 +1,113 @@
 """
-=====================
-Configuration
-=====================
+# Configuration
 
-.. currentmodule:: flytekit.configuration
+## Flytekit Configuration Sources
 
-Flytekit Configuration Ecosystem
---------------------------------
+There are multiple ways to configure flytekit settings:
 
-Where can configuration come from?
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+### Command Line Arguments
+This is the recommended way of setting configuration values for many cases. For example, see `pyflyte package` command.
 
-- Command line arguments. This is the ideal location for settings to go. (See ``pyflyte package --help`` for example.)
-- Environment variables. Users can specify these at compile time, but when your task is run, Flyte Propeller will also set configuration to ensure correct interaction with the platform.
-- A config file - an INI style configuration file. By default, flytekit will look for a file in two places
-  1. First, a file named ``flytekit.config`` in the Python interpreter's starting directory
-  2. A file in ``~/.flyte/config`` in the home directory as detected by Python.
+### Python Config Object
+A `Config` object can be used directly, e.g. when initializing a `FlyteRemote` object. See the Control Plane design docs for examples on how to specify a `Config` object.
 
-How is configuration used?
-^^^^^^^^^^^^^^^^^^^^^^^^^^
+### Environment Variables
+Users can specify these at compile time, but when your task is run, Flyte Propeller will also set configuration to ensure correct interaction with the platform. The environment variables must be specified with the format `FLYTE_{SECTION}_{OPTION}`, all in upper case. For example, to specify the `PlatformConfig.endpoint` setting, the environment variable would be `FLYTE_PLATFORM_URL`.
 
-Configuration usage can roughly be bucketed into the following areas,
+> [!NOTE]
+> Environment variables won't work for image configuration, which need to be specified with the `pyflyte package --image ...` option or in a configuration file.
 
-- Compile-time settings - things like the default image, where to look for Flyte code, etc.
-- Platform settings - Where to find the Flyte backend (Admin DNS, whether to use SSL)
-- Run time (registration) settings - these are things like the K8s service account to use, a specific S3/GCS bucket to write off-loaded data (dataframes and files) to, notifications, labels & annotations, etc.
-- Data access settings - Is there a custom S3 endpoint in use? Backoff/retry behavior for accessing S3/GCS, key and password, etc.
-- Other settings - Statsd configuration, which is a run-time applicable setting but is not necessarily relevant to the Flyte platform.
+### YAML Format Configuration File
+A configuration file that contains settings for both `flytectl` and `flytekit`. This is the recommended configuration file format. Invoke the `flytectl config init` command to create a boilerplate `~/.flyte/config.yaml` file, and `flytectl --help` to learn about all of the configuration yaml options.
 
-Configuration Objects
----------------------
+Example `config.yaml` file:
 
-The following objects are encapsulated in a parent object called ``Config``.
+```YAML
+# Sample config file
 
-.. autosummary::
-   :template: custom.rst
-   :toctree: generated/
-   :nosignatures:
+admin:
+  # For GRPC endpoints you might want to use dns:///flyte.myexample.com
+  endpoint: dns:///localhost:8089
+  authType: Pkce
+  insecure: true
 
-   ~Config
+logger:
+  show-source: true
+  level: 0
 
-.. _configuration-compile-time-settings:
+console:
+  endpoint: http://localhost:8080
+  insecure: true
 
-Compilation (Serialization) Time Settings
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+# This section is used only in the control plane to trigger a remote execution
+storage:
+  type: minio
+  stow:
+    kind: s3
+    config:
+      auth_type: accesskey
+      access_key_id: minio
+      secret_key: miniostorage
+      endpoint: http://localhost:9000
+      region: us-east-1
+      disable_ssl: true
+      addressing_style: "path"
+```
 
-.. autosummary::
-   :template: custom.rst
-   :toctree: generated/
-   :nosignatures:
+### INI Format Configuration File
+A configuration file for `flytekit`. By default, `flytekit` will look for a file in two places:
 
-   ~Image
-   ~ImageConfig
-   ~SerializationSettings
-   ~FastSerializationSettings
+1. First, a file named `flytekit.config` in the Python interpreter's working directory.
+2. A file in `~/.flyte/config` in the home directory as detected by Python.
 
-.. _configuration-execution-time-settings:
+Example `flytekit.config` file:
 
-Execution Time Settings
-^^^^^^^^^^^^^^^^^^^^^^^
+```ini
+[sdk]
+workflow_packages=my_cool_workflows, other_workflows
+```
 
-.. autosummary::
-   :template: custom.rst
-   :toctree: generated/
-   :nosignatures:
+> [!WARNING]
+> The INI format configuration is considered a legacy configuration format. We recommend using the yaml format instead if you're using a configuration file.
 
-   ~PlatformConfig
-   ~StatsConfig
-   ~SecretsConfig
-   ~S3Config
-   ~GCSConfig
-   ~DataConfig
-   ~Config
+## How is configuration used?
+
+Configuration usage can roughly be bucketed into the following areas:
+
+- **Compile-time settings**: these are settings like the default image and named images, where to look for Flyte code, etc.
+- **Platform settings**: Where to find the Flyte backend (Admin DNS, whether to use SSL)
+- **Registration Run-time settings**: these are things like the K8s service account to use, a specific S3/GCS bucket to write off-loaded data (dataframes and files) to, notifications, labels & annotations, etc.
+- **Data access settings**: Is there a custom S3 endpoint in use? Backoff/retry behavior for accessing S3/GCS, key and password, etc.
+- **Other settings** - Statsd configuration, which is a run-time applicable setting but is not necessarily relevant to the Flyte platform.
+
+## Configuration Objects
+
+The following objects are encapsulated in a parent object called `Config`:
+
+### Serialization Time Settings
+
+These are serialization/compile-time settings that are used when using commands like `pyflyte package` or `pyflyte register`. These configuration settings are typically passed in as flags to the above CLI commands.
+
+The image configurations are typically either passed in via an `--image` flag, or can be specified in the `yaml` or `ini` configuration files (see examples above).
+
+- **Image**: Represents a container image with optional configuration overrides.
+- **ImageConfig**: Represents an image configuration for a given project/domain combination.
+- **SerializationSettings**: Controls how to serialize Flyte entities when registering with Admin.
+- **FastSerializationSettings**: Configuration for faster serialization settings.
+
+### Execution Time Settings
+
+Users typically shouldn't be concerned with these configurations, as they are typically set by FlytePropeller or FlyteAdmin. The configurations below are useful for authenticating to a Flyte backend, configuring data access credentials, secrets, and statsd metrics.
+
+- **PlatformConfig**: Configuration for how to connect to the Flyte platform.
+- **StatsConfig**: Configuration for how to emit statsd metrics.
+- **SecretsConfig**: Configuration for how to access secrets.
+- **S3Config**: Amazon S3 specific configuration.
+- **GCSConfig**: Google Cloud Storage specific configuration.
+- **DataConfig**: Configuration for data access.
 
 """
+
 from __future__ import annotations
 
 import base64
@@ -89,12 +123,15 @@ from dataclasses import dataclass, field
 from io import BytesIO
 from typing import Dict, List, Optional
 
-from dataclasses_json import dataclass_json
-from docker_image import reference
+import yaml
+from dataclasses_json import DataClassJsonMixin
 
 from flytekit.configuration import internal as _internal
 from flytekit.configuration.default_images import DefaultImages
 from flytekit.configuration.file import ConfigEntry, ConfigFile, get_config_file, read_file_if_exists, set_if_exists
+from flytekit.core.resources import ResourceSpec
+from flytekit.image_spec import ImageSpec
+from flytekit.image_spec.image_spec import ImageBuildEngine
 from flytekit.loggers import logger
 
 PROJECT_PLACEHOLDER = "{{ registration.project }}"
@@ -108,9 +145,8 @@ _IMAGE_FQN_TAG_REGEX = re.compile(r"([^:]+)(?=:.+)?")
 SERIALIZED_CONTEXT_ENV_VAR = "_F_SS_C"
 
 
-@dataclass_json
 @dataclass(init=True, repr=True, eq=True, frozen=True)
-class Image(object):
+class Image(DataClassJsonMixin):
     """
     Image is a structured wrapper for task container images used in object serialization.
 
@@ -122,51 +158,94 @@ class Image(object):
             #. a repository name
             For example: `hostname/username/reponame`
         tag (str): Optional tag used to specify which version of an image to pull
+        digest (str): Optional digest used to specify which version of an image to pull
     """
 
     name: str
     fqn: str
-    tag: str
+    tag: Optional[str] = None
+    digest: Optional[str] = None
+
+    def __post_init__(self):
+        if not ((self.tag is None) or (self.digest is None)):
+            raise ValueError(f"Cannot specify both tag and digest. Got tag={self.tag} and digest={self.digest}")
 
     @property
     def full(self) -> str:
         """ "
-        Return the full image name with tag.
+        Return the full image name with tag or digest, whichever is available.
+
+        When using a tag the separator is `:` and when using a digest the separator is `@`.
         """
-        return f"{self.fqn}:{self.tag}"
+        return f"{self.fqn}@{self.digest}" if self.digest else f"{self.fqn}:{self.tag}"
+
+    @property
+    def version(self) -> Optional[str]:
+        """
+        Return the version of the image. This could be the tag or digest, whichever is available.
+        """
+        return self.digest or self.tag
 
     @staticmethod
-    def look_up_image_info(name: str, tag: str, optional_tag: bool = False) -> Image:
+    def look_up_image_info(name: str, image_identifier: str, allow_no_tag_or_digest: bool = False) -> Image:
         """
-        Looks up the image tag from environment variable (should be set from the Dockerfile).
-            FLYTE_INTERNAL_IMAGE should be the environment variable.
+        Creates an `Image` object from an image identifier string or a path to an ImageSpec yaml file.
 
-        This function is used when registering tasks/workflows with Admin.
-        When using the canonical Python-based development cycle, the version that is used to register workflows
-        and tasks with Admin should be the version of the image itself, which should ideally be something unique
-        like the sha of the latest commit.
+        This function is used when registering tasks/workflows with Admin. When using
+        the canonical Python-based development cycle, the version that is used to
+        register workflows and tasks with Admin should be the version of the image
+        itself, which should ideally be something unique like the git revision SHA1 of
+        the latest commit.
 
-        :param optional_tag:
         :param name:
-        :param Text tag: e.g. somedocker.com/myimage:someversion123
-        :rtype: Text
+        :param image_identifier: Either the full image identifier string e.g. somedocker.com/myimage:someversion123
+            or a path to a file containing a `ImageSpec`.
+        :param allow_no_tag_or_digest:
+        :rtype: Image
         """
-        ref = reference.Reference.parse(tag)
-        if not optional_tag and ref["tag"] is None:
-            raise AssertionError(f"Incorrectly formatted image {tag}, missing tag value")
-        else:
-            return Image(name=name, fqn=ref["name"], tag=ref["tag"])
+        if pathlib.Path(image_identifier).is_file():
+            with open(image_identifier, "r") as f:
+                image_spec_dict = yaml.safe_load(f)
+                image_spec = ImageSpec(**image_spec_dict)
+                ImageBuildEngine.build(image_spec)
+                image_identifier = image_spec.image_name()
+
+        fqn, tag, digest = _parse_image_identifier(image_identifier)
+
+        if not allow_no_tag_or_digest and tag is None and digest is None:
+            raise AssertionError(f"Incorrectly formatted image {image_identifier}, missing tag or digest")
+        return Image(name=name, fqn=fqn, tag=tag, digest=digest)
 
 
-@dataclass_json
-@dataclass(init=True, repr=True, eq=True, frozen=True)
-class ImageConfig(object):
+def _parse_image_identifier(image_identifier: str) -> typing.Tuple[str, Optional[str], Optional[str]]:
     """
+    Largely copied from `docker.utils.parse_repository_tag`, but returns tags and digests separately.
+    Returns:
+        Tuple[str, str, str]: fully_qualified_name, tag, digest
+    """
+    parts = image_identifier.rsplit("@", 1)
+    if len(parts) == 2:
+        # The image identifier used a digest e.g. `xyz.com/abc@sha256:26c68657ccce2cb0a31b330cb0be2b5e108d467f641c62e13ab40cbec258c68d`
+        return parts[0], None, parts[1]
+    parts = image_identifier.rsplit(":", 1)
+    if len(parts) == 2 and "/" not in parts[1]:
+        # The image identifier used a tag e.g. `xyz.com/abc:latest`
+        return parts[0], parts[1], None
+    # The image identifier had no tag or digest e.g. `xyz.com/abc`
+    return image_identifier, None, None
+
+
+@dataclass(init=True, repr=True, eq=True, frozen=True)
+class ImageConfig(DataClassJsonMixin):
+    """
+    We recommend you to use ImageConfig.auto(img_name=None) to create an ImageConfig.
+    For example, ImageConfig.auto(img_name=""ghcr.io/flyteorg/flytecookbook:v1.0.0"") will create an ImageConfig.
+
     ImageConfig holds available images which can be used at registration time. A default image can be specified
     along with optional additional images. Each image in the config must have a unique name.
 
     Attributes:
-        default_image (str): The default image to be used as a container for task serialization.
+        default_image (Optional[Image]): The default image to be used as a container for task serialization.
         images (List[Image]): Optional, additional images which can be used in task container definitions.
     """
 
@@ -190,10 +269,9 @@ class ImageConfig(object):
     def validate_image(_: typing.Any, param: str, values: tuple) -> ImageConfig:
         """
         Validates the image to match the standard format. Also validates that only one default image
-        is provided. a default image, is one that is specified as
-          default=img or just img. All other images should be provided with a name, in the format
-          name=img
-        This method can be used with the CLI
+        is provided. a default image, is one that is specified as ``default=<image_uri>`` or just ``<image_uri>``. All
+        other images should be provided with a name, in the format ``name=<image_uri>`` This method can be used with the
+        CLI
 
         :param _: click argument, ignored here.
         :param param: the click argument, here should be "image"
@@ -205,7 +283,7 @@ class ImageConfig(object):
         for v in values:
             if "=" in v:
                 splits = v.split("=", maxsplit=1)
-                img = Image.look_up_image_info(name=splits[0], tag=splits[1], optional_tag=False)
+                img = Image.look_up_image_info(name=splits[0], image_identifier=splits[1], allow_no_tag_or_digest=False)
             else:
                 img = Image.look_up_image_info(DEFAULT_IMAGE_NAME, v, False)
 
@@ -218,6 +296,9 @@ class ImageConfig(object):
             else:
                 images.append(img)
 
+        if default_image is None:
+            default_image_str = DefaultImages.default_image()
+            default_image = Image.look_up_image_info(DEFAULT_IMAGE_NAME, default_image_str, False)
         return ImageConfig.create_from(default_image=default_image, other_images=images)
 
     @classmethod
@@ -248,7 +329,7 @@ class ImageConfig(object):
 
         config_file = get_config_file(config_file)
         other_images = [
-            Image.look_up_image_info(k, tag=v, optional_tag=True)
+            Image.look_up_image_info(k, image_identifier=v, allow_no_tag_or_digest=True)
             for k, v in _internal.Images.get_specified_images(config_file).items()
         ]
         return cls.create_from(default_img, other_images)
@@ -259,20 +340,23 @@ class ImageConfig(object):
             Allows you to programmatically create an ImageConfig. Usually only the default_image is required, unless
             your workflow uses multiple images
 
-            .. code:: python
-
-              ImageConfig.from_dict(
-                  "ghcr.io/flyteorg/flytecookbook:v1.0.0",
-                   {
-                        "spark": "ghcr.io/flyteorg/myspark:...",
-                        "other": "...",
-              })
+            ```python
+            ImageConfig.from_dict(
+                "ghcr.io/flyteorg/flytecookbook:v1.0.0",
+                {
+                    "spark": "ghcr.io/flyteorg/myspark:...",
+                    "other": "...",
+                }
+            )
+            ```
 
         :return:
         """
         m = m or {}
         def_img = Image.look_up_image_info("default", default_image) if default_image else None
-        other_images = [Image.look_up_image_info(k, tag=v, optional_tag=True) for k, v in m.items()]
+        other_images = [
+            Image.look_up_image_info(k, image_identifier=v, allow_no_tag_or_digest=True) for k, v in m.items()
+        ]
         return cls.create_from(def_img, other_images)
 
     @classmethod
@@ -290,6 +374,7 @@ class AuthType(enum.Enum):
     CLIENTSECRET = "ClientSecret"
     PKCE = "Pkce"
     EXTERNALCOMMAND = "ExternalCommand"
+    DEVICEFLOW = "DeviceFlow"
 
 
 @dataclass(init=True, repr=True, eq=True, frozen=True)
@@ -299,27 +384,35 @@ class PlatformConfig(object):
 
     :param endpoint: DNS for Flyte backend
     :param insecure: Whether or not to use SSL
-    :param insecure_skip_verify: Wether to skip SSL certificate verification
-    :param console_endpoint: endpoint for console if different than Flyte backend
-    :param command: This command is executed to return a token using an external process.
+    :param insecure_skip_verify: Whether to skip SSL certificate verification
+    :param console_endpoint: endpoint for console if different from Flyte backend
+    :param command: This command is executed to return a token using an external process
+    :param proxy_command: This command is executed to return a token for proxy authorization using an external process
     :param client_id: This is the public identifier for the app which handles authorization for a Flyte deployment.
       More details here: https://www.oauth.com/oauth2-servers/client-registration/client-id-secret/.
     :param client_credentials_secret: Used for service auth, which is automatically called during pyflyte. This will
       allow the Flyte engine to read the password directly from the environment variable. Note that this is
-      less secure! Please only use this if mounting the secret as a file is impossible.
-    :param scopes: List of scopes to request. This is only applicable to the client credentials flow.
-    :param auth_mode: The OAuth mode to use. Defaults to pkce flow.
+      less secure! Please only use this if mounting the secret as a file is impossible
+    :param scopes: List of scopes to request. This is only applicable to the client credentials flow
+    :param auth_mode: The OAuth mode to use. Defaults to pkce flow
+    :param ca_cert_file_path: [optional] str Root Cert to be loaded and used to verify admin
+    :param http_proxy_url: [optional] HTTP Proxy to be used for OAuth requests
     """
 
     endpoint: str = "localhost:30080"
     insecure: bool = False
     insecure_skip_verify: bool = False
+    ca_cert_file_path: typing.Optional[str] = None
     console_endpoint: typing.Optional[str] = None
     command: typing.Optional[typing.List[str]] = None
+    proxy_command: typing.Optional[typing.List[str]] = None
     client_id: typing.Optional[str] = None
     client_credentials_secret: typing.Optional[str] = None
     scopes: List[str] = field(default_factory=list)
     auth_mode: AuthType = AuthType.STANDARD
+    audience: typing.Optional[str] = None
+    rpc_retries: int = 3
+    http_proxy_url: typing.Optional[str] = None
 
     @classmethod
     def auto(cls, config_file: typing.Optional[typing.Union[str, ConfigFile]] = None) -> PlatformConfig:
@@ -334,27 +427,43 @@ class PlatformConfig(object):
         kwargs = set_if_exists(
             kwargs, "insecure_skip_verify", _internal.Platform.INSECURE_SKIP_VERIFY.read(config_file)
         )
+        kwargs = set_if_exists(kwargs, "ca_cert_file_path", _internal.Platform.CA_CERT_FILE_PATH.read(config_file))
         kwargs = set_if_exists(kwargs, "command", _internal.Credentials.COMMAND.read(config_file))
+        kwargs = set_if_exists(kwargs, "proxy_command", _internal.Credentials.PROXY_COMMAND.read(config_file))
         kwargs = set_if_exists(kwargs, "client_id", _internal.Credentials.CLIENT_ID.read(config_file))
         kwargs = set_if_exists(
             kwargs, "client_credentials_secret", _internal.Credentials.CLIENT_CREDENTIALS_SECRET.read(config_file)
         )
 
+        is_client_secret = False
         client_credentials_secret = read_file_if_exists(
             _internal.Credentials.CLIENT_CREDENTIALS_SECRET_LOCATION.read(config_file)
         )
-        if client_credentials_secret and client_credentials_secret.endswith("\n"):
-            logger.info("Newline stripped from client secret")
-            client_credentials_secret = client_credentials_secret.strip()
+        if client_credentials_secret:
+            is_client_secret = True
+            if client_credentials_secret.endswith("\n"):
+                logger.info("Newline stripped from client secret")
+                client_credentials_secret = client_credentials_secret.strip()
         kwargs = set_if_exists(
             kwargs,
             "client_credentials_secret",
             client_credentials_secret,
         )
+
+        client_credentials_secret_env_var = _internal.Credentials.CLIENT_CREDENTIALS_SECRET_ENV_VAR.read(config_file)
+        if client_credentials_secret_env_var:
+            client_credentials_secret = os.getenv(client_credentials_secret_env_var)
+            if client_credentials_secret:
+                is_client_secret = True
+        kwargs = set_if_exists(kwargs, "client_credentials_secret", client_credentials_secret)
         kwargs = set_if_exists(kwargs, "scopes", _internal.Credentials.SCOPES.read(config_file))
         kwargs = set_if_exists(kwargs, "auth_mode", _internal.Credentials.AUTH_MODE.read(config_file))
+        if is_client_secret:
+            kwargs = set_if_exists(kwargs, "auth_mode", AuthType.CLIENTSECRET.value)
         kwargs = set_if_exists(kwargs, "endpoint", _internal.Platform.URL.read(config_file))
         kwargs = set_if_exists(kwargs, "console_endpoint", _internal.Platform.CONSOLE_ENDPOINT.read(config_file))
+
+        kwargs = set_if_exists(kwargs, "http_proxy_url", _internal.Platform.HTTP_PROXY_URL.read(config_file))
         return PlatformConfig(**kwargs)
 
     @classmethod
@@ -423,7 +532,7 @@ class SecretsConfig(object):
         return SecretsConfig(**kwargs)
 
 
-@dataclass
+@dataclass(init=True, repr=True, eq=True, frozen=True)
 class S3Config(object):
     """
     S3 specific configuration
@@ -454,7 +563,7 @@ class S3Config(object):
         return S3Config(**kwargs)
 
 
-@dataclass
+@dataclass(init=True, repr=True, eq=True, frozen=True)
 class GCSConfig(object):
     """
     Any GCS specific configuration.
@@ -470,6 +579,48 @@ class GCSConfig(object):
         return GCSConfig(**kwargs)
 
 
+@dataclass(init=True, repr=True, eq=True, frozen=True)
+class GenericPersistenceConfig(object):
+    """
+    Data storage configuration that applies across any provider.
+    """
+
+    attach_execution_metadata: bool = True
+
+    @classmethod
+    def auto(cls, config_file: typing.Union[str, ConfigFile] = None) -> GCSConfig:
+        config_file = get_config_file(config_file)
+        kwargs = {}
+        kwargs = set_if_exists(
+            kwargs, "attach_execution_metadata", _internal.Persistence.ATTACH_EXECUTION_METADATA.read(config_file)
+        )
+        return GenericPersistenceConfig(**kwargs)
+
+
+@dataclass(init=True, repr=True, eq=True, frozen=True)
+class AzureBlobStorageConfig(object):
+    """
+    Any Azure Blob Storage specific configuration.
+    """
+
+    account_name: typing.Optional[str] = None
+    account_key: typing.Optional[str] = None
+    tenant_id: typing.Optional[str] = None
+    client_id: typing.Optional[str] = None
+    client_secret: typing.Optional[str] = None
+
+    @classmethod
+    def auto(cls, config_file: typing.Union[str, ConfigFile] = None) -> GCSConfig:
+        config_file = get_config_file(config_file)
+        kwargs = {}
+        kwargs = set_if_exists(kwargs, "account_name", _internal.AZURE.STORAGE_ACCOUNT_NAME.read(config_file))
+        kwargs = set_if_exists(kwargs, "account_key", _internal.AZURE.STORAGE_ACCOUNT_KEY.read(config_file))
+        kwargs = set_if_exists(kwargs, "tenant_id", _internal.AZURE.TENANT_ID.read(config_file))
+        kwargs = set_if_exists(kwargs, "client_id", _internal.AZURE.CLIENT_ID.read(config_file))
+        kwargs = set_if_exists(kwargs, "client_secret", _internal.AZURE.CLIENT_SECRET.read(config_file))
+        return AzureBlobStorageConfig(**kwargs)
+
+
 @dataclass(init=True, repr=True, eq=True, frozen=True)
 class DataConfig(object):
     """
@@ -480,30 +631,74 @@ class DataConfig(object):
 
     s3: S3Config = S3Config()
     gcs: GCSConfig = GCSConfig()
+    azure: AzureBlobStorageConfig = AzureBlobStorageConfig()
+    generic: GenericPersistenceConfig = GenericPersistenceConfig()
 
     @classmethod
     def auto(cls, config_file: typing.Union[str, ConfigFile] = None) -> DataConfig:
         config_file = get_config_file(config_file)
         return DataConfig(
+            azure=AzureBlobStorageConfig.auto(config_file),
             s3=S3Config.auto(config_file),
             gcs=GCSConfig.auto(config_file),
+            generic=GenericPersistenceConfig.auto(config_file),
         )
 
 
+@dataclass(init=True, repr=True, eq=True, frozen=True)
+class LocalConfig(object):
+    """
+    Any configuration specific to local runs.
+    """
+
+    cache_enabled: bool = True
+    cache_overwrite: bool = False
+
+    @classmethod
+    def auto(cls, config_file: typing.Union[str, ConfigFile] = None) -> LocalConfig:
+        config_file = get_config_file(config_file)
+        kwargs = {}
+        kwargs = set_if_exists(kwargs, "cache_enabled", _internal.Local.CACHE_ENABLED.read(config_file))
+        kwargs = set_if_exists(kwargs, "cache_overwrite", _internal.Local.CACHE_OVERWRITE.read(config_file))
+        return LocalConfig(**kwargs)
+
+
+@dataclass(init=True, repr=True, eq=True, frozen=True)
+class TaskConfig(object):
+    """
+    Any Project/Domain/Org configuration.
+    """
+
+    project: Optional[str] = field(default="flytesnacks")
+    domain: Optional[str] = field(default="development")
+    org: Optional[str] = field(default="None")
+
+    @classmethod
+    def auto(cls, config_file: typing.Union[str, ConfigFile] = None) -> TaskConfig:
+        config_file = get_config_file(config_file)
+        kwargs = {}
+        kwargs = set_if_exists(kwargs, "project", _internal.Local.USER_PROJECT.read(config_file))
+        kwargs = set_if_exists(kwargs, "domain", _internal.Local.USER_DOMAIN.read(config_file))
+        kwargs = set_if_exists(kwargs, "org", _internal.Local.USER_ORG.read(config_file))
+        return TaskConfig(**kwargs)
+
+
 @dataclass(init=True, repr=True, eq=True, frozen=True)
 class Config(object):
     """
-    This is the parent configuration object and holds all the underlying configuration object types. An instance of
+    This the parent configuration object and holds all the underlying configuration object types. An instance of
     this object holds all the config necessary to
 
     1. Interactive session with Flyte backend
     2. Some parts are required for Serialization, for example Platform Config is not required
     3. Runtime of a task
 
-    Args:
-        entrypoint_settings: EntrypointSettings object for use with Spark tasks. If supplied, this will be
-          used when serializing Spark tasks, which need to know the path to the flytekit entrypoint.py file,
-          inside the container.
+    Attributes:
+        platform (PlatformConfig): Settings to connect to a Flyte backend.
+        secrets (SecretsConfig): Configuration for secrets management.
+        stats (StatsConfig): Configuration for statsd metrics.
+        data_config (DataConfig): Data storage configuration.
+        local_sandbox_path (str): Path for local sandbox runs.
     """
 
     platform: PlatformConfig = PlatformConfig()
@@ -553,7 +748,7 @@ class Config(object):
     @classmethod
     def for_sandbox(cls) -> Config:
         """
-        Constructs a new Config object specifically to connect to :std:ref:`deploy-sandbox-local`.
+        Constructs a new Config object specifically to connect to :std:ref:`deployment-deployment-sandbox`.
         If you are using a hosted Sandbox like environment, then you may need to use port-forward or ingress urls
         :return: Config
         """
@@ -586,12 +781,18 @@ class Config(object):
         :return: Config
         """
         c = cls.auto(config_file)
-        return c.with_params(platform=PlatformConfig.for_endpoint(endpoint, insecure), data_config=data_config)
+        from dataclasses import replace
+
+        p = (
+            replace(c.platform, endpoint=endpoint, insecure=insecure)
+            if config_file and c.platform
+            else PlatformConfig.for_endpoint(endpoint, insecure)
+        )
+        return c.with_params(platform=p, data_config=data_config)
 
 
-@dataclass_json
 @dataclass
-class EntrypointSettings(object):
+class EntrypointSettings(DataClassJsonMixin):
     """
     This object carries information about the path of the entrypoint command that will be invoked at runtime.
     This is where `pyflyte-execute` code can be found. This is useful for cases like pyspark execution.
@@ -600,9 +801,8 @@ class EntrypointSettings(object):
     path: Optional[str] = None
 
 
-@dataclass_json
 @dataclass
-class FastSerializationSettings(object):
+class FastSerializationSettings(DataClassJsonMixin):
     """
     This object hold information about settings necessary to serialize an object so that it can be fast-registered.
     """
@@ -615,21 +815,21 @@ class FastSerializationSettings(object):
     distribution_location: Optional[str] = None
 
 
-@dataclass_json
-@dataclass()
-class SerializationSettings(object):
+# TODO: ImageConfig, python_interpreter, venv_root, fast_serialization_settings.destination_dir should be combined.
+@dataclass
+class SerializationSettings(DataClassJsonMixin):
     """
     These settings are provided while serializing a workflow and task, before registration. This is required to get
     runtime information at serialization time, as well as some defaults.
 
-    TODO: ImageConfig, python_interpreter, venv_root, fast_serialization_settings.destination_dir should be combined.
-
     Attributes:
         project (str): The project (if any) with which to register entities under.
         domain (str): The domain (if any) with which to register entities under.
         version (str): The version (if any) with which to register entities under.
         image_config (ImageConfig): The image config used to define task container images.
         env (Optional[Dict[str, str]]): Environment variables injected into task container definitions.
+        default_resources (Optional[ResourceSpec]): The resources to request for the task - this is useful
+            if users need to override the default resource spec of an entity at registration time.
         flytekit_virtualenv_root (Optional[str]):  During out of container serialize the absolute path of the flytekit
             virtualenv at serialization time won't match the in-container value at execution time. This optional value
             is used to provide the in-container virtualenv path
@@ -640,6 +840,7 @@ class SerializationSettings(object):
         fast_serialization_settings (Optional[FastSerializationSettings]): If the code is being serialized so that it
             can be fast registered (and thus omit building a Docker image) this object contains additional parameters
             for serialization.
+        source_root (Optional[str]): The root directory of the source code.
     """
 
     image_config: ImageConfig
@@ -647,10 +848,12 @@ class SerializationSettings(object):
     domain: typing.Optional[str] = None
     version: typing.Optional[str] = None
     env: Optional[Dict[str, str]] = None
+    default_resources: Optional[ResourceSpec] = None
     git_repo: Optional[str] = None
     python_interpreter: str = DEFAULT_RUNTIME_PYTHON_INTERPRETER
     flytekit_virtualenv_root: Optional[str] = None
     fast_serialization_settings: Optional[FastSerializationSettings] = None
+    source_root: Optional[str] = None
 
     def __post_init__(self):
         if self.flytekit_virtualenv_root is None:
@@ -680,7 +883,7 @@ class SerializationSettings(object):
         domain: str = "",
         python_interpreter_path: str = DEFAULT_RUNTIME_PYTHON_INTERPRETER,
     ) -> SerializationSettings:
-        img = ImageConfig(default_image=Image.look_up_image_info(DEFAULT_IMAGE_NAME, tag=image))
+        img = ImageConfig(default_image=Image.look_up_image_info(DEFAULT_IMAGE_NAME, image_identifier=image))
         return SerializationSettings(
             image_config=img,
             project=project,
@@ -720,10 +923,12 @@ class SerializationSettings(object):
             version=self.version,
             image_config=self.image_config,
             env=self.env.copy() if self.env else None,
+            default_resources=self.default_resources,
             git_repo=self.git_repo,
             flytekit_virtualenv_root=self.flytekit_virtualenv_root,
             python_interpreter=self.python_interpreter,
             fast_serialization_settings=self.fast_serialization_settings,
+            source_root=self.source_root,
         )
 
     def should_fast_serialize(self) -> bool:
@@ -752,7 +957,7 @@ class SerializationSettings(object):
         """
         Use this method to create a new SerializationSettings that has an environment variable set with the SerializedContext
         This is useful in transporting SerializedContext to serialized and registered tasks.
-        The setting will be availabe in the `env` field with the key `SERIALIZED_CONTEXT_ENV_VAR`
+        The setting will be available in the `env` field with the key `SERIALIZED_CONTEXT_ENV_VAR`
         :return: A newly constructed SerializationSettings, or self, if it already has the serializationSettings
         """
         if self._has_serialized_context():
@@ -770,10 +975,12 @@ class SerializationSettings(object):
         version: str
         image_config: ImageConfig
         env: Optional[Dict[str, str]] = None
+        default_resources: Optional[ResourceSpec] = None
         git_repo: Optional[str] = None
         flytekit_virtualenv_root: Optional[str] = None
         python_interpreter: Optional[str] = None
         fast_serialization_settings: Optional[FastSerializationSettings] = None
+        source_root: Optional[str] = None
 
         def with_fast_serialization_settings(self, fss: fast_serialization_settings) -> SerializationSettings.Builder:
             self.fast_serialization_settings = fss
@@ -786,8 +993,10 @@ class SerializationSettings(object):
                 version=self.version,
                 image_config=self.image_config,
                 env=self.env,
+                default_resources=self.default_resources,
                 git_repo=self.git_repo,
                 flytekit_virtualenv_root=self.flytekit_virtualenv_root,
                 python_interpreter=self.python_interpreter,
                 fast_serialization_settings=self.fast_serialization_settings,
+                source_root=self.source_root,
             )
