@@ -35,6 +35,11 @@ class Images(object):
             return cfg.yaml_config.get("images", images)
 
 
+class Persistence(object):
+    SECTION = "persistence"
+    ATTACH_EXECUTION_METADATA = ConfigEntry(LegacyConfigEntry(SECTION, "attach_execution_metadata", bool))
+
+
 class AWS(object):
     SECTION = "aws"
     S3_ENDPOINT = ConfigEntry(LegacyConfigEntry(SECTION, "endpoint"), YamlConfigEntry("storage.connection.endpoint"))
@@ -57,6 +62,42 @@ class GCP(object):
     GSUTIL_PARALLELISM = ConfigEntry(LegacyConfigEntry(SECTION, "gsutil_parallelism", bool))
 
 
+class AZURE(object):
+    SECTION = "azure"
+    STORAGE_ACCOUNT_NAME = ConfigEntry(LegacyConfigEntry(SECTION, "storage_account_name"))
+    STORAGE_ACCOUNT_KEY = ConfigEntry(LegacyConfigEntry(SECTION, "storage_account_key"))
+    TENANT_ID = ConfigEntry(LegacyConfigEntry(SECTION, "tenant_id"))
+    CLIENT_ID = ConfigEntry(LegacyConfigEntry(SECTION, "client_id"))
+    CLIENT_SECRET = ConfigEntry(LegacyConfigEntry(SECTION, "client_secret"))
+
+
+class Local(object):
+    SECTION = "local"
+    CACHE_ENABLED = ConfigEntry(LegacyConfigEntry(SECTION, "cache_enabled", bool))
+    CACHE_OVERWRITE = ConfigEntry(LegacyConfigEntry(SECTION, "cache_overwrite", bool))
+    USER_PROJECT = ConfigEntry(
+        LegacyConfigEntry(
+            SECTION,
+            "project",
+        ),
+        YamlConfigEntry("task.project"),
+    )
+    USER_DOMAIN = ConfigEntry(
+        LegacyConfigEntry(
+            SECTION,
+            "domain",
+        ),
+        YamlConfigEntry("task.domain"),
+    )
+    USER_ORG = ConfigEntry(
+        LegacyConfigEntry(
+            SECTION,
+            "org",
+        ),
+        YamlConfigEntry("task.org"),
+    )
+
+
 class Credentials(object):
     SECTION = "credentials"
     COMMAND = ConfigEntry(LegacyConfigEntry(SECTION, "command", list), YamlConfigEntry("admin.command", list))
@@ -64,6 +105,13 @@ class Credentials(object):
     This command is executed to return a token using an external process.
     """
 
+    PROXY_COMMAND = ConfigEntry(
+        LegacyConfigEntry(SECTION, "proxy_command", list), YamlConfigEntry("admin.proxyCommand", list)
+    )
+    """
+    This command is executed to return a token for authorization with a proxy in front of Flyte using an external process.
+    """
+
     CLIENT_ID = ConfigEntry(LegacyConfigEntry(SECTION, "client_id"), YamlConfigEntry("admin.clientId"))
     """
     This is the public identifier for the app which handles authorization for a Flyte deployment.
@@ -85,13 +133,25 @@ class Credentials(object):
     password from a mounted file.
     """
 
+    CLIENT_CREDENTIALS_SECRET_ENV_VAR = ConfigEntry(
+        LegacyConfigEntry(SECTION, "client_secret_env_var"), YamlConfigEntry("admin.clientSecretEnvVar")
+    )
+    """
+    Used for basic auth, which is automatically called during pyflyte. This will allow the Flyte engine to read the
+    password from a mounted environment variable.
+    """
+
     SCOPES = ConfigEntry(LegacyConfigEntry(SECTION, "scopes", list), YamlConfigEntry("admin.scopes", list))
+    """
+    This setting can be used to manually pass in scopes into authenticator flows - eg.) for Auth0 compatibility
+    """
 
     AUTH_MODE = ConfigEntry(LegacyConfigEntry(SECTION, "auth_mode"), YamlConfigEntry("admin.authType"))
     """
     The auth mode defines the behavior used to request and refresh credentials. The currently supported modes include:
     - 'standard' or 'Pkce': This uses the pkce-enhanced authorization code flow by opening a browser window to initiate
             credentials access.
+    - "DeviceFlow": This uses the Device Authorization Flow
     - 'basic', 'client_credentials' or 'clientSecret': This uses symmetric key auth in which the end user enters a
             client id and a client secret and public key encryption is used to facilitate authentication.
     - None: No auth will be attempted.
@@ -108,6 +168,10 @@ class Platform(object):
         LegacyConfigEntry(SECTION, "insecure_skip_verify", bool), YamlConfigEntry("admin.insecureSkipVerify", bool)
     )
     CONSOLE_ENDPOINT = ConfigEntry(LegacyConfigEntry(SECTION, "console_endpoint"), YamlConfigEntry("console.endpoint"))
+    CA_CERT_FILE_PATH = ConfigEntry(
+        LegacyConfigEntry(SECTION, "ca_cert_file_path"), YamlConfigEntry("admin.caCertFilePath")
+    )
+    HTTP_PROXY_URL = ConfigEntry(LegacyConfigEntry(SECTION, "http_proxy_url"), YamlConfigEntry("admin.httpProxyURL"))
 
 
 class LocalSDK(object):
