@@ -1,5 +1,5 @@
 import re
-from typing import Any, Dict, Optional, Type, TypeVar
+from typing import Any, Dict, Optional, Tuple, Type, TypeVar
 
 from flytekit.core.base_task import PythonTask, TaskMetadata
 from flytekit.core.interface import Interface
@@ -9,8 +9,8 @@ T = TypeVar("T")
 
 class SQLTask(PythonTask[T]):
     """
-    Base task types for all SQL tasks. See :py:class:`flytekit.extras.sqlite3.task.SQLite3Task`
-    and :py:class:`flytekitplugins.athena.task.AthenaTask` for examples of how to use it as a base class.
+    Base task types for all SQL tasks. See `flytekit.extras.sqlite3.task.SQLite3Task`
+    and `flytekitplugins.athena.task.AthenaTask` for examples of how to use it as a base class.
 
     .. autoclass:: flytekit.extras.sqlite3.task.SQLite3Task
        :noindex:
@@ -22,16 +22,16 @@ class SQLTask(PythonTask[T]):
         self,
         name: str,
         query_template: str,
-        task_type="sql_task",
-        inputs: Optional[Dict[str, Type]] = None,
-        metadata: Optional[TaskMetadata] = None,
         task_config: Optional[T] = None,
-        outputs: Dict[str, Type] = None,
+        task_type="sql_task",
+        inputs: Optional[Dict[str, Tuple[Type, Any]]] = None,
+        metadata: Optional[TaskMetadata] = None,
+        outputs: Optional[Dict[str, Type]] = None,
         **kwargs,
     ):
         """
         This SQLTask should mostly just be used as a base class for other SQL task types and should not be used
-        directly. See :py:class:`flytekit.extras.sqlite3.task.SQLite3Task`
+        directly. See `flytekit.extras.sqlite3.task.SQLite3Task`
         """
         super().__init__(
             task_type=task_type,
@@ -48,7 +48,7 @@ class SQLTask(PythonTask[T]):
         return self._query_template
 
     def execute(self, **kwargs) -> Any:
-        raise Exception("Cannot run a SQL Task natively, please mock.")
+        raise NotImplementedError("Cannot run a SQL Task natively, please mock.")
 
     def get_query(self, **kwargs) -> str:
         return self.interpolate_query(self.query_template, **kwargs)
