@@ -1,30 +1,64 @@
 """
-==============================
-:mod:`flytekit.core.base_task`
-==============================
+# flytekit.core.base_task
 
-.. currentmodule:: flytekit.core.base_task
+This module provides the core task-related functionality in Flytekit.
 
-.. autosummary::
-   :toctree: generated/
+## Core Components
 
-   kwtypes
-   PythonTask
-   Task
-   TaskMetadata
-   TaskResolverMixin
-   IgnoreOutputs
+### kwtypes
+Utility for creating keyword type annotations for tasks.
+
+### PythonTask
+Base class for Python-based task implementations.
+
+### Task
+The base class for all Flyte tasks.
+
+### TaskResolverMixin
+Mixin class that helps resolve a task implementation.
+
+### IgnoreOutputs
+Exception that can be raised to ignore task outputs.
 
 """
 
+import asyncio
 import collections
 import datetime
+import warnings
 from abc import abstractmethod
+from base64 import b64encode
 from dataclasses import dataclass
-from typing import Any, Dict, Generic, List, Optional, OrderedDict, Tuple, Type, TypeVar, Union
+from typing import (
+    Any,
+    Coroutine,
+    Dict,
+    Generic,
+    List,
+    Optional,
+    OrderedDict,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
-from flytekit.configuration import SerializationSettings
-from flytekit.core.context_manager import ExecutionParameters, FlyteContext, FlyteContextManager, FlyteEntities
+from flyteidl.core import artifact_id_pb2 as art_id
+from flyteidl.core import tasks_pb2
+
+from flytekit.configuration import LocalConfig, SerializationSettings
+from flytekit.core.artifact_utils import (
+    idl_partitions_from_dict,
+    idl_time_partition_from_datetime,
+)
+from flytekit.core.context_manager import (
+    ExecutionParameters,
+    ExecutionState,
+    FlyteContext,
+    FlyteContextManager,
+    FlyteEntities,
+)
 from flytekit.core.interface import Interface, transform_interface_to_typed_interface
 from flytekit.core.local_cache import LocalTaskCache
 from flytekit.core.promise import (
@@ -37,8 +71,15 @@ from flytekit.core.promise import (
     translate_inputs_to_literals,
 )
 from flytekit.core.tracker import TrackedInstance
-from flytekit.core.type_engine import TypeEngine
-from flytekit.deck.deck import Deck
+from flytekit.core.type_engine import TypeEngine, TypeTransformerFailedError
+from flytekit.core.utils import timeit
+from flytekit.deck import DeckField
+from flytekit.exceptions.system import (
+    FlyteDownloadDataException,
+    FlyteNonRecoverableSystemException,
+    FlyteUploadDataException,
+)
+from flytekit.exceptions.user import FlyteUserRuntimeException
 from flytekit.loggers import logger
 from flytekit.models import dynamic_job as _dynamic_job
 from flytekit.models import interface as _interface_models
@@ -48,15 +89,21 @@ from flytekit.models.core import workflow as _workflow_model
 from flytekit.models.documentation import Description, Documentation
 from flytekit.models.interface import Variable
 from flytekit.models.security import SecurityContext
+from flytekit.utils.asyn import run_sync
+
+DYNAMIC_PARTITIONS = "_uap"
+MODEL_CARD = "_ucm"
+DATA_CARD = "_ucd"
+UNSET_CARD = "_uc"
 
 
 def kwtypes(**kwargs) -> OrderedDict[str, Type]:
     """
     This is a small helper function to convert the keyword arguments to an OrderedDict of types.
 
-    .. code-block:: python
-
-        kwtypes(a=int, b=str)
+    ```python
+    kwtypes(a=int, b=str)
+    ```
     """
     d = collections.OrderedDict()
     for k, v in kwargs.items():
@@ -72,30 +119,35 @@ class TaskMetadata(object):
 
     See the :std:ref:`IDL <idl:protos/docs/core/core:taskmetadata>` for the protobuf definition.
 
-    Args:
-        cache (bool): Indicates if caching should be enabled. See :std:ref:`Caching <cookbook:caching>`
-        cache_serialize (bool): Indicates if identical (ie. same inputs) instances of this task should be executed in serial when caching is enabled. See :std:ref:`Caching <cookbook:caching>`
-        cache_version (str): Version to be used for the cached value
-        interruptible (Optional[bool]): Indicates that this task can be interrupted and/or scheduled on nodes with
-            lower QoS guarantees that can include pre-emption. This can reduce the monetary cost executions incur at the
-            cost of performance penalties due to potential interruptions
-        deprecated (str): Can be used to provide a warning message for deprecated task. Absence or empty str indicates
-            that the task is active and not deprecated
+    Attributes:
+        cache (bool): Indicates if caching should be enabled. See :std:ref:`Caching <cookbook:caching>`.
+        cache_serialize (bool): Indicates if identical (i.e. same inputs) instances of this task should be executed in serial when caching is enabled. See :std:ref:`Caching <cookbook:caching>`.
+        cache_version (str): Version to be used for the cached value.
+        cache_ignore_input_vars (Tuple[str, ...]): Input variables that should not be included when calculating hash for cache.
+        interruptible (Optional[bool]): Indicates that this task can be interrupted and/or scheduled on nodes with lower QoS guarantees that can include pre-emption.
+        deprecated (str): Can be used to provide a warning message for a deprecated task. An absence or empty string indicates that the task is active and not deprecated.
         retries (int): for retries=n; n > 0, on failures of this task, the task will be retried at-least n number of times.
-        timeout (Optional[Union[datetime.timedelta, int]]): the max amount of time for which one execution of this task
-            should be executed for. The execution will be terminated if the runtime exceeds the given timeout
-            (approximately)
-        pod_template_name (Optional[str]): the name of existing PodTemplate resource in the cluster which will be used in this task.
+        timeout (Optional[Union[datetime.timedelta, int]]): The maximum duration for which one execution of this task should run. The execution will be terminated if the runtime exceeds this timeout.
+        pod_template_name (Optional[str]): The name of an existing PodTemplate resource in the cluster which will be used for this task.
+        generates_deck (bool): Indicates whether the task will generate a Deck URI.
+        is_eager (bool): Indicates whether the task should be treated as eager.
+        labels (Optional[dict[str, str]]): Labels to be applied to the task resource.
+        annotations (Optional[dict[str, str]]): Annotations to be applied to the task resource.
     """
 
     cache: bool = False
     cache_serialize: bool = False
     cache_version: str = ""
+    cache_ignore_input_vars: Tuple[str, ...] = ()
     interruptible: Optional[bool] = None
     deprecated: str = ""
     retries: int = 0
     timeout: Optional[Union[datetime.timedelta, int]] = None
     pod_template_name: Optional[str] = None
+    generates_deck: bool = False
+    is_eager: bool = False
+    labels: Optional[dict[str, str]] = None
+    annotations: Optional[dict[str, str]] = None
 
     def __post_init__(self):
         if self.timeout:
@@ -107,6 +159,10 @@ class TaskMetadata(object):
             raise ValueError("Caching is enabled ``cache=True`` but ``cache_version`` is not set.")
         if self.cache_serialize and not self.cache:
             raise ValueError("Cache serialize is enabled ``cache_serialize=True`` but ``cache`` is not enabled.")
+        if self.cache_ignore_input_vars and not self.cache:
+            raise ValueError(
+                f"Cache ignore input vars are specified ``cache_ignore_input_vars={self.cache_ignore_input_vars}`` but ``cache`` is not enabled."
+            )
 
     @property
     def retry_strategy(self) -> _literal_models.RetryStrategy:
@@ -129,7 +185,14 @@ class TaskMetadata(object):
             discovery_version=self.cache_version,
             deprecated_error_message=self.deprecated,
             cache_serializable=self.cache_serialize,
+            generates_deck=self.generates_deck,
             pod_template_name=self.pod_template_name,
+            cache_ignore_input_vars=self.cache_ignore_input_vars,
+            is_eager=self.is_eager,
+            k8s_object_metadata=_task_model.K8sObjectMetadata(
+                labels=self.labels,
+                annotations=self.annotations,
+            ),
         )
 
 
@@ -137,9 +200,6 @@ class IgnoreOutputs(Exception):
     """
     This exception should be used to indicate that the outputs generated by this can be safely ignored.
     This is useful in case of distributed training or peer-to-peer parallel algorithms.
-
-    For example look at Sagemaker training, e.g.
-    :py:class:`plugins.awssagemaker.flytekitplugins.awssagemaker.training.SagemakerBuiltinAlgorithmsTask`.
     """
 
     pass
@@ -156,7 +216,7 @@ class Task(object):
         self,
         task_type: str,
         name: str,
-        interface: Optional[_interface_models.TypedInterface] = None,
+        interface: _interface_models.TypedInterface,
         metadata: Optional[TaskMetadata] = None,
         task_type_version=0,
         security_ctx: Optional[SecurityContext] = None,
@@ -174,7 +234,7 @@ class Task(object):
         FlyteEntities.entities.append(self)
 
     @property
-    def interface(self) -> Optional[_interface_models.TypedInterface]:
+    def interface(self) -> _interface_models.TypedInterface:
         return self._interface
 
     @property
@@ -227,7 +287,9 @@ class Task(object):
         """
         return None
 
-    def local_execute(self, ctx: FlyteContext, **kwargs) -> Union[Tuple[Promise], Promise, VoidPromise, None]:
+    def local_execute(
+        self, ctx: FlyteContext, **kwargs
+    ) -> Union[Tuple[Promise], Promise, VoidPromise, Coroutine, None]:
         """
         This function is used only in the local execution path and is responsible for calling dispatch execute.
         Use this function when calling a task with native values (or Promises containing Flyte literals derived from
@@ -239,39 +301,57 @@ class Task(object):
         #  Promises as essentially inputs from previous task executions
         #  native constants are just bound to this specific task (default values for a task input)
         #  Also along with promises and constants, there could be dictionary or list of promises or constants
-        kwargs = translate_inputs_to_literals(
-            ctx,
-            incoming_values=kwargs,
-            flyte_interface_types=self.interface.inputs,  # type: ignore
-            native_types=self.get_input_types(),
-        )
-        input_literal_map = _literal_models.LiteralMap(literals=kwargs)
+        try:
+            literals = translate_inputs_to_literals(
+                ctx,
+                incoming_values=kwargs,
+                flyte_interface_types=self.interface.inputs,
+                native_types=self.get_input_types(),  # type: ignore
+            )
+        except TypeTransformerFailedError as exc:
+            exc.args = (f"Failed to convert inputs of task '{self.name}':\n  {exc.args[0]}",)
+            raise
+        input_literal_map = _literal_models.LiteralMap(literals=literals)
 
         # if metadata.cache is set, check memoized version
-        if self.metadata.cache:
-            # TODO: how to get a nice `native_inputs` here?
-            logger.info(
-                f"Checking cache for task named {self.name}, cache version {self.metadata.cache_version} "
-                f"and inputs: {input_literal_map}"
-            )
-            outputs_literal_map = LocalTaskCache.get(self.name, self.metadata.cache_version, input_literal_map)
-            # The cache returns None iff the key does not exist in the cache
+        local_config = LocalConfig.auto()
+        if self.metadata.cache and local_config.cache_enabled:
+            if local_config.cache_overwrite:
+                outputs_literal_map = None
+                logger.info("Cache overwrite, task will be executed now")
+            else:
+                logger.info(
+                    f"Checking cache for task named {self.name}, cache version {self.metadata.cache_version} "
+                    f", inputs: {kwargs}, and ignore input vars: {self.metadata.cache_ignore_input_vars}"
+                )
+                outputs_literal_map = LocalTaskCache.get(
+                    self.name, self.metadata.cache_version, input_literal_map, self.metadata.cache_ignore_input_vars
+                )
+                # The cache returns None iff the key does not exist in the cache
+                if outputs_literal_map is None:
+                    logger.info("Cache miss, task will be executed now")
+                else:
+                    logger.info("Cache hit")
             if outputs_literal_map is None:
-                logger.info("Cache miss, task will be executed now")
-                outputs_literal_map = self.dispatch_execute(ctx, input_literal_map)
+                outputs_literal_map = self.sandbox_execute(ctx, input_literal_map)
                 # TODO: need `native_inputs`
-                LocalTaskCache.set(self.name, self.metadata.cache_version, input_literal_map, outputs_literal_map)
+                LocalTaskCache.set(
+                    self.name,
+                    self.metadata.cache_version,
+                    input_literal_map,
+                    self.metadata.cache_ignore_input_vars,
+                    outputs_literal_map,
+                )
                 logger.info(
                     f"Cache set for task named {self.name}, cache version {self.metadata.cache_version} "
-                    f"and inputs: {input_literal_map}"
+                    f", inputs: {kwargs}, and ignore input vars: {self.metadata.cache_ignore_input_vars}"
                 )
-            else:
-                logger.info("Cache hit")
         else:
-            es = ctx.execution_state
-            b = es.user_space_params.with_task_sandbox()
-            ctx = ctx.current_context().with_execution_state(es.with_params(user_space_params=b.build())).build()
-            outputs_literal_map = self.dispatch_execute(ctx, input_literal_map)
+            # This code should mirror the call to `sandbox_execute` in the above cache case.
+            # Code is simpler with duplication and less metaprogramming, but introduces regressions
+            # if one is changed and not the other.
+            outputs_literal_map = self.sandbox_execute(ctx, input_literal_map)
+
         outputs_literals = outputs_literal_map.literals
 
         # TODO maybe this is the part that should be done for local execution, we pass the outputs to some special
@@ -289,11 +369,11 @@ class Task(object):
         vals = [Promise(var, outputs_literals[var]) for var in output_names]
         return create_task_output(vals, self.python_interface)
 
-    def __call__(self, *args, **kwargs):
-        return flyte_entity_call_handler(self, *args, **kwargs)
+    def __call__(self, *args: object, **kwargs: object) -> Union[Tuple[Promise], Promise, VoidPromise, Tuple, None]:
+        return flyte_entity_call_handler(self, *args, **kwargs)  # type: ignore
 
     def compile(self, ctx: FlyteContext, *args, **kwargs):
-        raise Exception("not implemented")
+        raise NotImplementedError
 
     def get_container(self, settings: SerializationSettings) -> Optional[_task_model.Container]:
         """
@@ -326,6 +406,29 @@ class Task(object):
         """
         return None
 
+    def get_extended_resources(self, settings: SerializationSettings) -> Optional[tasks_pb2.ExtendedResources]:
+        """
+        Returns the extended resources to allocate to the task on hosted Flyte.
+        """
+        return None
+
+    def local_execution_mode(self) -> ExecutionState.Mode:
+        """ """
+        return ExecutionState.Mode.LOCAL_TASK_EXECUTION
+
+    def sandbox_execute(
+        self,
+        ctx: FlyteContext,
+        input_literal_map: _literal_models.LiteralMap,
+    ) -> _literal_models.LiteralMap:
+        """
+        Call dispatch_execute, in the context of a local sandbox execution. Not invoked during runtime.
+        """
+        es = cast(ExecutionState, ctx.execution_state)
+        b = cast(ExecutionParameters, es.user_space_params).with_task_sandbox()
+        ctx = ctx.current_context().with_execution_state(es.with_params(user_space_params=b.build())).build()
+        return self.dispatch_execute(ctx, input_literal_map)
+
     @abstractmethod
     def dispatch_execute(
         self,
@@ -364,17 +467,25 @@ T = TypeVar("T")
 class PythonTask(TrackedInstance, Task, Generic[T]):
     """
     Base Class for all Tasks with a Python native ``Interface``. This should be directly used for task types, that do
-    not have a python function to be executed. Otherwise refer to :py:class:`flytekit.PythonFunctionTask`.
+    not have a python function to be executed. Otherwise refer to {{< py_class_ref flytekit.PythonFunctionTask >}}.
     """
 
     def __init__(
         self,
         task_type: str,
         name: str,
-        task_config: T,
+        task_config: Optional[T] = None,
         interface: Optional[Interface] = None,
         environment: Optional[Dict[str, str]] = None,
-        disable_deck: bool = True,
+        disable_deck: Optional[bool] = None,
+        enable_deck: Optional[bool] = None,
+        deck_fields: Optional[Tuple[DeckField, ...]] = (
+            DeckField.SOURCE_CODE,
+            DeckField.DEPENDENCIES,
+            DeckField.TIMELINE,
+            DeckField.INPUT,
+            DeckField.OUTPUT,
+        ),
         **kwargs,
     ):
         """
@@ -388,18 +499,50 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
                 signature of the task
             environment (Optional[Dict[str, str]]): Any environment variables that should be supplied during the
                 execution of the task. Supplied as a dictionary of key/value pairs
-            disable_deck (bool): If true, this task will not output deck html file
+            disable_deck (bool): (deprecated) If true, this task will not output deck html file
+            enable_deck (bool): If true, this task will output deck html file
+            deck_fields (Tuple[DeckField]): Tuple of decks to be
+                generated for this task. Valid values can be selected from fields of ``flytekit.deck.DeckField`` enum
         """
         super().__init__(
             task_type=task_type,
             name=name,
-            interface=transform_interface_to_typed_interface(interface),
+            interface=transform_interface_to_typed_interface(interface, allow_partial_artifact_id_binding=True),
             **kwargs,
         )
         self._python_interface = interface if interface else Interface()
         self._environment = environment if environment else {}
         self._task_config = task_config
-        self._disable_deck = disable_deck
+
+        # first we resolve the conflict between params regarding decks, if any two of [disable_deck, enable_deck]
+        # are set, we raise an error
+        configured_deck_params = [disable_deck is not None, enable_deck is not None]
+        if sum(configured_deck_params) > 1:
+            raise ValueError("only one of [disable_deck, enable_deck] can be set")
+
+        if disable_deck is not None:
+            warnings.warn(
+                "disable_deck was deprecated in 1.10.0, please use enable_deck instead",
+                FutureWarning,
+            )
+
+        if enable_deck is not None:
+            self._disable_deck = not enable_deck
+        elif disable_deck is not None:
+            self._disable_deck = disable_deck
+        else:
+            self._disable_deck = True
+
+        self._deck_fields = list(deck_fields) if (deck_fields is not None and self.enable_deck) else []
+
+        deck_members = set([_field for _field in DeckField])
+        # enumerate additional decks, check if any of them are invalid
+        for deck in self._deck_fields:
+            if deck not in deck_members:
+                raise ValueError(
+                    f"Element {deck} from deck_fields param is not a valid deck field. Please use one of {deck_members}"
+                )
+
         if self._python_interface.docstring:
             if self.docs is None:
                 self._docs = Documentation(
@@ -408,9 +551,13 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
                 )
             else:
                 if self._python_interface.docstring.short_description:
-                    self._docs.short_description = self._python_interface.docstring.short_description
+                    cast(
+                        Documentation, self._docs
+                    ).short_description = self._python_interface.docstring.short_description
                 if self._python_interface.docstring.long_description:
-                    self._docs.long_description = Description(value=self._python_interface.docstring.long_description)
+                    cast(Documentation, self._docs).long_description = Description(
+                        value=self._python_interface.docstring.long_description
+                    )
 
     # TODO lets call this interface and the other as flyte_interface?
     @property
@@ -421,25 +568,25 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
         return self._python_interface
 
     @property
-    def task_config(self) -> T:
+    def task_config(self) -> Optional[T]:
         """
         Returns the user-specified task config which is used for plugin-specific handling of the task.
         """
         return self._task_config
 
-    def get_type_for_input_var(self, k: str, v: Any) -> Optional[Type[Any]]:
+    def get_type_for_input_var(self, k: str, v: Any) -> Type[Any]:
         """
         Returns the python type for an input variable by name.
         """
         return self._python_interface.inputs[k]
 
-    def get_type_for_output_var(self, k: str, v: Any) -> Optional[Type[Any]]:
+    def get_type_for_output_var(self, k: str, v: Any) -> Type[Any]:
         """
         Returns the python type for the specified output variable by name.
         """
         return self._python_interface.outputs[k]
 
-    def get_input_types(self) -> Optional[Dict[str, type]]:
+    def get_input_types(self) -> Dict[str, type]:
         """
         Returns the names and python types as a dictionary for the inputs of this task.
         """
@@ -456,7 +603,7 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
             interruptible=self.metadata.interruptible,
         )
 
-    def compile(self, ctx: FlyteContext, *args, **kwargs):
+    def compile(self, ctx: FlyteContext, *args, **kwargs) -> Optional[Union[Tuple[Promise], Promise, VoidPromise]]:
         """
         Generates a node that encapsulates this task in a workflow definition.
         """
@@ -466,9 +613,115 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
     def _outputs_interface(self) -> Dict[Any, Variable]:
         return self.interface.outputs  # type: ignore
 
+    def _literal_map_to_python_input(
+        self, literal_map: _literal_models.LiteralMap, ctx: FlyteContext
+    ) -> Dict[str, Any]:
+        return TypeEngine.literal_map_to_kwargs(ctx, literal_map, self.python_interface.inputs)
+
+    async def _output_to_literal_map(self, native_outputs: Dict[int, Any], ctx: FlyteContext):
+        expected_output_names = list(self._outputs_interface.keys())
+        if len(expected_output_names) == 1:
+            # Here we have to handle the fact that the task could've been declared with a typing.NamedTuple of
+            # length one. That convention is used for naming outputs - and single-length-NamedTuples are
+            # particularly troublesome, but elegant handling of them is not a high priority
+            # Again, we're using the output_tuple_name as a proxy.
+            if self.python_interface.output_tuple_name and isinstance(native_outputs, tuple):
+                native_outputs_as_map = {expected_output_names[0]: native_outputs[0]}
+            else:
+                native_outputs_as_map = {expected_output_names[0]: native_outputs}
+        elif len(expected_output_names) == 0:
+            native_outputs_as_map = {}
+        else:
+            native_outputs_as_map = {expected_output_names[i]: native_outputs[i] for i, _ in enumerate(native_outputs)}
+
+        # We manually construct a LiteralMap here because task inputs and outputs actually violate the assumption
+        # built into the IDL that all the values of a literal map are of the same type.
+        with timeit("Translate the output to literals"):
+            literals = {}
+            omt = ctx.output_metadata_tracker
+            # Here is where we iterate through the outputs, need to call new type engine.
+            for i, (k, v) in enumerate(native_outputs_as_map.items()):
+                literal_type = self._outputs_interface[k].type
+                py_type = self.get_type_for_output_var(k, v)
+
+                if isinstance(v, tuple):
+                    raise TypeError(f"Output({k}) in task '{self.name}' received a tuple {v}, instead of {py_type}")
+                literals[k] = asyncio.create_task(TypeEngine.async_to_literal(ctx, v, py_type, literal_type))
+
+            await asyncio.gather(*literals.values(), return_exceptions=True)
+
+            for i, (k2, v2) in enumerate(literals.items()):
+                if v2.exception() is not None:
+                    # only show the name of output key if it's user-defined (by default Flyte names these as "o<n>")
+                    key = k2 if k2 != f"o{i}" else i
+                    e: BaseException = v2.exception()  # type: ignore  # we know this is not optional
+                    py_type = self.get_type_for_output_var(k2, native_outputs_as_map[k2])
+                    e.args = (
+                        f"Failed to convert outputs of task '{self.name}' at position {key}.\n"
+                        f"Failed to convert type {type(native_outputs_as_map[expected_output_names[i]])} to type {py_type}.\n"
+                        f"Error Message: {e.args[0]}.",
+                    )
+                    raise e
+                literals[k2] = v2.result()
+
+            if omt is not None:
+                for i, (k, v) in enumerate(native_outputs_as_map.items()):
+                    # Now check if there is any output metadata associated with this output variable and attach it to the
+                    # literal
+                    om = omt.get(v)
+                    if om:
+                        metadata = {}
+                        if om.additional_items:
+                            for ii in om.additional_items:
+                                md_key, md_val = ii.serialize_to_string(ctx, k)
+                                metadata[md_key] = md_val
+                            logger.info(f"Adding {om.additional_items} additional metadata items {metadata} for {k}")
+                        if om.dynamic_partitions or om.time_partition:
+                            a = art_id.ArtifactID(
+                                partitions=idl_partitions_from_dict(om.dynamic_partitions),
+                                time_partition=idl_time_partition_from_datetime(
+                                    om.time_partition, om.artifact.time_partition_granularity
+                                ),
+                            )
+                            s = a.SerializeToString()
+                            encoded = b64encode(s).decode("utf-8")
+                            metadata[DYNAMIC_PARTITIONS] = encoded
+                        if metadata:
+                            literals[k].set_metadata(metadata)  # type: ignore  # we know these have been resolved
+
+        return _literal_models.LiteralMap(literals=literals), native_outputs_as_map
+
+    def _write_decks(self, native_inputs, native_outputs_as_map, ctx, new_user_params):
+        if self._disable_deck is False:
+            from flytekit.deck.deck import Deck, DeckField, _output_deck
+
+            INPUT = DeckField.INPUT
+            OUTPUT = DeckField.OUTPUT
+
+            if DeckField.INPUT in self.deck_fields:
+                input_deck = Deck(INPUT.value)
+                for k, v in native_inputs.items():
+                    input_deck.append(TypeEngine.to_html(ctx, v, self.get_type_for_input_var(k, v)))
+
+            if DeckField.OUTPUT in self.deck_fields:
+                output_deck = Deck(OUTPUT.value)
+                for k, v in native_outputs_as_map.items():
+                    output_deck.append(TypeEngine.to_html(ctx, v, self.get_type_for_output_var(k, v)))
+
+            if ctx.execution_state and ctx.execution_state.is_local_execution():
+                # When we run the workflow remotely, flytekit outputs decks at the end of _dispatch_execute
+                _output_deck(self.name.split(".")[-1], new_user_params)
+
+    async def _async_execute(self, native_inputs, native_outputs, ctx, exec_ctx, new_user_params):
+        native_outputs = await native_outputs
+        native_outputs = self.post_execute(new_user_params, native_outputs)
+        literals_map, native_outputs_as_map = await self._output_to_literal_map(native_outputs, exec_ctx)
+        self._write_decks(native_inputs, native_outputs_as_map, ctx, new_user_params)
+        return literals_map
+
     def dispatch_execute(
         self, ctx: FlyteContext, input_literal_map: _literal_models.LiteralMap
-    ) -> Union[_literal_models.LiteralMap, _dynamic_job.DynamicJobSpec]:
+    ) -> Union[_literal_models.LiteralMap, _dynamic_job.DynamicJobSpec, Coroutine]:
         """
         This method translates Flyte's Type system based input values and invokes the actual call to the executor
         This method is also invoked during runtime.
@@ -481,27 +734,44 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
 
         # Invoked before the task is executed
         new_user_params = self.pre_execute(ctx.user_space_params)
-        from flytekit.deck.deck import _output_deck
+
+        if self.enable_deck and ctx.user_space_params is not None:
+            if DeckField.TIMELINE.value in self.deck_fields:
+                ctx.user_space_params.decks.append(ctx.user_space_params.timeline_deck)
+            new_user_params = ctx.user_space_params.with_enable_deck(enable_deck=True).build()
 
         # Create another execution context with the new user params, but let's keep the same working dir
         with FlyteContextManager.with_context(
-            ctx.with_execution_state(ctx.execution_state.with_params(user_space_params=new_user_params))
+            ctx.with_execution_state(
+                cast(ExecutionState, ctx.execution_state).with_params(user_space_params=new_user_params)
+            )
             # type: ignore
         ) as exec_ctx:
+            is_local_execution = cast(ExecutionState, exec_ctx.execution_state).is_local_execution()
             # TODO We could support default values here too - but not part of the plan right now
             # Translate the input literals to Python native
-            native_inputs = TypeEngine.literal_map_to_kwargs(exec_ctx, input_literal_map, self.python_interface.inputs)
-
+            try:
+                native_inputs = self._literal_map_to_python_input(input_literal_map, exec_ctx)
+            except (FlyteUploadDataException, FlyteDownloadDataException):
+                raise
+            except Exception as e:
+                if is_local_execution:
+                    e.args = (f"Error encountered while converting inputs of '{self.name}':\n  {e}",)
+                    raise
+                raise FlyteNonRecoverableSystemException(e) from e
             # TODO: Logger should auto inject the current context information to indicate if the task is running within
             #   a workflow or a subworkflow etc
             logger.info(f"Invoking {self.name} with inputs: {native_inputs}")
-            try:
-                native_outputs = self.execute(**native_inputs)
-            except Exception as e:
-                logger.exception(f"Exception when executing {e}")
-                raise e
+            with timeit("Execute user level code"):
+                try:
+                    native_outputs = self.execute(**native_inputs)
+                except Exception as e:
+                    if is_local_execution:
+                        # If the task is being executed locally, we want to raise the original exception
+                        e.args = (f"Error encountered while executing '{self.name}':\n  {e}",)
+                        raise
+                    raise FlyteUserRuntimeException(e) from e
 
-            logger.debug("Task executed successfully in user level")
             # Lets run the post_execute method. This may result in a IgnoreOutputs Exception, which is
             # bubbled up to be handled at the callee layer.
             native_outputs = self.post_execute(new_user_params, native_outputs)
@@ -509,64 +779,29 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
             # Short circuit the translation to literal map because what's returned may be a dj spec (or an
             # already-constructed LiteralMap if the dynamic task was a no-op), not python native values
             # dynamic_execute returns a literal map in local execute so this also gets triggered.
-            if isinstance(native_outputs, _literal_models.LiteralMap) or isinstance(
-                native_outputs, _dynamic_job.DynamicJobSpec
+            if isinstance(
+                native_outputs,
+                (_literal_models.LiteralMap, _dynamic_job.DynamicJobSpec),
             ):
                 return native_outputs
 
-            expected_output_names = list(self._outputs_interface.keys())
-            if len(expected_output_names) == 1:
-                # Here we have to handle the fact that the task could've been declared with a typing.NamedTuple of
-                # length one. That convention is used for naming outputs - and single-length-NamedTuples are
-                # particularly troublesome but elegant handling of them is not a high priority
-                # Again, we're using the output_tuple_name as a proxy.
-                if self.python_interface.output_tuple_name and isinstance(native_outputs, tuple):
-                    native_outputs_as_map = {expected_output_names[0]: native_outputs[0]}
-                else:
-                    native_outputs_as_map = {expected_output_names[0]: native_outputs}
-            elif len(expected_output_names) == 0:
-                native_outputs_as_map = {}
-            else:
-                native_outputs_as_map = {
-                    expected_output_names[i]: native_outputs[i] for i, _ in enumerate(native_outputs)
-                }
+            try:
+                with timeit("dispatch execute"):
+                    literals_map, native_outputs_as_map = run_sync(
+                        self._output_to_literal_map, native_outputs, exec_ctx
+                    )
+                self._write_decks(native_inputs, native_outputs_as_map, ctx, new_user_params)
+            except (FlyteUploadDataException, FlyteDownloadDataException):
+                raise
+            except Exception as e:
+                if is_local_execution:
+                    raise
+                raise FlyteNonRecoverableSystemException(e) from e
 
-            # We manually construct a LiteralMap here because task inputs and outputs actually violate the assumption
-            # built into the IDL that all the values of a literal map are of the same type.
-            literals = {}
-            for k, v in native_outputs_as_map.items():
-                literal_type = self._outputs_interface[k].type
-                py_type = self.get_type_for_output_var(k, v)
+            # After the execution has been successfully completed
+            return literals_map
 
-                if isinstance(v, tuple):
-                    raise TypeError(f"Output({k}) in task{self.name} received a tuple {v}, instead of {py_type}")
-                try:
-                    literals[k] = TypeEngine.to_literal(exec_ctx, v, py_type, literal_type)
-                except Exception as e:
-                    logger.error(f"Failed to convert return value for var {k} with error {type(e)}: {e}")
-                    raise TypeError(
-                        f"Failed to convert return value for var {k} for function {self.name} with error {type(e)}: {e}"
-                    ) from e
-
-            if self._disable_deck is False:
-                INPUT = "input"
-                OUTPUT = "output"
-
-                input_deck = Deck(INPUT)
-                for k, v in native_inputs.items():
-                    input_deck.append(TypeEngine.to_html(ctx, v, self.get_type_for_input_var(k, v)))
-
-                output_deck = Deck(OUTPUT)
-                for k, v in native_outputs_as_map.items():
-                    output_deck.append(TypeEngine.to_html(ctx, v, self.get_type_for_output_var(k, v)))
-
-                _output_deck(self.name.split(".")[-1], new_user_params)
-
-            outputs_literal_map = _literal_models.LiteralMap(literals=literals)
-            # After the execute has been successfully completed
-            return outputs_literal_map
-
-    def pre_execute(self, user_params: ExecutionParameters) -> ExecutionParameters:
+    def pre_execute(self, user_params: Optional[ExecutionParameters]) -> Optional[ExecutionParameters]:  # type: ignore
         """
         This is the method that will be invoked directly before executing the task method and before all the inputs
         are converted. One particular case where this is useful is if the context is to be modified for the user process
@@ -584,7 +819,7 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
         """
         pass
 
-    def post_execute(self, user_params: ExecutionParameters, rval: Any) -> Any:
+    def post_execute(self, user_params: Optional[ExecutionParameters], rval: Any) -> Any:
         """
         Post execute is called after the execution has completed, with the user_params and can be used to clean-up,
         or alter the outputs to match the intended tasks outputs. If not overridden, then this function is a No-op
@@ -607,8 +842,26 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
         """
         If true, this task will not output deck html file
         """
+        warnings.warn(
+            "`disable_deck` is deprecated and will be removed in the future.\n" "Please use `enable_deck` instead.",
+            DeprecationWarning,
+        )
         return self._disable_deck
 
+    @property
+    def enable_deck(self) -> bool:
+        """
+        If true, this task will output deck html file
+        """
+        return not self._disable_deck
+
+    @property
+    def deck_fields(self) -> List[DeckField]:
+        """
+        If not empty, this task will output deck html file for the specified decks
+        """
+        return self._deck_fields
+
 
 class TaskResolverMixin(object):
     """
