@@ -2,9 +2,9 @@ from __future__ import annotations
 
 import datetime
 import typing
-from typing import Optional, Tuple, Union
+from typing import Optional, Tuple, Union, cast
 
-from flytekit.core.context_manager import ExecutionState, FlyteContextManager
+from flytekit.core.context_manager import FlyteContextManager
 from flytekit.core.node import Node
 from flytekit.core.promise import (
     ComparisonExpression,
@@ -38,16 +38,14 @@ class ConditionalSection:
     for Compilation mode. It is advised to derive the class and re-implement the `start_branch` and `end_branch` methods
     to override the compilation behavior
 
-    .. note::
-
-        Conditions can only be used within a workflow context.
+    > [!NOTE]
+    > Conditions can only be used within a workflow context.
 
     Usage:
 
-    .. code-block:: python
-
-        v =  conditional("fractions").if_((my_input > 0.1) & (my_input < 1.0)).then(...)...
-
+    ```python
+    v =  conditional("fractions").if_((my_input > 0.1) & (my_input < 1.0)).then(...)...
+    ```
     """
 
     def __init__(self, name: str):
@@ -77,7 +75,7 @@ class ConditionalSection:
         self._cases.append(c)
         return self._cases[-1]
 
-    def end_branch(self) -> Union[Condition, Promise]:
+    def end_branch(self) -> Optional[Union[Condition, Promise, Tuple[Promise], VoidPromise]]:
         """
         This should be invoked after every branch has been visited.
         In case this is not local workflow execution then, we should check if this is the last case.
@@ -101,52 +99,70 @@ class ConditionalSection:
                     upstream_nodes.add(p.ref.node)
 
             n = Node(
-                id=f"{ctx.compilation_state.prefix}n{len(ctx.compilation_state.nodes)}",
+                id=f"{ctx.compilation_state.prefix}n{len(ctx.compilation_state.nodes)}",  # type: ignore
                 metadata=_core_wf.NodeMetadata(self._name, timeout=datetime.timedelta(), retries=RetryStrategy(0)),
                 bindings=sorted(bindings, key=lambda b: b.var),
                 upstream_nodes=list(upstream_nodes),  # type: ignore
                 flyte_entity=node,
             )
-            FlyteContextManager.current_context().compilation_state.add_node(n)
+            FlyteContextManager.current_context().compilation_state.add_node(n)  # type: ignore
             return self._compute_outputs(n)
         return self._condition
 
-    def if_(self, expr: bool) -> Case:
+    def if_(self, expr: Union[ComparisonExpression, ConjunctionExpression]) -> Case:
         return self._condition._if(expr)
 
-    def compute_output_set(self) -> typing.Optional[typing.Set[str]]:
+    def compute_output_vars(self) -> typing.Optional[typing.List[str]]:
         """
         Computes and returns the minimum set of outputs for this conditional block, based on all the cases that have
         been registered
         """
-        output_var_sets: typing.List[typing.Set[str]] = []
+        output_vars: typing.List[str] = []
+        output_vars_set = set()
         for c in self._cases:
             if c.output_promise is None and c.err is None:
                 # One node returns a void output and no error, we will default to None return
                 return None
+            if isinstance(c.output_promise, VoidPromise):
+                return None
             if c.output_promise is not None:
+                var = []
                 if isinstance(c.output_promise, tuple):
-                    output_var_sets.append(set([i.var for i in c.output_promise]))
+                    var = [i.var for i in c.output_promise]
                 else:
-                    output_var_sets.append({c.output_promise.var})
-        curr = output_var_sets[0]
-        if len(output_var_sets) > 1:
-            for x in output_var_sets[1:]:
-                curr = curr.intersection(x)
-        return curr
+                    var = [c.output_promise.var]
+                curr_set = set(var)
+                if not output_vars:
+                    output_vars = var
+                    output_vars_set = curr_set
+                else:
+                    output_vars_set = output_vars_set.intersection(curr_set)
+                    new_output_var = []
+                    for v in output_vars:
+                        if v in output_vars_set:
+                            new_output_var.append(v)
+                    output_vars = new_output_var
 
-    def _compute_outputs(self, n: Node) -> Union[Promise, Tuple[Promise], VoidPromise]:
-        curr = self.compute_output_set()
-        if curr is None:
-            return VoidPromise(n.id)
+        return output_vars
+
+    def _compute_outputs(self, n: Node) -> Optional[Union[Promise, Tuple[Promise], VoidPromise]]:
+        curr = self.compute_output_vars()
+        if curr is None or len(curr) == 0:
+            return VoidPromise(n.id, NodeOutput(node=n, var="placeholder"))
         promises = [Promise(var=x, val=NodeOutput(node=n, var=x)) for x in curr]
         # TODO: Is there a way to add the Python interface here? Currently, it's an optional arg.
         return create_task_output(promises)
 
+    def __repr__(self):
+        return self._condition.__repr__()
+
+    def __str__(self):
+        return self.__repr__()
+
 
 class LocalExecutedConditionalSection(ConditionalSection):
     def __init__(self, name: str):
-        self._selected_case = None
+        self._selected_case: Optional[Case] = None
         super().__init__(name=name)
 
     def start_branch(self, c: Case, last_case: bool = False) -> Case:
@@ -161,7 +177,7 @@ class LocalExecutedConditionalSection(ConditionalSection):
         # We already have a candidate case selected
         if self._selected_case is None:
             if c.expr is None or c.expr.eval() or last_case:
-                ctx.execution_state.take_branch()
+                ctx.execution_state.take_branch()  # type: ignore
                 self._selected_case = added_case
         return added_case
 
@@ -176,22 +192,22 @@ class LocalExecutedConditionalSection(ConditionalSection):
         """
         ctx = FlyteContextManager.current_context()
         # Let us mark the execution state as complete
-        ctx.execution_state.branch_complete()
-        if self._last_case:
+        ctx.execution_state.branch_complete()  # type: ignore
+        if self._last_case and self._selected_case:
             # We have completed the conditional section, lets pop off the branch context
             FlyteContextManager.pop_context()
             if self._selected_case.output_promise is None and self._selected_case.err is None:
                 raise AssertionError("Bad conditional statements, did not resolve in a promise")
             elif self._selected_case.output_promise is not None:
-                return self._compute_outputs(self._selected_case.output_promise)
+                return typing.cast(Promise, self._compute_outputs(self._selected_case.output_promise))
             raise ValueError(self._selected_case.err)
         return self._condition
 
-    def _compute_outputs(self, selected_output_promise) -> Union[Promise, Tuple[Promise], VoidPromise]:
+    def _compute_outputs(self, selected_output_promise) -> Optional[Union[Tuple[Promise], Promise, VoidPromise]]:
         """
         For the local execution case only returns the least common set of outputs
         """
-        curr = self.compute_output_set()
+        curr = self.compute_output_vars()
         if curr is None:
             return VoidPromise(self.name)
         if not isinstance(selected_output_promise, tuple):
@@ -209,13 +225,13 @@ class SkippedConditionalSection(ConditionalSection):
     def __init__(self, name: str):
         super().__init__(name=name)
 
-    def end_branch(self) -> Union[Condition, Promise]:
+    def end_branch(self) -> Optional[Union[Condition, Tuple[Promise], Promise, VoidPromise]]:
         """
         This should be invoked after every branch has been visited
         """
         if self._last_case:
             FlyteContextManager.pop_context()
-            curr = self.compute_output_set()
+            curr = self.compute_output_vars()
             if curr is None:
                 return VoidPromise(self.name)
             promises = [Promise(var=x, val=None) for x in curr]
@@ -251,7 +267,15 @@ class Case(object):
                 )
         self._expr = expr
         self._output_promise: Optional[Union[Tuple[Promise], Promise]] = None
-        self._err = None
+        self._err: Optional[str] = None
+        self._stmt = stmt
+        self._output_node = None
+
+    @property
+    def output_node(self) -> Optional[Node]:
+        # This is supposed to hold a pointer to the node that created this case.
+        # It is set in the then() call. but the value will not be set if it's a VoidPromise or None was returned.
+        return self._output_node
 
     @property
     def expr(self) -> Optional[Union[ComparisonExpression, ConjunctionExpression]]:
@@ -266,14 +290,37 @@ class Case(object):
         return self._err
 
     # TODO this is complicated. We do not want this to run
-    def then(self, p: Union[Promise, Tuple[Promise]]) -> Union[Condition, Promise]:
+    def then(
+        self, p: Union[Promise, Tuple[Promise]]
+    ) -> Optional[Union[Condition, Promise, Tuple[Promise], VoidPromise]]:
         self._output_promise = p
+        if isinstance(p, Promise):
+            if not p.is_ready:
+                self._output_node = p.ref.node  # type: ignore
+        elif isinstance(p, VoidPromise):
+            if p.ref is not None:
+                self._output_node = p.ref.node
+        elif hasattr(p, "_fields"):
+            # This condition detects the NamedTuple case and iterates through the fields to find one that has a node
+            # which should be the first one.
+            for f in p._fields:  # type: ignore
+                prom = getattr(p, f)
+                if not prom.is_ready:
+                    self._output_node = prom.ref.node
+                    break
+
         # We can always mark branch as completed
         return self._cs.end_branch()
 
     def fail(self, err: str) -> Promise:
         self._err = err
-        return self._cs.end_branch()
+        return cast(Promise, self._cs.end_branch())
+
+    def __repr__(self):
+        return f"{self._stmt}({self.expr.__repr__()})"
+
+    def __str__(self):
+        return self.__repr__()
 
 
 class Condition(object):
@@ -293,6 +340,12 @@ class Condition(object):
     def else_(self) -> Case:
         return self._cs.start_branch(Case(cs=self._cs, expr=None, stmt="else_"), last_case=True)
 
+    def __repr__(self):
+        return f"condition({self._cs.name}) " + "".join([x.__repr__() for x in self._cs.cases])
+
+    def __str__(self):
+        return self.__repr__()
+
 
 _logical_ops = {
     ConjunctionOps.AND: _core_cond.ConjunctionExpression.LogicalOperator.AND,
@@ -327,7 +380,7 @@ def create_branch_node_promise_var(node_id: str, var: str) -> str:
     return f"{node_id}.{var}"
 
 
-def merge_promises(*args: Promise) -> typing.List[Promise]:
+def merge_promises(*args: Optional[Promise]) -> typing.List[Promise]:
     node_vars: typing.Set[typing.Tuple[str, str]] = set()
     merged_promises: typing.List[Promise] = []
     for p in args:
@@ -358,10 +411,12 @@ def transform_to_conj_expr(
 def transform_to_operand(v: Union[Promise, Literal]) -> Tuple[_core_cond.Operand, Optional[Promise]]:
     if isinstance(v, Promise):
         return _core_cond.Operand(var=create_branch_node_promise_var(v.ref.node_id, v.var)), v
+    if v.scalar.none_type:
+        return _core_cond.Operand(scalar=v.scalar), None
     return _core_cond.Operand(primitive=v.scalar.primitive), None
 
 
-def transform_to_comp_expr(expr: ComparisonExpression) -> (_core_cond.ComparisonExpression, typing.List[Promise]):
+def transform_to_comp_expr(expr: ComparisonExpression) -> Tuple[_core_cond.ComparisonExpression, typing.List[Promise]]:
     o_lhs, b_lhs = transform_to_operand(expr.lhs)
     o_rhs, b_rhs = transform_to_operand(expr.rhs)
     return (
@@ -371,8 +426,8 @@ def transform_to_comp_expr(expr: ComparisonExpression) -> (_core_cond.Comparison
 
 
 def transform_to_boolexpr(
-    expr: Union[ComparisonExpression, ConjunctionExpression]
-) -> (_core_cond.BooleanExpression, typing.List[Promise]):
+    expr: Union[ComparisonExpression, ConjunctionExpression],
+) -> Tuple[_core_cond.BooleanExpression, typing.List[Promise]]:
     if isinstance(expr, ConjunctionExpression):
         cexpr, promises = transform_to_conj_expr(expr)
         return _core_cond.BooleanExpression(conjunction=cexpr), promises
@@ -380,13 +435,14 @@ def transform_to_boolexpr(
     return _core_cond.BooleanExpression(comparison=cexpr), promises
 
 
-def to_case_block(c: Case) -> (Union[_core_wf.IfBlock], typing.List[Promise]):
-    expr, promises = transform_to_boolexpr(c.expr)
-    n = c.output_promise.ref.node
+def to_case_block(c: Case) -> Tuple[Union[_core_wf.IfBlock], typing.List[Promise]]:
+    expr, promises = transform_to_boolexpr(cast(Union[ComparisonExpression, ConjunctionExpression], c.expr))
+    if c.output_promise is not None:
+        n = c.output_node
     return _core_wf.IfBlock(condition=expr, then_node=n), promises
 
 
-def to_ifelse_block(node_id: str, cs: ConditionalSection) -> (_core_wf.IfElseBlock, typing.List[Binding]):
+def to_ifelse_block(node_id: str, cs: ConditionalSection) -> Tuple[_core_wf.IfElseBlock, typing.List[Binding]]:
     if len(cs.cases) == 0:
         raise AssertionError("Illegal Condition block, with no if-else cases")
     if len(cs.cases) < 2:
@@ -405,7 +461,7 @@ def to_ifelse_block(node_id: str, cs: ConditionalSection) -> (_core_wf.IfElseBlo
     node = None
     err = None
     if last_case.output_promise is not None:
-        node = last_case.output_promise.ref.node
+        node = last_case.output_node
     else:
         err = Error(failed_node_id=node_id, message=last_case.err if last_case.err else "Condition failed")
     return (
@@ -414,7 +470,7 @@ def to_ifelse_block(node_id: str, cs: ConditionalSection) -> (_core_wf.IfElseBlo
     )
 
 
-def to_branch_node(name: str, cs: ConditionalSection) -> (BranchNode, typing.List[Promise]):
+def to_branch_node(name: str, cs: ConditionalSection) -> Tuple[BranchNode, typing.List[Promise]]:
     blocks, promises = to_ifelse_block(name, cs)
     return BranchNode(name=name, ifelse_block=blocks), promises
 
@@ -430,32 +486,32 @@ def conditional(name: str) -> ConditionalSection:
 
     Example of a condition usage. Note the nesting and the assignment to a LHS variable
 
-    .. code-block:: python
-
-         v = (
-            conditional("fractions")
-            .if_((my_input > 0.1) & (my_input < 1.0))
-            .then(
-                conditional("inner_fractions")
-                .if_(my_input < 0.5)
-                .then(double(n=my_input))
-                .elif_((my_input > 0.5) & (my_input < 0.7))
-                .then(square(n=my_input))
-                .else_()
-                .fail("Only <0.7 allowed")
-            )
-            .elif_((my_input > 1.0) & (my_input < 10.0))
-            .then(square(n=my_input))
-            .else_()
-            .then(double(n=my_input))
-        )
+    ```python
+    v = (
+    conditional("fractions")
+    .if_((my_input > 0.1) & (my_input < 1.0))
+    .then(
+        conditional("inner_fractions")
+        .if_(my_input < 0.5)
+        .then(double(n=my_input))
+        .elif_((my_input > 0.5) & (my_input < 0.7))
+        .then(square(n=my_input))
+        .else_()
+        .fail("Only <0.7 allowed")
+    )
+    .elif_((my_input > 1.0) & (my_input < 10.0))
+    .then(square(n=my_input))
+    .else_()
+    .then(double(n=my_input))
+    )
+    ```
     """
     ctx = FlyteContextManager.current_context()
 
     if ctx.compilation_state:
         return ConditionalSection(name)
     elif ctx.execution_state:
-        if ctx.execution_state.mode == ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION:
+        if ctx.execution_state.is_local_execution():
             # In case of Local workflow execution, we will actually evaluate the expression and based on the result
             # make the branch to be active using `take_branch` method
             from flytekit.core.context_manager import BranchEvalMode
