@@ -1,33 +1,25 @@
 """
-
-.. autoclass:: flytekit.core.context_manager::ExecutionState.Mode
-   :noindex:
-.. autoclass:: flytekit.core.context_manager::ExecutionState.Mode.TASK_EXECUTION
-   :noindex:
-.. autoclass:: flytekit.core.context_manager::ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION
-   :noindex:
-.. autoclass:: flytekit.core.context_manager::ExecutionState.Mode.LOCAL_TASK_EXECUTION
-   :noindex:
-
+These classes provide functionality related context management.
 """
 
 from __future__ import annotations
 
-import datetime as _datetime
 import logging as _logging
 import os
 import pathlib
+import signal
 import tempfile
+import threading
 import traceback
 import typing
 from contextlib import contextmanager
 from contextvars import ContextVar
 from dataclasses import dataclass, field
-from datetime import datetime
+from datetime import datetime, timezone
 from enum import Enum
+from types import FrameType
 from typing import Generator, List, Optional, Union
 
-from flytekit.clients import friendly as friendly_client  # noqa
 from flytekit.configuration import Config, SecretsConfig, SerializationSettings
 from flytekit.core import mock_stats, utils
 from flytekit.core.checkpointer import Checkpoint, SyncCheckpoint
@@ -35,10 +27,13 @@ from flytekit.core.data_persistence import FileAccessProvider, default_local_fil
 from flytekit.core.node import Node
 from flytekit.interfaces.cli_identifiers import WorkflowExecutionIdentifier
 from flytekit.interfaces.stats import taggable
-from flytekit.loggers import logger, user_space_logger
+from flytekit.loggers import developer_logger, user_space_logger
 from flytekit.models.core import identifier as _identifier
 
 if typing.TYPE_CHECKING:
+    from flytekit.clients import friendly as friendly_client  # noqa
+    from flytekit.clients.friendly import SynchronousFlyteClient
+    from flytekit.core.worker_queue import Controller
     from flytekit.deck.deck import Deck
 
 # TODO: resolve circular import from flytekit.core.python_auto_container import TaskResolverMixin
@@ -48,7 +43,7 @@ if typing.TYPE_CHECKING:
 flyte_context_Var: ContextVar[typing.List[FlyteContext]] = ContextVar("", default=[])
 
 if typing.TYPE_CHECKING:
-    from flytekit.core.base_task import TaskResolverMixin
+    from flytekit.core.base_task import Task, TaskResolverMixin
 
 
 # Identifier fields use placeholders for registration-time substitution.
@@ -64,17 +59,18 @@ class ExecutionParameters(object):
     """
     This is a run-time user-centric context object that is accessible to every @task method. It can be accessed using
 
-    .. code-block:: python
 
-        flytekit.current_context()
+    ```python
+    flytekit.current_context()
+    ```
 
-    This object provides the following
+    This object provides the following objections
     * a statsd handler
     * a logging handler
-    * the execution ID as an :py:class:`flytekit.models.core.identifier.WorkflowExecutionIdentifier` object
+    * the execution ID as an `flytekit.models.core.identifier.WorkflowExecutionIdentifier` object
     * a working directory for the user to write arbitrary files to
 
-    Please do not confuse this object with the :py:class:`flytekit.FlyteContext` object.
+    Please do not confuse this object with the `flytekit.FlyteContext` object.
     """
 
     @dataclass(init=False)
@@ -84,11 +80,13 @@ class ExecutionParameters(object):
         decks: List[Deck]
         raw_output_prefix: Optional[str] = None
         execution_id: typing.Optional[_identifier.WorkflowExecutionIdentifier] = None
-        working_dir: typing.Optional[utils.AutoDeletingTempDir] = None
+        working_dir: typing.Optional[str] = None
         checkpoint: typing.Optional[Checkpoint] = None
         execution_date: typing.Optional[datetime] = None
         logging: Optional[_logging.Logger] = None
         task_id: typing.Optional[_identifier.Identifier] = None
+        output_metadata_prefix: Optional[str] = None
+        enable_deck: bool = False
 
         def __init__(self, current: typing.Optional[ExecutionParameters] = None):
             self.stats = current.stats if current else None
@@ -101,14 +99,16 @@ class ExecutionParameters(object):
             self.attrs = current._attrs if current else {}
             self.raw_output_prefix = current.raw_output_prefix if current else None
             self.task_id = current.task_id if current else None
+            self.output_metadata_prefix = current.output_metadata_prefix if current else None
+            self.enable_deck = current.enable_deck if current else False
 
         def add_attr(self, key: str, v: typing.Any) -> ExecutionParameters.Builder:
             self.attrs[key] = v
             return self
 
         def build(self) -> ExecutionParameters:
-            if not isinstance(self.working_dir, utils.AutoDeletingTempDir):
-                pathlib.Path(self.working_dir).mkdir(parents=True, exist_ok=True)
+            if self.working_dir and not isinstance(self.working_dir, utils.AutoDeletingTempDir):
+                pathlib.Path(typing.cast(str, self.working_dir)).mkdir(parents=True, exist_ok=True)
             return ExecutionParameters(
                 execution_date=self.execution_date,
                 stats=self.stats,
@@ -119,20 +119,22 @@ class ExecutionParameters(object):
                 decks=self.decks,
                 raw_output_prefix=self.raw_output_prefix,
                 task_id=self.task_id,
+                output_metadata_prefix=self.output_metadata_prefix,
+                enable_deck=self.enable_deck,
                 **self.attrs,
             )
 
     @staticmethod
-    def new_builder(current: ExecutionParameters = None) -> Builder:
+    def new_builder(current: Optional[ExecutionParameters] = None) -> Builder:
         return ExecutionParameters.Builder(current=current)
 
     def with_task_sandbox(self) -> Builder:
         prefix = self.working_directory
         if isinstance(self.working_directory, utils.AutoDeletingTempDir):
             prefix = self.working_directory.name
-        task_sandbox_dir = tempfile.mkdtemp(prefix=prefix)
+        task_sandbox_dir = tempfile.mkdtemp(prefix=prefix)  # type: ignore
         p = pathlib.Path(task_sandbox_dir)
-        cp_dir = p.joinpath("__cp")
+        cp_dir = p / "__cp"
         cp_dir.mkdir(exist_ok=True)
         cp = SyncCheckpoint(checkpoint_dest=str(cp_dir))
         b = self.new_builder(self)
@@ -140,9 +142,29 @@ class ExecutionParameters(object):
         b.working_dir = task_sandbox_dir
         return b
 
+    def with_enable_deck(self, enable_deck: bool) -> Builder:
+        b = self.new_builder(self)
+        b.enable_deck = enable_deck
+        return b
+
     def builder(self) -> Builder:
         return ExecutionParameters.Builder(current=self)
 
+    def __repr__(self):
+        cp = f" checkpoint={self.checkpoint}," if self._checkpoint else ""
+        return (
+            f"ExecutionParameters(execution_date={self.execution_date},"
+            f" stats={self.stats},"
+            f" tmp_dir={self.working_directory},"
+            f" execution_id={self.execution_id},"
+            f" {cp},"
+            f" decks={self.decks},"
+            f" raw_output_prefix={self.raw_output_prefix},"
+            f" task_id={self.task_id},"
+            f" output_metadata_prefix={self.output_metadata_prefix},"
+            f" enable_deck={self.enable_deck})"
+        )
+
     def __init__(
         self,
         execution_date,
@@ -151,9 +173,11 @@ class ExecutionParameters(object):
         execution_id: typing.Optional[_identifier.WorkflowExecutionIdentifier],
         logging,
         raw_output_prefix,
+        output_metadata_prefix=None,
         checkpoint=None,
         decks=None,
         task_id: typing.Optional[_identifier.Identifier] = None,
+        enable_deck: bool = False,
         **kwargs,
     ):
         """
@@ -173,6 +197,7 @@ class ExecutionParameters(object):
         self._execution_id = execution_id
         self._logging = logging
         self._raw_output_prefix = raw_output_prefix
+        self._output_metadata_prefix = output_metadata_prefix
         # AutoDeletingTempDir's should be used with a with block, which creates upon entry
         self._attrs = kwargs
         # It is safe to recreate the Secrets Manager
@@ -180,6 +205,8 @@ class ExecutionParameters(object):
         self._checkpoint = checkpoint
         self._decks = decks
         self._task_id = task_id
+        self._timeline_deck = None
+        self._enable_deck = enable_deck
 
     @property
     def stats(self) -> taggable.TaggableStats:
@@ -202,12 +229,14 @@ class ExecutionParameters(object):
         return self._raw_output_prefix
 
     @property
-    def working_directory(self) -> utils.AutoDeletingTempDir:
+    def output_metadata_prefix(self) -> str:
+        return self._output_metadata_prefix
+
+    @property
+    def working_directory(self) -> str:
         """
         A handle to a special working directory for easily producing temporary files.
-
         TODO: Usage examples
-        TODO: This does not always return a AutoDeletingTempDir
         """
         return self._working_directory
 
@@ -217,9 +246,8 @@ class ExecutionParameters(object):
         This is a datetime representing the time at which a workflow was started.  This is consistent across all tasks
         executed in a workflow or sub-workflow.
 
-        .. note::
-
-            Do NOT use this execution_date to drive any production logic.  It might be useful as a tag for data to help
+        > [!NOTE]
+        > Do NOT use this execution_date to drive any production logic.  It might be useful as a tag for data to help
             in debugging.
         """
         return self._execution_date
@@ -230,9 +258,8 @@ class ExecutionParameters(object):
         This is the identifier of the workflow execution within the underlying engine.  It will be consistent across all
         task executions in a workflow or sub-workflow execution.
 
-        .. note::
-
-            Do NOT use this execution_id to drive any production logic.  This execution ID should only be used as a tag
+        > [!NOTE]
+        > Do NOT use this execution_id to drive any production logic.  This execution ID should only be used as a tag
             on output data to link back to the workflow run that created it.
         """
         return self._execution_id
@@ -264,10 +291,35 @@ class ExecutionParameters(object):
 
     @property
     def default_deck(self) -> Deck:
-        from flytekit.deck.deck import Deck
+        from flytekit import Deck
 
         return Deck("default")
 
+    @property
+    def timeline_deck(self) -> "TimeLineDeck":  # type: ignore
+        from flytekit.deck.deck import DeckField, TimeLineDeck
+
+        time_line_deck = None
+        for deck in self.decks:
+            if isinstance(deck, TimeLineDeck):
+                time_line_deck = deck
+                break
+        if time_line_deck is None:
+            if self._timeline_deck is not None:
+                time_line_deck = self._timeline_deck
+            else:
+                time_line_deck = TimeLineDeck(DeckField.TIMELINE.value, auto_add_to_deck=False)
+
+        self._timeline_deck = time_line_deck
+        return time_line_deck
+
+    @property
+    def enable_deck(self) -> bool:
+        """
+        Returns whether deck is enabled or not
+        """
+        return self._enable_deck
+
     def __getattr__(self, attr_name: str) -> typing.Any:
         """
         This houses certain task specific context. For example in Spark, it houses the SparkSession, etc
@@ -287,7 +339,7 @@ class ExecutionParameters(object):
         """
         Returns task specific context if present else raise an error. The returned context will match the key
         """
-        return self.__getattr__(attr_name=key)
+        return self.__getattr__(attr_name=key)  # type: ignore
 
 
 class SecretsManager(object):
@@ -321,6 +373,7 @@ class SecretsManager(object):
     def __init__(self, secrets_cfg: typing.Optional[SecretsConfig] = None):
         if secrets_cfg is None:
             secrets_cfg = SecretsConfig.auto()
+
         self._base_dir = secrets_cfg.default_dir.strip()
         self._file_prefix = secrets_cfg.file_prefix.strip()
         self._env_prefix = secrets_cfg.env_prefix.strip()
@@ -331,43 +384,68 @@ class SecretsManager(object):
         """
         return self._GroupSecrets(item, self)
 
-    def get(self, group: str, key: str) -> str:
+    def get(
+        self,
+        group: Optional[str] = None,
+        key: Optional[str] = None,
+        group_version: Optional[str] = None,
+        encode_mode: str = "r",
+    ) -> str:
         """
         Retrieves a secret using the resolution order -> Env followed by file. If not found raises a ValueError
+        param encode_mode, defines the mode to open files, it can either be "r" to read file, or "rb" to read binary file
         """
-        self.check_group_key(group, key)
-        env_var = self.get_secrets_env_var(group, key)
-        fpath = self.get_secrets_file(group, key)
-        v = os.environ.get(env_var)
-        if v is not None:
-            return v
+        env_prefixes = [self._env_prefix]
+
+        # During local execution check for the key without a prefix
+        ctx = FlyteContextManager.current_context()
+        if ctx.execution_state is None or ctx.execution_state.is_local_execution():
+            env_prefixes.append("")
+
+        for env_prefix in env_prefixes:
+            env_var = self._get_secrets_env_var(
+                group=group, key=key, group_version=group_version, env_prefix=env_prefix
+            )
+            v = os.environ.get(env_var)
+            if v is not None:
+                return v.strip()
+
+        fpath = self.get_secrets_file(group, key, group_version)
         if os.path.exists(fpath):
-            with open(fpath, "r") as f:
+            with open(fpath, encode_mode) as f:
                 return f.read().strip()
         raise ValueError(
-            f"Unable to find secret for key {key} in group {group} " f"in Env Var:{env_var} and FilePath: {fpath}"
+            f"Please make sure to add secret_requests=[Secret(group={group}, key={key})] in @task. Unable to find secret for key {key} in group {group} "
+            f"in Env Var:{env_var} and FilePath: {fpath}"
         )
 
-    def get_secrets_env_var(self, group: str, key: str) -> str:
+    def get_secrets_env_var(
+        self, group: Optional[str] = None, key: Optional[str] = None, group_version: Optional[str] = None
+    ) -> str:
         """
         Returns a string that matches the ENV Variable to look for the secrets
         """
-        self.check_group_key(group, key)
-        return f"{self._env_prefix}{group.upper()}_{key.upper()}"
+        return self._get_secrets_env_var(group=group, key=key, group_version=group_version, env_prefix=self._env_prefix)
 
-    def get_secrets_file(self, group: str, key: str) -> str:
+    def _get_secrets_env_var(
+        self,
+        group: Optional[str] = None,
+        key: Optional[str] = None,
+        group_version: Optional[str] = None,
+        env_prefix: str = "",
+    ):
+        l = [k.upper() for k in filter(None, (group, group_version, key))]
+        return f"{env_prefix}{'_'.join(l)}"
+
+    def get_secrets_file(
+        self, group: Optional[str] = None, key: Optional[str] = None, group_version: Optional[str] = None
+    ) -> str:
         """
         Returns a path that matches the file to look for the secrets
         """
-        self.check_group_key(group, key)
-        return os.path.join(self._base_dir, group.lower(), f"{self._file_prefix}{key.lower()}")
-
-    @staticmethod
-    def check_group_key(group: str, key: str):
-        if group is None or group == "":
-            raise ValueError("secrets group is a mandatory field.")
-        if key is None or key == "":
-            raise ValueError("secrets key is a mandatory field.")
+        l = [k.lower() for k in filter(None, (group, group_version, key))]
+        l[-1] = f"{self._file_prefix}{l[-1]}"
+        return os.path.join(self._base_dir, *l)
 
 
 @dataclass(frozen=True)
@@ -380,8 +458,8 @@ class CompilationState(object):
         prefix (str): This is because we may one day want to be able to have subworkflows inside other workflows. If
             users choose to not specify their node names, then we can end up with multiple "n0"s. This prefix allows
             us to give those nested nodes a distinct name, as well as properly identify them in the workflow.
-        mode (int): refer to :py:class:`flytekit.extend.ExecutionState.Mode`
-        task_resolver (Optional[TaskResolverMixin]): Please see :py:class:`flytekit.extend.TaskResolverMixin`
+        mode (int): refer to `flytekit.extend.ExecutionState.Mode`
+        task_resolver (Optional[TaskResolverMixin]): Please see `flytekit.extend.TaskResolverMixin`
         nodes (Optional[List]): Stores currently compiled nodes so far.
     """
 
@@ -466,15 +544,24 @@ class ExecutionState(object):
         # or propeller.
         LOCAL_TASK_EXECUTION = 3
 
+        # This is the mode that is used to indicate a dynamic task
+        DYNAMIC_TASK_EXECUTION = 4
+
+        EAGER_EXECUTION = 5
+
+        EAGER_LOCAL_EXECUTION = 6
+
+        LOCAL_DYNAMIC_TASK_EXECUTION = 7
+
     mode: Optional[ExecutionState.Mode]
-    working_dir: os.PathLike
+    working_dir: Union[os.PathLike, str]
     engine_dir: Optional[Union[os.PathLike, str]]
     branch_eval_mode: Optional[BranchEvalMode]
     user_space_params: Optional[ExecutionParameters]
 
     def __init__(
         self,
-        working_dir: os.PathLike,
+        working_dir: Union[os.PathLike, str],
         mode: Optional[ExecutionState.Mode] = None,
         engine_dir: Optional[Union[os.PathLike, str]] = None,
         branch_eval_mode: Optional[BranchEvalMode] = None,
@@ -522,6 +609,66 @@ class ExecutionState(object):
             user_space_params=user_space_params if user_space_params else self.user_space_params,
         )
 
+    def is_local_execution(self) -> bool:
+        return (
+            self.mode == ExecutionState.Mode.LOCAL_TASK_EXECUTION
+            or self.mode == ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION
+            or self.mode == ExecutionState.Mode.EAGER_LOCAL_EXECUTION
+            or self.mode == ExecutionState.Mode.LOCAL_DYNAMIC_TASK_EXECUTION
+        )
+
+
+class SerializableToString(typing.Protocol):
+    """
+    This protocol is used by the Artifact create_from function. Basically these objects are serialized when running,
+    and then added to a literal's metadata.
+    """
+
+    def serialize_to_string(self, ctx: FlyteContext, variable_name: str) -> typing.Tuple[str, str]: ...
+
+
+@dataclass
+class OutputMetadata(object):
+    artifact: "Artifact"  # type: ignore[name-defined]
+    # I would simplify this to be called partitions
+    # and add a separate field called time_partition
+    dynamic_partitions: Optional[typing.Dict[str, str]]
+    time_partition: Optional[datetime] = None
+    additional_items: Optional[typing.List[SerializableToString]] = None
+
+
+TaskOutputMetadata = typing.Dict[typing.Any, OutputMetadata]
+
+
+@dataclass
+class OutputMetadataTracker(object):
+    """
+    This class is for the users to set arbitrary metadata on output literals.
+
+    Attributes:
+        output_metadata Optional[TaskOutputMetadata]: is a sparse dictionary of metadata that the user wants to attach
+            to each output of a task. The key is the output value (object) and the value is an OutputMetadata object.
+    """
+
+    output_metadata: typing.Dict[typing.Any, OutputMetadata] = field(default_factory=dict)
+
+    def add(self, obj: typing.Any, metadata: OutputMetadata):
+        self.output_metadata[id(obj)] = metadata
+
+    def get(self, obj: typing.Any) -> Optional[OutputMetadata]:
+        return self.output_metadata.get(id(obj))
+
+    def with_params(
+        self,
+        output_metadata: Optional[TaskOutputMetadata] = None,
+    ) -> OutputMetadataTracker:
+        """
+        Produces a copy of the current object and set new things
+        """
+        return OutputMetadataTracker(
+            output_metadata=output_metadata if output_metadata else self.output_metadata,
+        )
+
 
 @dataclass(frozen=True)
 class FlyteContext(object):
@@ -531,19 +678,21 @@ class FlyteContext(object):
     compile workflows, serialize Flyte entities, etc.
 
     Even though this object as a ``current_context`` function on it, it should not be called directly. Please use the
-    :py:class:`flytekit.FlyteContextManager` object instead.
+    `flytekit.FlyteContextManager` object instead.
 
-    Please do not confuse this object with the :py:class:`flytekit.ExecutionParameters` object.
+    Please do not confuse this object with the `flytekit.ExecutionParameters` object.
     """
 
     file_access: FileAccessProvider
     level: int = 0
-    flyte_client: Optional[friendly_client.SynchronousFlyteClient] = None
+    flyte_client: Optional["friendly_client.SynchronousFlyteClient"] = None
     compilation_state: Optional[CompilationState] = None
     execution_state: Optional[ExecutionState] = None
     serialization_settings: Optional[SerializationSettings] = None
     in_a_condition: bool = False
     origin_stackframe: Optional[traceback.FrameSummary] = None
+    output_metadata_tracker: Optional[OutputMetadataTracker] = None
+    worker_queue: Optional[Controller] = None
 
     @property
     def user_space_params(self) -> Optional[ExecutionParameters]:
@@ -569,6 +718,8 @@ class FlyteContext(object):
             compilation_state=self.compilation_state,
             execution_state=self.execution_state,
             in_a_condition=self.in_a_condition,
+            output_metadata_tracker=self.output_metadata_tracker,
+            worker_queue=self.worker_queue,
         )
 
     def enter_conditional_section(self) -> Builder:
@@ -590,12 +741,23 @@ class FlyteContext(object):
     def with_serialization_settings(self, ss: SerializationSettings) -> Builder:
         return self.new_builder().with_serialization_settings(ss)
 
+    def with_output_metadata_tracker(self, t: OutputMetadataTracker) -> Builder:
+        return self.new_builder().with_output_metadata_tracker(t)
+
+    def with_worker_queue(self, wq: Controller) -> Builder:
+        return self.new_builder().with_worker_queue(wq)
+
+    def with_client(self, c: SynchronousFlyteClient) -> Builder:
+        return self.new_builder().with_client(c)
+
     def new_compilation_state(self, prefix: str = "") -> CompilationState:
         """
         Creates and returns a default compilation state. For most of the code this should be the entrypoint
         of compilation, otherwise the code should always uses - with_compilation_state
         """
-        return CompilationState(prefix=prefix)
+        from flytekit.core.python_auto_container import default_task_resolver
+
+        return CompilationState(prefix=prefix, task_resolver=default_task_resolver)
 
     def new_execution_state(self, working_dir: Optional[os.PathLike] = None) -> ExecutionState:
         """
@@ -607,13 +769,13 @@ class FlyteContext(object):
         return ExecutionState(working_dir=working_dir, user_space_params=self.user_space_params)
 
     @staticmethod
-    def current_context() -> Optional[FlyteContext]:
+    def current_context() -> FlyteContext:
         """
         This method exists only to maintain backwards compatibility. Please use
         ``FlyteContextManager.current_context()`` instead.
 
         Users of flytekit should be wary not to confuse the object returned from this function
-        with :py:func:`flytekit.current_context`
+        with {{< py_func_ref flytekit.current_context >}}
         """
         return FlyteContextManager.current_context()
 
@@ -624,22 +786,22 @@ class FlyteContext(object):
         The return value depends on the execution environment. In a notebook, the return value is compatible with
         IPython.display and should be rendered in the notebook.
 
-        .. code-block:: python
+        ```python
+        with flytekit.new_context() as ctx:
+            my_task(...)
+        ctx.get_deck()
+        ```
 
-            with flytekit.new_context() as ctx:
-                my_task(...)
-            ctx.get_deck()
+        OR if you wish to explicitly display
 
-        OR if you wish to explicity display
-
-        .. code-block:: python
-
-            from IPython import display
-            display(ctx.get_deck())
+        ```python
+        from IPython import display
+        display(ctx.get_deck())
+        ```
         """
         from flytekit.deck.deck import _get_deck
 
-        return _get_deck(self.execution_state.user_space_params)
+        return _get_deck(typing.cast(ExecutionState, self.execution_state).user_space_params)
 
     @dataclass
     class Builder(object):
@@ -647,9 +809,11 @@ class FlyteContext(object):
         level: int = 0
         compilation_state: Optional[CompilationState] = None
         execution_state: Optional[ExecutionState] = None
-        flyte_client: Optional[friendly_client.SynchronousFlyteClient] = None
+        flyte_client: Optional["friendly_client.SynchronousFlyteClient"] = None
         serialization_settings: Optional[SerializationSettings] = None
         in_a_condition: bool = False
+        output_metadata_tracker: Optional[OutputMetadataTracker] = None
+        worker_queue: Optional[Controller] = None
 
         def build(self) -> FlyteContext:
             return FlyteContext(
@@ -660,6 +824,8 @@ class FlyteContext(object):
                 flyte_client=self.flyte_client,
                 serialization_settings=self.serialization_settings,
                 in_a_condition=self.in_a_condition,
+                output_metadata_tracker=self.output_metadata_tracker,
+                worker_queue=self.worker_queue,
             )
 
         def enter_conditional_section(self) -> FlyteContext.Builder:
@@ -671,7 +837,7 @@ class FlyteContext(object):
                 self.compilation_state = self.compilation_state.with_params(prefix=self.compilation_state.prefix)
 
             if self.execution_state:
-                if self.execution_state.mode == ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION:
+                if self.execution_state.is_local_execution():
                     if self.in_a_condition:
                         if self.execution_state.branch_eval_mode == BranchEvalMode.BRANCH_SKIPPED:
                             self.execution_state = self.execution_state.with_params()
@@ -704,6 +870,18 @@ class FlyteContext(object):
             self.serialization_settings = ss
             return self
 
+        def with_output_metadata_tracker(self, t: OutputMetadataTracker) -> FlyteContext.Builder:
+            self.output_metadata_tracker = t
+            return self
+
+        def with_worker_queue(self, wq: Controller) -> FlyteContext.Builder:
+            self.worker_queue = wq
+            return self
+
+        def with_client(self, c: SynchronousFlyteClient) -> FlyteContext.Builder:
+            self.flyte_client = c
+            return self
+
         def new_compilation_state(self, prefix: str = "") -> CompilationState:
             """
             Creates and returns a default compilation state. For most of the code this should be the entrypoint
@@ -726,22 +904,28 @@ class FlyteContextManager(object):
     FlyteContextManager manages the execution context within Flytekit. It holds global state of either compilation
     or Execution. It is not thread-safe and can only be run as a single threaded application currently.
     Context's within Flytekit is useful to manage compilation state and execution state. Refer to ``CompilationState``
-    and ``ExecutionState`` for for information. FlyteContextManager provides a singleton stack to manage these contexts.
+    and ``ExecutionState`` for more information. FlyteContextManager provides a singleton stack to manage these contexts.
 
     Typical usage is
 
-    .. code-block:: python
+    ```python
+    FlyteContextManager.initialize()
+    with FlyteContextManager.with_context(o) as ctx:
+        pass
 
-        FlyteContextManager.initialize()
-        with FlyteContextManager.with_context(o) as ctx:
-          pass
-
-        # If required - not recommended you can use
-        FlyteContextManager.push_context()
-        # but correspondingly a pop_context should be called
-        FlyteContextManager.pop_context()
+    # If required - not recommended you can use
+    FlyteContextManager.push_context()
+    # but correspondingly a pop_context should be called
+    FlyteContextManager.pop_context()
+    ```
     """
 
+    signal_handlers: typing.List[typing.Callable[[int, FrameType], typing.Any]] = []
+
+    @staticmethod
+    def add_signal_handler(handler: typing.Callable[[int, FrameType], typing.Any]):
+        FlyteContextManager.signal_handlers.append(handler)
+
     @staticmethod
     def get_origin_stackframe(limit=2) -> traceback.FrameSummary:
         ss = traceback.extract_stack(limit=limit + 1)
@@ -765,7 +949,7 @@ class FlyteContextManager(object):
         context_list.append(ctx)
         flyte_context_Var.set(context_list)
         t = "\t"
-        logger.debug(
+        developer_logger.debug(
             f"{t * ctx.level}[{len(flyte_context_Var.get())}] Pushing context - {'compile' if ctx.compilation_state else 'execute'}, branch[{ctx.in_a_condition}], {ctx.get_origin_stackframe_repr()}"
         )
         return ctx
@@ -776,7 +960,7 @@ class FlyteContextManager(object):
         ctx = context_list.pop()
         flyte_context_Var.set(context_list)
         t = "\t"
-        logger.debug(
+        developer_logger.debug(
             f"{t * ctx.level}[{len(flyte_context_Var.get()) + 1}] Popping context - {'compile' if ctx.compilation_state else 'execute'}, branch[{ctx.in_a_condition}], {ctx.get_origin_stackframe_repr()}"
         )
         if len(flyte_context_Var.get()) == 0:
@@ -825,13 +1009,23 @@ class FlyteContextManager(object):
         user_space_path = os.path.join(cfg.local_sandbox_path, "user_space")
         pathlib.Path(user_space_path).mkdir(parents=True, exist_ok=True)
 
+        def main_signal_handler(signum: int, frame: FrameType):
+            for handler in FlyteContextManager.signal_handlers:
+                handler(signum, frame)
+            exit(1)
+
+        # This initialize function is also called by other threads (since the context manager lives in a ContextVar)
+        # so we should not run this if we're not the main thread.
+        if threading.current_thread().name == threading.main_thread().name:
+            signal.signal(signal.SIGINT, main_signal_handler)
+
         # Note we use the SdkWorkflowExecution object purely for formatting into the ex:project:domain:name format users
         # are already acquainted with
         default_context = FlyteContext(file_access=default_local_file_access_provider)
         default_user_space_params = ExecutionParameters(
             execution_id=WorkflowExecutionIdentifier.promote_from_model(default_execution_id),
             task_id=_identifier.Identifier(_identifier.ResourceType.TASK, "local", "local", "local", "local"),
-            execution_date=_datetime.datetime.utcnow(),
+            execution_date=datetime.now(timezone.utc),
             stats=mock_stats.MockStats(),
             logging=user_space_logger,
             tmp_dir=user_space_path,
@@ -852,7 +1046,7 @@ class FlyteEntities(object):
      registration process
     """
 
-    entities = []
+    entities: List[Union["LaunchPlan", Task, "WorkflowBase"]] = []  # type: ignore
 
 
 FlyteContextManager.initialize()
