@@ -6,13 +6,16 @@ from typing import Tuple, Union
 
 import click
 
+from flytekit.core import constants
 from flytekit.core import interface as flyte_interface
-from flytekit.core.context_manager import FlyteContext, FlyteContextManager
+from flytekit.core.context_manager import ExecutionState, FlyteContext, FlyteContextManager
 from flytekit.core.promise import Promise, VoidPromise, flyte_entity_call_handler
 from flytekit.core.type_engine import TypeEngine
 from flytekit.exceptions.user import FlyteDisapprovalException
 from flytekit.interaction.parse_stdin import parse_stdin_to_literal
+from flytekit.interaction.string_literals import scalar_to_string
 from flytekit.models.core import workflow as _workflow_model
+from flytekit.models.literals import Scalar
 from flytekit.models.types import LiteralType
 
 DEFAULT_TIMEOUT = datetime.timedelta(hours=1)
@@ -53,7 +56,7 @@ class Gate(object):
             )
         else:
             # We don't know how to find the python interface here, approve() sets it below, See the code.
-            self._python_interface = None
+            self._python_interface = None  # type: ignore
 
     @property
     def name(self) -> str:
@@ -94,18 +97,28 @@ class Gate(object):
     # This is to satisfy the LocallyExecutable protocol
     def local_execute(self, ctx: FlyteContext, **kwargs) -> Union[Tuple[Promise], Promise, VoidPromise]:
         if self.sleep_duration:
-            print(f"Mock sleeping for {self.sleep_duration}")
+            click.echo(
+                f'{click.style("[Sleep Gate]", fg="yellow")} '
+                f'{click.style(f"Simulating Sleep for {self.sleep_duration}", fg="cyan")}'
+            )
             return VoidPromise(self.name)
 
         # Trigger stdin
         if self.input_type:
-            msg = f"Execution stopped for gate {self.name}...\n"
+            msg = click.style("[Input Gate] ", fg="yellow") + click.style(
+                f"Waiting for input @{self.name} of type {self.input_type}", fg="cyan"
+            )
             literal = parse_stdin_to_literal(ctx, self.input_type, msg)
             p = Promise(var="o0", val=literal)
             return p
 
         # Assume this is an approval operation since that's the only remaining option.
-        msg = f"Pausing execution for {self.name}, literal value is:\n{self._upstream_item.val}\nContinue?"
+        v = typing.cast(Promise, self._upstream_item).val.value
+        if isinstance(v, Scalar):
+            v = scalar_to_string(v)
+        msg = click.style("[Approval Gate] ", fg="yellow") + click.style(
+            f"@{self.name} Approve {click.style(v, fg='green')}?", fg="cyan"
+        )
         proceed = click.confirm(msg, default=True)
         if proceed:
             # We need to return a promise here, and a promise is what should've been passed in by the call in approve()
@@ -116,9 +129,13 @@ class Gate(object):
         else:
             raise FlyteDisapprovalException(f"User did not approve the transaction for gate node {self.name}")
 
+    def local_execution_mode(self):
+        return ExecutionState.Mode.LOCAL_TASK_EXECUTION
+
 
 def wait_for_input(name: str, timeout: datetime.timedelta, expected_type: typing.Type):
-    """
+    """Create a Gate object that waits for user input of the specified type.
+
     Create a Gate object. This object will function like a task. Note that unlike a task,
     each time this function is called, a new Python object is created. If a workflow
     calls a subworkflow twice, and the subworkflow has a signal, then two Gate
@@ -136,7 +153,8 @@ def wait_for_input(name: str, timeout: datetime.timedelta, expected_type: typing
 
 
 def sleep(duration: datetime.timedelta):
-    """
+    """Create a sleep Gate object.
+
     :param duration: How long to sleep for
     :return:
     """
@@ -146,7 +164,8 @@ def sleep(duration: datetime.timedelta):
 
 
 def approve(upstream_item: Union[Tuple[Promise], Promise, VoidPromise], name: str, timeout: datetime.timedelta):
-    """
+    """Create a Gate object for binary approval.
+
     Create a Gate object. This object will function like a task. Note that unlike a task,
     each time this function is called, a new Python object is created. If a workflow
     calls a subworkflow twice, and the subworkflow has a signal, then two Gate
@@ -164,7 +183,10 @@ def approve(upstream_item: Union[Tuple[Promise], Promise, VoidPromise], name: st
         raise ValueError("You can't use approval on a task that doesn't return anything.")
 
     ctx = FlyteContextManager.current_context()
+    upstream_item = typing.cast(Promise, upstream_item)
     if ctx.compilation_state is not None and ctx.compilation_state.mode == 1:
+        if upstream_item.ref.node_id == constants.GLOBAL_INPUT_NODE_ID:
+            raise ValueError("Workflow inputs cannot be passed to approval nodes.")
         if not upstream_item.ref.node.flyte_entity.python_interface:
             raise ValueError(
                 f"Upstream node doesn't have a Python interface. Node entity is: "
