@@ -3,24 +3,66 @@ from __future__ import annotations
 import collections
 import copy
 import inspect
+import sys
 import typing
 from collections import OrderedDict
-from typing import Any, Dict, Generator, List, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    Any,
+    Dict,
+    Generator,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
-from typing_extensions import Annotated, get_args, get_origin, get_type_hints
+from flyteidl.core import artifact_id_pb2 as art_id
+from typing_extensions import get_args, get_type_hints
 
 from flytekit.core import context_manager
+from flytekit.core.artifact import Artifact, ArtifactIDSpecification, ArtifactQuery
+from flytekit.core.context_manager import FlyteContextManager
 from flytekit.core.docstring import Docstring
-from flytekit.core.type_engine import TypeEngine
-from flytekit.exceptions.user import FlyteValidationException
-from flytekit.loggers import logger
+from flytekit.core.sentinel import DYNAMIC_INPUT_BINDING
+from flytekit.core.type_engine import TypeEngine, UnionTransformer
+from flytekit.core.utils import has_return_statement
+from flytekit.exceptions.user import (
+    FlyteMissingReturnValueException,
+    FlyteMissingTypeException,
+    FlyteValidationException,
+)
+from flytekit.loggers import developer_logger, logger
 from flytekit.models import interface as _interface_models
-from flytekit.models.literals import Void
-from flytekit.types.pickle import FlytePickle
+from flytekit.models.literals import Literal, Scalar, Void
 
 T = typing.TypeVar("T")
 
 
+def repr_kv(k: str, v: Union[Type, Tuple[Type, Any]]) -> str:
+    if isinstance(v, tuple):
+        if v[1] is not None:
+            return f"{k}: {v[0]}={v[1]}"
+        return f"{k}: {v[0]}"
+    return f"{k}: {v}"
+
+
+def repr_type_signature(io: Union[Dict[str, Tuple[Type, Any]], Dict[str, Type]]) -> str:
+    """
+    Converts an inputs and outputs to a type signature
+    """
+    s = "("
+    i = 0
+    for k, v in io.items():
+        if i > 0:
+            s += ", "
+        s += repr_kv(k, v)
+        i = i + 1
+    return s + ")"
+
+
 class Interface(object):
     """
     A Python native interface object, like inspect.signature but simpler.
@@ -28,8 +70,8 @@ class Interface(object):
 
     def __init__(
         self,
-        inputs: typing.Optional[typing.Dict[str, Union[Type, Tuple[Type, Any]], None]] = None,
-        outputs: typing.Optional[typing.Dict[str, Type]] = None,
+        inputs: Union[Optional[Dict[str, Type]], Optional[Dict[str, Tuple[Type, Any]]]] = None,
+        outputs: Union[Optional[Dict[str, Type]], Optional[Dict[str, Optional[Type]]]] = None,
         output_tuple_name: Optional[str] = None,
         docstring: Optional[Docstring] = None,
     ):
@@ -43,21 +85,25 @@ class Interface(object):
             primarily used when handling one-element NamedTuples.
         :param docstring: Docstring of the annotated @task or @workflow from which the interface derives from.
         """
-        self._inputs = {}
+        self._inputs: Union[Dict[str, Tuple[Type, Any]], Dict[str, Type]] = {}  # type: ignore
         if inputs:
             for k, v in inputs.items():
-                if isinstance(v, Tuple) and len(v) > 1:
-                    self._inputs[k] = v
+                if not k.isidentifier():
+                    raise ValueError(f"Input name must be a valid Python identifier: {k!r}")
+                if type(v) is tuple and len(cast(Tuple, v)) > 1:
+                    self._inputs[k] = v  # type: ignore
                 else:
-                    self._inputs[k] = (v, None)
-        self._outputs = outputs if outputs else {}
+                    self._inputs[k] = (v, None)  # type: ignore
+        self._outputs = outputs if outputs else {}  # type: ignore
         self._output_tuple_name = output_tuple_name
 
         if outputs:
             variables = [k for k in outputs.keys()]
 
             # TODO: This class is a duplicate of the one in create_task_outputs. Over time, we should move to this one.
-            class Output(collections.namedtuple(output_tuple_name or "DefaultNamedTupleOutput", variables)):
+            class Output(  # type: ignore
+                collections.namedtuple(output_tuple_name or "DefaultNamedTupleOutput", variables)  # type: ignore
+            ):  # type: ignore
                 """
                 This class can be used in two different places. For multivariate-return entities this class is used
                 to rewrap the outputs so that our with_overrides function can work.
@@ -83,14 +129,13 @@ class Interface(object):
                     where runs_before is manually called.
                     """
 
-                def __rshift__(self, *args, **kwargs):
-                    ...  # See runs_before
+                def __rshift__(self, *args, **kwargs): ...  # See runs_before
 
             self._output_tuple_class = Output
         self._docstring = docstring
 
     @property
-    def output_tuple(self) -> Optional[Type[collections.namedtuple]]:
+    def output_tuple(self) -> Type[collections.namedtuple]:  # type: ignore
         return self._output_tuple_class
 
     @property
@@ -98,7 +143,7 @@ class Interface(object):
         return self._output_tuple_name
 
     @property
-    def inputs(self) -> typing.Dict[str, Type]:
+    def inputs(self) -> Dict[str, type]:
         r = {}
         for k, v in self._inputs.items():
             r[k] = v[0]
@@ -111,8 +156,8 @@ class Interface(object):
         return None
 
     @property
-    def inputs_with_defaults(self) -> typing.Dict[str, Tuple[Type, Any]]:
-        return self._inputs
+    def inputs_with_defaults(self) -> Dict[str, Tuple[Type, Any]]:
+        return cast(Dict[str, Tuple[Type, Any]], self._inputs)
 
     @property
     def default_inputs_as_kwargs(self) -> Dict[str, Any]:
@@ -120,13 +165,13 @@ class Interface(object):
 
     @property
     def outputs(self) -> typing.Dict[str, type]:
-        return self._outputs
+        return self._outputs  # type: ignore
 
     @property
     def docstring(self) -> Optional[Docstring]:
         return self._docstring
 
-    def remove_inputs(self, vars: List[str]) -> Interface:
+    def remove_inputs(self, vars: Optional[List[str]]) -> Interface:
         """
         This method is useful in removing some variables from the Flyte backend inputs specification, as these are
         implicit local only inputs or will be supplied by the library at runtime. For example, spark-session etc
@@ -151,7 +196,7 @@ class Interface(object):
         for k, v in extra_inputs.items():
             if k in new_inputs:
                 raise ValueError(f"Input {k} cannot be added as it already exists in the interface")
-            new_inputs[k] = v
+            cast(Dict[str, Type], new_inputs)[k] = v
         return Interface(new_inputs, self._outputs, docstring=self.docstring)
 
     def with_outputs(self, extra_outputs: Dict[str, Type]) -> Interface:
@@ -167,12 +212,19 @@ class Interface(object):
             new_outputs[k] = v
         return Interface(self._inputs, new_outputs)
 
+    def __str__(self):
+        return f"{repr_type_signature(self._inputs)} -> {repr_type_signature(self._outputs)}"
+
+    def __repr__(self):
+        return str(self)
+
 
 def transform_inputs_to_parameters(
     ctx: context_manager.FlyteContext, interface: Interface
 ) -> _interface_models.ParameterMap:
     """
     Transforms the given interface (with inputs) to a Parameter Map with defaults set
+    :param ctx: context
     :param interface: the interface object
     """
     if interface is None or interface.inputs_with_defaults is None:
@@ -185,22 +237,36 @@ def transform_inputs_to_parameters(
     inputs_with_def = interface.inputs_with_defaults
     for k, v in inputs_vars.items():
         val, _default = inputs_with_def[k]
-        if _default is None and get_origin(val) is typing.Union and type(None) in get_args(val):
-            from flytekit import Literal, Scalar
-
+        if _default is None and UnionTransformer.is_optional_type(val):
             literal = Literal(scalar=Scalar(none_type=Void()))
             params[k] = _interface_models.Parameter(var=v, default=literal, required=False)
         else:
-            required = _default is None
-            default_lv = None
-            if _default is not None:
-                default_lv = TypeEngine.to_literal(ctx, _default, python_type=interface.inputs[k], expected=v.type)
-            params[k] = _interface_models.Parameter(var=v, default=default_lv, required=required)
+            if isinstance(_default, ArtifactQuery):
+                params[k] = _interface_models.Parameter(var=v, required=False, artifact_query=_default.to_flyte_idl())
+            elif isinstance(_default, Artifact):
+                if not _default.version:
+                    # If the artifact is not versioned, assume it's meant to be a query.
+                    q = _default.query()
+                    if q.bound:
+                        params[k] = _interface_models.Parameter(var=v, required=False, artifact_query=q.to_flyte_idl())
+                    else:
+                        raise FlyteValidationException(f"Cannot use default query with artifact {_default.name}")
+                else:
+                    # If it is versioned, assumed it's intentionally hard-coded
+                    artifact_id = _default.concrete_artifact_id  # may raise
+                    params[k] = _interface_models.Parameter(var=v, required=False, artifact_id=artifact_id)
+            else:
+                required = _default is None
+                default_lv = None
+                if _default is not None:
+                    default_lv = TypeEngine.to_literal(ctx, _default, python_type=interface.inputs[k], expected=v.type)
+                params[k] = _interface_models.Parameter(var=v, default=default_lv, required=required)
     return _interface_models.ParameterMap(params)
 
 
 def transform_interface_to_typed_interface(
     interface: typing.Optional[Interface],
+    allow_partial_artifact_id_binding: bool = False,
 ) -> typing.Optional[_interface_models.TypedInterface]:
     """
     Transform the given simple python native interface to FlyteIDL's interface
@@ -217,19 +283,71 @@ def transform_interface_to_typed_interface(
 
     inputs_map = transform_variable_map(interface.inputs, input_descriptions)
     outputs_map = transform_variable_map(interface.outputs, output_descriptions)
+    verify_outputs_artifact_bindings(interface.inputs, outputs_map, allow_partial_artifact_id_binding)
     return _interface_models.TypedInterface(inputs_map, outputs_map)
 
 
-def transform_types_to_list_of_type(m: Dict[str, type]) -> Dict[str, type]:
+def verify_outputs_artifact_bindings(
+    inputs: Dict[str, type],
+    outputs: Dict[str, _interface_models.Variable],
+    allow_partial_artifact_id_binding: bool = False,
+):
+    # collect Artifacts
+    for k, v in outputs.items():
+        # Iterate through output partition values if any and verify that if they're bound to an input, that that input
+        # actually exists in the interface.
+        if (
+            v.artifact_partial_id
+            and v.artifact_partial_id.HasField("partitions")
+            and v.artifact_partial_id.partitions.value
+        ):
+            for pk, pv in v.artifact_partial_id.partitions.value.items():
+                if pv == DYNAMIC_INPUT_BINDING:
+                    if not allow_partial_artifact_id_binding:
+                        raise FlyteValidationException(
+                            f"Binding a partition {pk}'s value dynamically is not allowed for workflows"
+                        )
+                    else:
+                        continue
+                if pv.HasField("input_binding"):
+                    input_name = pv.input_binding.var
+                    if input_name not in inputs:
+                        raise FlyteValidationException(
+                            f"Output partition {k} is bound to input {input_name} which does not exist in the interface"
+                        )
+            if v.artifact_partial_id.HasField("time_partition"):
+                if (
+                    v.artifact_partial_id.time_partition.value == DYNAMIC_INPUT_BINDING
+                    and not allow_partial_artifact_id_binding
+                ):
+                    raise FlyteValidationException(
+                        "Binding a time partition's value dynamically is not allowed for workflows"
+                    )
+
+                if v.artifact_partial_id.time_partition.value.HasField("input_binding"):
+                    input_name = v.artifact_partial_id.time_partition.value.input_binding.var
+                    if input_name not in inputs:
+                        raise FlyteValidationException(
+                            f"Output time partition is bound to input {input_name} which does not exist in the interface"
+                        )
+
+
+def transform_types_to_list_of_type(
+    m: Dict[str, type], bound_inputs: typing.Set[str], list_as_optional: bool = False
+) -> Dict[str, type]:
     """
-    Converts a given variables to be collections of their type. This is useful for array jobs / map style code.
-    It will create a collection of types even if any one these types is not a collection type
+    Converts unbound inputs into the equivalent (optional) collections. This is useful for array jobs / map style code.
+    It will create a collection of types even if any one these types is not a collection type.
     """
     if m is None:
         return {}
 
     all_types_are_collection = True
     for k, v in m.items():
+        if k in bound_inputs:
+            # Skip the inputs that are bound. If they are bound, it does not matter if they are collection or
+            # singletons
+            continue
         v_type = type(v)
         if v_type != typing.List and v_type != list:
             all_types_are_collection = False
@@ -240,45 +358,39 @@ def transform_types_to_list_of_type(m: Dict[str, type]) -> Dict[str, type]:
 
     om = {}
     for k, v in m.items():
-        om[k] = typing.List[v]
+        if k in bound_inputs:
+            om[k] = v
+        else:
+            om[k] = typing.List[typing.Optional[v] if list_as_optional else v]  # type: ignore
     return om  # type: ignore
 
 
-def transform_interface_to_list_interface(interface: Interface) -> Interface:
+def transform_interface_to_list_interface(
+    interface: Interface,
+    bound_inputs: typing.Set[str],
+    excluded_inputs: typing.Set[str],
+    optional_outputs: bool = False,
+) -> Interface:
     """
     Takes a single task interface and interpolates it to an array interface - to allow performing distributed python map
     like functions
+    :param interface: Interface to be upgraded to a list interface
+    :param bound_inputs: fixed inputs that should not be updated to a list and will be maintained as is
+    :param excluded_inputs: inputs that should be excluded from the interface such as fixed_inputs in a launch_plan
     """
-    map_inputs = transform_types_to_list_of_type(interface.inputs)
-    map_outputs = transform_types_to_list_of_type(interface.outputs)
+    filtered_inputs = {k: v for k, v in interface.inputs.items() if k not in excluded_inputs}
+    map_inputs = transform_types_to_list_of_type(filtered_inputs, bound_inputs)
+    map_outputs = transform_types_to_list_of_type(interface.outputs, set(), optional_outputs)
 
     return Interface(inputs=map_inputs, outputs=map_outputs)
 
 
-def _change_unrecognized_type_to_pickle(t: Type[T]) -> typing.Union[Tuple[Type[T]], Type[T], Annotated]:
-    try:
-        if hasattr(t, "__origin__") and hasattr(t, "__args__"):
-            if get_origin(t) is list:
-                return typing.List[_change_unrecognized_type_to_pickle(t.__args__[0])]
-            elif get_origin(t) is dict and t.__args__[0] == str:
-                return typing.Dict[str, _change_unrecognized_type_to_pickle(t.__args__[1])]
-            elif get_origin(t) is typing.Union:
-                return typing.Union[tuple(_change_unrecognized_type_to_pickle(v) for v in get_args(t))]
-            elif get_origin(t) is Annotated:
-                base_type, *config = get_args(t)
-                return Annotated[(_change_unrecognized_type_to_pickle(base_type), *config)]
-        TypeEngine.get_transformer(t)
-    except ValueError:
-        logger.warning(
-            f"Unsupported Type {t} found, Flyte will default to use PickleFile as the transport. "
-            f"Pickle can only be used to send objects between the exact same version of Python, "
-            f"and we strongly recommend to use python type that flyte support."
-        )
-        return FlytePickle[t]
-    return t
-
-
-def transform_function_to_interface(fn: typing.Callable, docstring: Optional[Docstring] = None) -> Interface:
+def transform_function_to_interface(
+    fn: typing.Callable,
+    docstring: Optional[Docstring] = None,
+    is_reference_entity: bool = False,
+    pickle_untyped: bool = False,
+) -> Interface:
     """
     From the annotations on a task function that the user should have provided, and the output names they want to use
     for each output parameter, construct the TypedInterface object
@@ -286,20 +398,44 @@ def transform_function_to_interface(fn: typing.Callable, docstring: Optional[Doc
     For now the fancy object, maybe in the future a dumb object.
 
     """
-
     type_hints = get_type_hints(fn, include_extras=True)
     signature = inspect.signature(fn)
     return_annotation = type_hints.get("return", None)
+    # If the return annotation is None and the pickle_untyped is True, we will use it as Any
+    if return_annotation is None and pickle_untyped:
+        return_annotation = Any
+
+    ctx = FlyteContextManager.current_context()
+
+    # Check if the function has a return statement at compile time locally.
+    # Skip it if the function is a reference task/workflow since it doesn't have a body.
+    if (
+        not is_reference_entity
+        and ctx.execution_state
+        and ctx.execution_state.mode is None
+        # inspect module does not work correctly with Python <3.10.10. https://github.com/flyteorg/flyte/issues/5608
+        and sys.version_info >= (3, 10, 10)
+        and return_annotation
+        and type(None) not in get_args(return_annotation)
+        and return_annotation is not type(None)
+        and has_return_statement(fn) is False
+    ):
+        raise FlyteMissingReturnValueException(fn=fn)
 
     outputs = extract_return_annotation(return_annotation)
     for k, v in outputs.items():
-        outputs[k] = _change_unrecognized_type_to_pickle(v)  # type: ignore
-    inputs = OrderedDict()
+        outputs[k] = v  # type: ignore
+    inputs: Dict[str, Tuple[Type, Any]] = OrderedDict()
     for k, v in signature.parameters.items():  # type: ignore
         annotation = type_hints.get(k, None)
+        if annotation is None:
+            if not pickle_untyped:
+                raise FlyteMissingTypeException(fn=fn, param_name=k)
+            # If the pickle_untyped is True, we will use it as Any
+            annotation = Any
         default = v.default if v.default is not inspect.Parameter.empty else None
         # Inputs with default values are currently ignored, we may want to look into that in the future
-        inputs[k] = (_change_unrecognized_type_to_pickle(annotation), default)
+        inputs[k] = (annotation, default)  # type: ignore
 
     # This is just for typing.NamedTuples - in those cases, the user can select a name to call the NamedTuple. We
     # would like to preserve that name in our custom collections.namedtuple.
@@ -315,34 +451,47 @@ def transform_function_to_interface(fn: typing.Callable, docstring: Optional[Doc
 
 def transform_variable_map(
     variable_map: Dict[str, type],
-    descriptions: Dict[str, str] = {},
+    descriptions: Optional[Dict[str, str]] = None,
 ) -> Dict[str, _interface_models.Variable]:
     """
     Given a map of str (names of inputs for instance) to their Python native types, return a map of the name to a
     Flyte Variable object with that type.
     """
     res = OrderedDict()
+    descriptions = descriptions or {}
     if variable_map:
         for k, v in variable_map.items():
             res[k] = transform_type(v, descriptions.get(k, k))
-            sub_type: Type[T] = v
-            if hasattr(v, "__origin__") and hasattr(v, "__args__"):
-                if v.__origin__ is list:
-                    sub_type = v.__args__[0]
-                elif v.__origin__ is dict:
-                    sub_type = v.__args__[1]
-            if hasattr(sub_type, "__origin__") and sub_type.__origin__ is FlytePickle:
-                if hasattr(sub_type.python_type(), "__name__"):
-                    res[k].type.metadata = {"python_class_name": sub_type.python_type().__name__}
-                elif hasattr(sub_type.python_type(), "_name"):
-                    # If the class doesn't have the __name__ attribute, like typing.Sequence, use _name instead.
-                    res[k].type.metadata = {"python_class_name": sub_type.python_type()._name}
-
     return res
 
 
-def transform_type(x: type, description: str = None) -> _interface_models.Variable:
-    return _interface_models.Variable(type=TypeEngine.to_literal_type(x), description=description)
+def detect_artifact(
+    ts: typing.Tuple[typing.Any, ...],
+) -> Optional[art_id.ArtifactID]:
+    """
+    If the user wishes to control how Artifacts are created (i.e. naming them, etc.) this is where we pick it up and
+    store it in the interface.
+    """
+    for t in ts:
+        if isinstance(t, Artifact):
+            id_spec = t()
+            return id_spec.to_partial_artifact_id()
+        elif isinstance(t, ArtifactIDSpecification):
+            artifact_id = t.to_partial_artifact_id()
+            return artifact_id
+
+    return None
+
+
+def transform_type(x: type, description: Optional[str] = None) -> _interface_models.Variable:
+    artifact_id = detect_artifact(get_args(x))
+    if artifact_id:
+        logger.debug(f"Found artifact id spec: {artifact_id}")
+    return _interface_models.Variable(
+        type=TypeEngine.to_literal_type(x),
+        description=description,
+        artifact_partial_id=artifact_id,
+    )
 
 
 def default_output_name(index: int = 0) -> str:
@@ -393,13 +542,15 @@ def extract_return_annotation(return_annotation: Union[Type, Tuple, None]) -> Di
 
     # This statement results in true for typing.Namedtuple, single and void return types, so this
     # handles Options 1, 2. Even though NamedTuple for us is multi-valued, it's a single value for Python
-    if isinstance(return_annotation, Type) or isinstance(return_annotation, TypeVar):
+    if hasattr(return_annotation, "__bases__") and (
+        isinstance(return_annotation, Type) or isinstance(return_annotation, TypeVar)  # type: ignore
+    ):
         # isinstance / issubclass does not work for Namedtuple.
         # Options 1 and 2
         bases = return_annotation.__bases__  # type: ignore
         if len(bases) == 1 and bases[0] == tuple and hasattr(return_annotation, "_fields"):
             logger.debug(f"Task returns named tuple {return_annotation}")
-            return dict(get_type_hints(return_annotation, include_extras=True))
+            return dict(get_type_hints(cast(Type, return_annotation), include_extras=True))
 
     if hasattr(return_annotation, "__origin__") and return_annotation.__origin__ is tuple:  # type: ignore
         # Handle option 3
@@ -418,8 +569,8 @@ def extract_return_annotation(return_annotation: Union[Type, Tuple, None]) -> Di
 
     else:
         # Handle all other single return types
-        logger.debug(f"Task returns unnamed native tuple {return_annotation}")
-        return {default_output_name(): return_annotation}
+        developer_logger.debug(f"Task returns unnamed native tuple {return_annotation}")
+        return {default_output_name(): cast(Type, return_annotation)}
 
 
 def remap_shared_output_descriptions(output_descriptions: Dict[str, str], outputs: Dict[str, Type]) -> Dict[str, str]:
