@@ -8,6 +8,7 @@ from flytekit.core.context_manager import FlyteContext, FlyteContextManager, Fly
 from flytekit.core.interface import Interface, transform_function_to_interface, transform_inputs_to_parameters
 from flytekit.core.promise import create_and_link_node, translate_inputs_to_literals
 from flytekit.core.reference_entity import LaunchPlanReference, ReferenceEntity
+from flytekit.core.schedule import LaunchPlanTriggerBase
 from flytekit.models import common as _common_models
 from flytekit.models import interface as _interface_models
 from flytekit.models import literals as _literal_models
@@ -24,27 +25,19 @@ class LaunchPlan(object):
     Every workflow is registered with a default launch plan, which is just a launch plan with none of the additional
     attributes set - no default values, fixed values, schedules, etc. Assuming you have the following workflow
 
-    .. code-block:: python
-
-        @workflow
-        def wf(a: int, c: str) -> str:
+    ```python
+    @workflow
+    def wf(a: int, c: str) -> str:
             ...
-
+    ```
     Create the default launch plan with
 
-    .. code-block:: python
-
-        LaunchPlan.get_or_create(workflow=my_wf)
-
+    ```python
+    LaunchPlan.get_or_create(workflow=my_wf)
+    ```
     If you specify additional parameters, you'll also have to give the launch plan a unique name. Default and
     fixed inputs can be expressed as Python native values like so:
 
-    .. literalinclude:: ../../../tests/flytekit/unit/core/test_launch_plan.py
-       :start-after: # fixed_and_default_start
-       :end-before: # fixed_and_default_end
-       :language: python
-       :dedent: 4
-
     Additionally, a launch plan can be configured to run on a schedule and emit notifications.
 
 
@@ -52,29 +45,16 @@ class LaunchPlan(object):
 
     To configure the remaining parameters, you'll need to import the relevant model objects as well.
 
-    .. literalinclude:: ../../../tests/flytekit/unit/core/test_launch_plan.py
-       :start-after: # schedule_start
-       :end-before: # schedule_end
-       :language: python
-       :dedent: 4
-
-    .. code-block:: python
-
-        from flytekit.models.common import Annotations, AuthRole, Labels, RawOutputDataConfig
-
-    Then use as follows
-
-    .. literalinclude:: ../../../tests/flytekit/unit/core/test_launch_plan.py
-       :start-after: # auth_role_start
-       :end-before: # auth_role_end
-       :language: python
-       :dedent: 4
+    ```python
+    from flytekit.models.common import Annotations, AuthRole, Labels, RawOutputDataConfig
+    ```
+    Then use as follows:
 
     """
 
     # The reason we cache is simply because users may get the default launch plan twice for a single Workflow. We
     # don't want to create two defaults, could be confusing.
-    CACHE = {}
+    CACHE: typing.Dict[str, LaunchPlan] = {}
 
     @staticmethod
     def get_default_launch_plan(ctx: FlyteContext, workflow: _annotated_workflow.WorkflowBase) -> LaunchPlan:
@@ -92,13 +72,27 @@ class LaunchPlan(object):
 
         parameter_map = transform_inputs_to_parameters(ctx, workflow.python_interface)
 
+        default_labels = None
+        default_annotations = None
+        if workflow.default_options is not None:
+            default_labels = workflow.default_options.labels
+            default_annotations = workflow.default_options.annotations
+
         lp = LaunchPlan(
             name=workflow.name,
             workflow=workflow,
             parameters=parameter_map,
             fixed_inputs=_literal_models.LiteralMap(literals={}),
+            labels=default_labels,
+            annotations=default_annotations,
         )
 
+        # Ensure default parameters are available when using lp.__call__()
+        default_inputs = {
+            name: default for name, (type, default) in workflow.python_interface.inputs_with_defaults.items()
+        }
+        lp._saved_inputs = default_inputs
+
         LaunchPlan.CACHE[workflow.name] = lp
         return lp
 
@@ -107,16 +101,19 @@ class LaunchPlan(object):
         cls,
         name: str,
         workflow: _annotated_workflow.WorkflowBase,
-        default_inputs: Dict[str, Any] = None,
-        fixed_inputs: Dict[str, Any] = None,
-        schedule: _schedule_model.Schedule = None,
-        notifications: List[_common_models.Notification] = None,
-        labels: _common_models.Labels = None,
-        annotations: _common_models.Annotations = None,
-        raw_output_data_config: _common_models.RawOutputDataConfig = None,
-        max_parallelism: int = None,
-        security_context: typing.Optional[security.SecurityContext] = None,
-        auth_role: _common_models.AuthRole = None,
+        default_inputs: Optional[Dict[str, Any]] = None,
+        fixed_inputs: Optional[Dict[str, Any]] = None,
+        schedule: Optional[_schedule_model.Schedule] = None,
+        notifications: Optional[List[_common_models.Notification]] = None,
+        labels: Optional[_common_models.Labels] = None,
+        annotations: Optional[_common_models.Annotations] = None,
+        raw_output_data_config: Optional[_common_models.RawOutputDataConfig] = None,
+        max_parallelism: Optional[int] = None,
+        security_context: Optional[security.SecurityContext] = None,
+        auth_role: Optional[_common_models.AuthRole] = None,
+        trigger: Optional[LaunchPlanTriggerBase] = None,
+        overwrite_cache: Optional[bool] = None,
+        auto_activate: bool = False,
     ) -> LaunchPlan:
         ctx = FlyteContextManager.current_context()
         default_inputs = default_inputs or {}
@@ -130,7 +127,7 @@ class LaunchPlan(object):
         temp_inputs = {}
         for k, v in default_inputs.items():
             temp_inputs[k] = (workflow.python_interface.inputs[k], v)
-        temp_interface = Interface(inputs=temp_inputs, outputs={})
+        temp_interface = Interface(inputs=temp_inputs, outputs={})  # type: ignore
         temp_signature = transform_inputs_to_parameters(ctx, temp_interface)
         wf_signature_parameters._parameters.update(temp_signature.parameters)
 
@@ -167,6 +164,9 @@ class LaunchPlan(object):
             raw_output_data_config=raw_output_data_config,
             max_parallelism=max_parallelism,
             security_context=security_context,
+            trigger=trigger,
+            overwrite_cache=overwrite_cache,
+            auto_activate=auto_activate,
         )
 
         # This is just a convenience - we'll need the fixed inputs LiteralMap for when serializing the Launch Plan out
@@ -185,16 +185,19 @@ class LaunchPlan(object):
         cls,
         workflow: _annotated_workflow.WorkflowBase,
         name: Optional[str] = None,
-        default_inputs: Dict[str, Any] = None,
-        fixed_inputs: Dict[str, Any] = None,
-        schedule: _schedule_model.Schedule = None,
-        notifications: List[_common_models.Notification] = None,
-        labels: _common_models.Labels = None,
-        annotations: _common_models.Annotations = None,
-        raw_output_data_config: _common_models.RawOutputDataConfig = None,
-        max_parallelism: int = None,
-        security_context: typing.Optional[security.SecurityContext] = None,
-        auth_role: _common_models.AuthRole = None,
+        default_inputs: Optional[Dict[str, Any]] = None,
+        fixed_inputs: Optional[Dict[str, Any]] = None,
+        schedule: Optional[_schedule_model.Schedule] = None,
+        notifications: Optional[List[_common_models.Notification]] = None,
+        labels: Optional[_common_models.Labels] = None,
+        annotations: Optional[_common_models.Annotations] = None,
+        raw_output_data_config: Optional[_common_models.RawOutputDataConfig] = None,
+        max_parallelism: Optional[int] = None,
+        security_context: Optional[security.SecurityContext] = None,
+        auth_role: Optional[_common_models.AuthRole] = None,
+        trigger: Optional[LaunchPlanTriggerBase] = None,
+        overwrite_cache: Optional[bool] = None,
+        auto_activate: bool = False,
     ) -> LaunchPlan:
         """
         This function offers a friendlier interface for creating launch plans. If the name for the launch plan is not
@@ -220,6 +223,10 @@ class LaunchPlan(object):
         :param max_parallelism: Controls the maximum number of tasknodes that can be run in parallel for the entire
             workflow. This is useful to achieve fairness. Note: MapTasks are regarded as one unit, and
             parallelism/concurrency of MapTasks is independent from this.
+        :param trigger: [alpha] This is a new syntax for specifying schedules.
+        :param overwrite_cache: If set to True, the execution will always overwrite cache
+        :param auto_activate: If set to True, the launch plan will be activated automatically on registration.
+         Default is False.
         """
         if name is None and (
             default_inputs is not None
@@ -232,6 +239,8 @@ class LaunchPlan(object):
             or auth_role is not None
             or max_parallelism is not None
             or security_context is not None
+            or trigger is not None
+            or overwrite_cache is not None
         ):
             raise ValueError(
                 "Only named launchplans can be created that have other properties. Drop the name if you want to create a default launchplan. Default launchplans cannot have any other associations"
@@ -253,18 +262,29 @@ class LaunchPlan(object):
                     ),
                 )
 
-            if (
-                workflow != cached_outputs["_workflow"]
-                or schedule != cached_outputs["_schedule"]
-                or notifications != cached_outputs["_notifications"]
-                or default_inputs != cached_outputs["_saved_inputs"]
-                or labels != cached_outputs["_labels"]
-                or annotations != cached_outputs["_annotations"]
-                or raw_output_data_config != cached_outputs["_raw_output_data_config"]
-                or max_parallelism != cached_outputs["_max_parallelism"]
-                or security_context != cached_outputs["_security_context"]
-            ):
-                raise AssertionError("The cached values aren't the same as the current call arguments")
+            if workflow != cached_outputs["_workflow"]:
+                raise AssertionError(
+                    f"Trying to create two launch plans both named '{name}' for the workflows '{workflow.name}' "
+                    f"and '{cached_outputs['_workflow'].name}' - please ensure unique names."
+                )
+
+            for arg_name, new, cached in [
+                ("schedule", schedule, cached_outputs["_schedule"]),
+                ("notifications", notifications, cached_outputs["_notifications"]),
+                ("default_inputs", default_inputs, cached_outputs["_saved_inputs"]),
+                ("labels", labels, cached_outputs["_labels"]),
+                ("annotations", annotations, cached_outputs["_annotations"]),
+                ("raw_output_data_config", raw_output_data_config, cached_outputs["_raw_output_data_config"]),
+                ("max_parallelism", max_parallelism, cached_outputs["_max_parallelism"]),
+                ("security_context", security_context, cached_outputs["_security_context"]),
+                ("overwrite_cache", overwrite_cache, cached_outputs["_overwrite_cache"]),
+                ("auto_activate", auto_activate, cached_outputs["_auto_activate"]),
+            ]:
+                if new != cached:
+                    raise AssertionError(
+                        f"Trying to create two launch plans for workflow '{workflow.name}' both named '{name}' "
+                        f"but with different values for '{arg_name}' - please use different launch plan names."
+                    )
 
             return LaunchPlan.CACHE[name]
         elif name is None and workflow.name in LaunchPlan.CACHE:
@@ -288,6 +308,9 @@ class LaunchPlan(object):
                 max_parallelism,
                 auth_role=auth_role,
                 security_context=security_context,
+                trigger=trigger,
+                overwrite_cache=overwrite_cache,
+                auto_activate=auto_activate,
             )
         LaunchPlan.CACHE[name or workflow.name] = lp
         return lp
@@ -298,13 +321,16 @@ class LaunchPlan(object):
         workflow: _annotated_workflow.WorkflowBase,
         parameters: _interface_models.ParameterMap,
         fixed_inputs: _literal_models.LiteralMap,
-        schedule: _schedule_model.Schedule = None,
-        notifications: List[_common_models.Notification] = None,
-        labels: _common_models.Labels = None,
-        annotations: _common_models.Annotations = None,
-        raw_output_data_config: _common_models.RawOutputDataConfig = None,
-        max_parallelism: typing.Optional[int] = None,
-        security_context: typing.Optional[security.SecurityContext] = None,
+        schedule: Optional[_schedule_model.Schedule] = None,
+        notifications: Optional[List[_common_models.Notification]] = None,
+        labels: Optional[_common_models.Labels] = None,
+        annotations: Optional[_common_models.Annotations] = None,
+        raw_output_data_config: Optional[_common_models.RawOutputDataConfig] = None,
+        max_parallelism: Optional[int] = None,
+        security_context: Optional[security.SecurityContext] = None,
+        trigger: Optional[LaunchPlanTriggerBase] = None,
+        overwrite_cache: Optional[bool] = None,
+        auto_activate: bool = False,
     ):
         self._name = name
         self._workflow = workflow
@@ -313,7 +339,7 @@ class LaunchPlan(object):
         self._parameters = _interface_models.ParameterMap(parameters=parameters)
         self._fixed_inputs = fixed_inputs
         # See create() for additional information
-        self._saved_inputs = {}
+        self._saved_inputs: Dict[str, Any] = {}
 
         self._schedule = schedule
         self._notifications = notifications or []
@@ -322,22 +348,27 @@ class LaunchPlan(object):
         self._raw_output_data_config = raw_output_data_config
         self._max_parallelism = max_parallelism
         self._security_context = security_context
+        self._trigger = trigger
+        self._overwrite_cache = overwrite_cache
+        self._auto_activate = auto_activate
 
         FlyteEntities.entities.append(self)
 
     def clone_with(
         self,
         name: str,
-        parameters: _interface_models.ParameterMap = None,
-        fixed_inputs: _literal_models.LiteralMap = None,
-        schedule: _schedule_model.Schedule = None,
-        notifications: List[_common_models.Notification] = None,
-        labels: _common_models.Labels = None,
-        annotations: _common_models.Annotations = None,
-        raw_output_data_config: _common_models.RawOutputDataConfig = None,
-        auth_role: _common_models.AuthRole = None,
-        max_parallelism: int = None,
-        security_context: typing.Optional[security.SecurityContext] = None,
+        parameters: Optional[_interface_models.ParameterMap] = None,
+        fixed_inputs: Optional[_literal_models.LiteralMap] = None,
+        schedule: Optional[_schedule_model.Schedule] = None,
+        notifications: Optional[List[_common_models.Notification]] = None,
+        labels: Optional[_common_models.Labels] = None,
+        annotations: Optional[_common_models.Annotations] = None,
+        raw_output_data_config: Optional[_common_models.RawOutputDataConfig] = None,
+        max_parallelism: Optional[int] = None,
+        security_context: Optional[security.SecurityContext] = None,
+        trigger: Optional[LaunchPlanTriggerBase] = None,
+        overwrite_cache: Optional[bool] = None,
+        auto_activate: bool = False,
     ) -> LaunchPlan:
         return LaunchPlan(
             name=name,
@@ -349,11 +380,17 @@ class LaunchPlan(object):
             labels=labels or self.labels,
             annotations=annotations or self.annotations,
             raw_output_data_config=raw_output_data_config or self.raw_output_data_config,
-            auth_role=auth_role or self._auth_role,
             max_parallelism=max_parallelism or self.max_parallelism,
             security_context=security_context or self.security_context,
+            trigger=trigger,
+            overwrite_cache=overwrite_cache or self.overwrite_cache,
+            auto_activate=auto_activate,
         )
 
+    @property
+    def overwrite_cache(self) -> Optional[bool]:
+        return self._overwrite_cache
+
     @property
     def python_interface(self) -> Interface:
         return self.workflow.python_interface
@@ -407,13 +444,21 @@ class LaunchPlan(object):
         return self._raw_output_data_config
 
     @property
-    def max_parallelism(self) -> typing.Optional[int]:
+    def max_parallelism(self) -> Optional[int]:
         return self._max_parallelism
 
     @property
-    def security_context(self) -> typing.Optional[security.SecurityContext]:
+    def security_context(self) -> Optional[security.SecurityContext]:
         return self._security_context
 
+    @property
+    def trigger(self) -> Optional[LaunchPlanTriggerBase]:
+        return self._trigger
+
+    @property
+    def should_auto_activate(self) -> bool:
+        return self._auto_activate
+
     def construct_node_metadata(self) -> _workflow_model.NodeMetadata:
         return self.workflow.construct_node_metadata()
 
@@ -455,12 +500,19 @@ def reference_launch_plan(
 ) -> Callable[[Callable[..., Any]], ReferenceLaunchPlan]:
     """
     A reference launch plan is a pointer to a launch plan that already exists on your Flyte installation. This
-    object will not initiate a network call to Admin, which is why the user is asked to provide the expected interface.
+    object will not initiate a network call to Admin, which is why the user is asked to provide the expected interface
+    via the function definition.
+
     If at registration time the interface provided causes an issue with compilation, an error will be returned.
+
+    :param project: Flyte project name of the launch plan
+    :param domain: Flyte domain name of the launch plan
+    :param name: launch plan name
+    :param version: specific version of the launch plan to use
     """
 
     def wrapper(fn) -> ReferenceLaunchPlan:
-        interface = transform_function_to_interface(fn)
+        interface = transform_function_to_interface(fn, is_reference_entity=True)
         return ReferenceLaunchPlan(project, domain, name, version, interface.inputs, interface.outputs)
 
     return wrapper
