@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-import collections
-from typing import TYPE_CHECKING, Type, Union
+import inspect
+from typing import TYPE_CHECKING, Union
 
 from flytekit.core.base_task import PythonTask
 from flytekit.core.context_manager import BranchEvalMode, ExecutionState, FlyteContext
@@ -11,6 +11,7 @@ from flytekit.core.promise import VoidPromise
 from flytekit.core.workflow import WorkflowBase
 from flytekit.exceptions import user as _user_exceptions
 from flytekit.loggers import logger
+from flytekit.utils.asyn import run_sync
 
 if TYPE_CHECKING:
     from flytekit.remote.remote_callable import RemoteEntity
@@ -21,11 +22,11 @@ if TYPE_CHECKING:
 
 def create_node(
     entity: Union[PythonTask, LaunchPlan, WorkflowBase, RemoteEntity], *args, **kwargs
-) -> Union[Node, VoidPromise, Type[collections.namedtuple]]:
+) -> Union[Node, VoidPromise]:
     """
     This is the function you want to call if you need to specify dependencies between tasks that don't consume and/or
     don't produce outputs. For example, if you have t1() and t2(), both of which do not take in nor produce any
-    outputs, how do you specify that t2 should run before t1?
+    outputs, how do you specify that t2 should run before t1? ::
 
         t1_node = create_node(t1)
         t2_node = create_node(t2)
@@ -34,36 +35,23 @@ def create_node(
         # OR
         t2_node >> t1_node
 
-    This works for tasks that take inputs as well, say a ``t3(in1: int)``
+    This works for tasks that take inputs as well, say a ``t3(in1: int)`` ::
 
         t3_node = create_node(t3, in1=some_int)  # basically calling t3(in1=some_int)
 
-    You can still use this method to handle setting certain overrides
+    You can still use this method to handle setting certain overrides ::
 
         t3_node = create_node(t3, in1=some_int).with_overrides(...)
 
-    Outputs, if there are any, will be accessible. A `t4() -> (int, str)`
+    Outputs, if there are any, will be accessible. A `t4() -> (int, str)` ::
 
         t4_node = create_node(t4)
 
-        in compilation node.o0 has the promise.
+    In compilation node.o0 has the promise. ::
         t5(in1=t4_node.o0)
 
-        in local workflow execution, what is the node?  Can it just be the named tuple?
-        t5(in1=t4_node.o0)
-
-    @workflow
-    def wf():
-        create_node(sub_wf)
-        create_node(wf2)
-
-    @dynamic
-    def sub_wf():
-        create_node(other_sub)
-        create_node(task)
-
     If t1 produces only one output, note that in local execution, you still get a wrapper object that
-    needs to be dereferenced by the output name.
+    needs to be dereferenced by the output name. ::
 
         t1_node = create_node(t1)
         t2(t1_node.o0)
@@ -91,7 +79,10 @@ def create_node(
     # When compiling, calling the entity will create a node.
     ctx = FlyteContext.current_context()
     if ctx.compilation_state is not None and ctx.compilation_state.mode == 1:
-        outputs = entity(**kwargs)
+        if inspect.iscoroutinefunction(entity.__call__):
+            outputs = run_sync(entity, **kwargs)
+        else:
+            outputs = entity(**kwargs)
         # This is always the output of create_and_link_node which returns create_task_output, which can be
         # VoidPromise, Promise, or our custom namedtuple of Promises.
         node = ctx.compilation_state.nodes[-1]
@@ -143,18 +134,18 @@ def create_node(
         return node
 
     # Handling local execution
-    # Note: execution state is set to TASK_EXECUTION when running dynamic task locally
+    # Note: execution state is set to DYNAMIC_TASK_EXECUTION when running a dynamic task locally
     # https://github.com/flyteorg/flytekit/blob/0815345faf0fae5dc26746a43d4bda4cc2cdf830/flytekit/core/python_function_task.py#L262
-    elif ctx.execution_state is not None and (
-        ctx.execution_state.mode == ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION
-        or ctx.execution_state.mode == ExecutionState.Mode.TASK_EXECUTION
+    elif ctx.execution_state and (
+        ctx.execution_state.is_local_execution()
+        or ctx.execution_state.mode == ExecutionState.Mode.LOCAL_DYNAMIC_TASK_EXECUTION
     ):
         if isinstance(entity, RemoteEntity):
             raise AssertionError(f"Remote entities are not yet runnable locally {entity.name}")
 
         if ctx.execution_state.branch_eval_mode == BranchEvalMode.BRANCH_SKIPPED:
             logger.warning(f"Manual node creation cannot be used in branch logic {entity.name}")
-            raise Exception("Being more restrictive for now and disallowing manual node creation in branch logic")
+            raise RuntimeError("Being more restrictive for now and disallowing manual node creation in branch logic")
 
         # This the output of __call__ under local execute conditions which means this is the output of local_execute
         # which means this is the output of create_task_output with Promises containing values (or a VoidPromise)
@@ -169,13 +160,13 @@ def create_node(
         output_names = entity.python_interface.output_names  # type: ignore
 
         if not output_names:
-            raise Exception(f"Non-VoidPromise received {results} but interface for {entity.name} doesn't have outputs")
+            raise ValueError(f"Non-VoidPromise received {results} but interface for {entity.name} doesn't have outputs")
 
         if len(output_names) == 1:
             # See explanation above for why we still tupletize a single element.
-            return entity.python_interface.output_tuple(results)
+            return entity.python_interface.output_tuple(results)  # type: ignore
 
-        return entity.python_interface.output_tuple(*results)
+        return entity.python_interface.output_tuple(*results)  # type: ignore
 
     else:
-        raise Exception(f"Cannot use explicit run to call Flyte entities {entity.name}")
+        raise RuntimeError(f"Cannot use explicit run to call Flyte entities {entity.name}")
