@@ -8,10 +8,12 @@ from flyteidl.core import tasks_pb2 as _tasks_pb2
 from flytekit.configuration import Image, ImageConfig, SerializationSettings
 from flytekit.core.base_task import PythonTask, Task, TaskResolverMixin
 from flytekit.core.context_manager import FlyteContext
+from flytekit.core.python_auto_container import get_registerable_container_image
 from flytekit.core.resources import Resources, ResourceSpec
 from flytekit.core.shim_task import ExecutableTemplateShimTask, ShimTaskExecutor
 from flytekit.core.tracker import TrackedInstance
 from flytekit.core.utils import _get_container_definition, load_proto_from_file
+from flytekit.image_spec.image_spec import ImageSpec
 from flytekit.loggers import logger
 from flytekit.models import task as _task_model
 from flytekit.models.core import identifier as identifier_models
@@ -21,9 +23,9 @@ from flytekit.tools.module_loader import load_object_from_module
 TC = TypeVar("TC")
 
 
-class PythonCustomizedContainerTask(ExecutableTemplateShimTask, PythonTask[TC]):
+class PythonCustomizedContainerTask(ExecutableTemplateShimTask, PythonTask[TC]):  # type: ignore
     """
-    Please take a look at the comments for :py:class`flytekit.extend.ExecutableTemplateShimTask` as well. This class
+    Please take a look at the comments for {{< py_class_ref flytekit.extend.ExecutableTemplateShimTask >}} as well. This class
     should be subclassed and a custom Executor provided as a default to this parent class constructor
     when building a new external-container flytekit-only plugin.
 
@@ -82,10 +84,10 @@ class PythonCustomizedContainerTask(ExecutableTemplateShimTask, PythonTask[TC]):
            The key values will be available from runtime, if the backend is configured to provide secrets and
            if secrets are available in the configured secrets store. Possible options for secret stores are
 
-           - `Vault <https://www.vaultproject.io/>`__
-           - `Confidant <https://lyft.github.io/confidant/>`__
-           - `Kube secrets <https://kubernetes.io/docs/concepts/configuration/secret/>`__
-           - `AWS Parameter store <https://docs.aws.amazon.com/systems-manager/latest/userguide/systems-manager-parameter-store.html>`__
+           - [`Vault`](https://www.vaultproject.io)
+           - [`Confidant`](https://lyft.github.io/confidant)
+           - [`Kube secrets`](https://kubernetes.io/docs/concepts/configuration/secret)
+           - [`AWS Parameter store`](https://docs.aws.amazon.com/systems-manager/latest/userguide/systems-manager-parameter-store.html)
         """
         sec_ctx = None
         if secret_requests:
@@ -157,24 +159,22 @@ class PythonCustomizedContainerTask(ExecutableTemplateShimTask, PythonTask[TC]):
 
         return container_args
 
+    def get_image(self, settings: SerializationSettings) -> str:
+        if settings.fast_serialization_settings is None or not settings.fast_serialization_settings.enabled:
+            if isinstance(self.container_image, ImageSpec):
+                # Set the source root for the image spec if it's non-fast registration
+                self.container_image.source_root = settings.source_root
+        return get_registerable_container_image(self.container_image, settings.image_config)
+
     def get_container(self, settings: SerializationSettings) -> _task_model.Container:
         env = {**settings.env, **self.environment} if self.environment else settings.env
         return _get_container_definition(
-            image=self.container_image,
+            image=self.get_image(settings),
+            resource_spec=self.resources,
             command=[],
             args=self.get_command(settings=settings),
             data_loading_config=None,
             environment=env,
-            storage_request=self.resources.requests.storage,
-            ephemeral_storage_request=self.resources.requests.ephemeral_storage,
-            cpu_request=self.resources.requests.cpu,
-            gpu_request=self.resources.requests.gpu,
-            memory_request=self.resources.requests.mem,
-            storage_limit=self.resources.limits.storage,
-            ephemeral_storage_limit=self.resources.limits.ephemeral_storage,
-            cpu_limit=self.resources.limits.cpu,
-            gpu_limit=self.resources.limits.gpu,
-            memory_limit=self.resources.limits.mem,
         )
 
     def serialize_to_model(self, settings: SerializationSettings) -> _task_model.TaskTemplate:
@@ -229,7 +229,7 @@ class TaskTemplateResolver(TrackedInstance, TaskResolverMixin):
 
     # The return type of this function is different, it should be a Task, but it's not because it doesn't make
     # sense for ExecutableTemplateShimTask to inherit from Task.
-    def load_task(self, loader_args: List[str]) -> ExecutableTemplateShimTask:
+    def load_task(self, loader_args: List[str]) -> ExecutableTemplateShimTask:  # type: ignore
         logger.info(f"Task template loader args: {loader_args}")
         ctx = FlyteContext.current_context()
         task_template_local_path = os.path.join(ctx.execution_state.working_dir, "task_template.pb")  # type: ignore
@@ -240,7 +240,7 @@ class TaskTemplateResolver(TrackedInstance, TaskResolverMixin):
         executor_class = load_object_from_module(loader_args[1])
         return ExecutableTemplateShimTask(task_template_model, executor_class)
 
-    def loader_args(self, settings: SerializationSettings, t: PythonCustomizedContainerTask) -> List[str]:
+    def loader_args(self, settings: SerializationSettings, t: PythonCustomizedContainerTask) -> List[str]:  # type: ignore
         return ["{{.taskTemplatePath}}", f"{t.executor_type.__module__}.{t.executor_type.__name__}"]
 
     def get_all_tasks(self) -> List[Task]:
