@@ -2,7 +2,6 @@ from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from typing import Any, Dict, Optional, Tuple, Type, Union
 
-from flytekit.common.exceptions import user as _user_exceptions
 from flytekit.core.context_manager import BranchEvalMode, ExecutionState, FlyteContext
 from flytekit.core.interface import Interface, transform_interface_to_typed_interface
 from flytekit.core.promise import (
@@ -10,18 +9,18 @@ from flytekit.core.promise import (
     VoidPromise,
     create_and_link_node,
     create_task_output,
+    extract_obj_name,
     translate_inputs_to_literals,
 )
 from flytekit.core.type_engine import TypeEngine
 from flytekit.loggers import logger
-from flytekit.models import dynamic_job as _dynamic_job
 from flytekit.models import interface as _interface_models
 from flytekit.models import literals as _literal_models
 from flytekit.models.core import identifier as _identifier_model
 from flytekit.models.core import workflow as _workflow_model
 
 
-@dataclass
+@dataclass  # type: ignore
 class Reference(ABC):
     project: str
     domain: str
@@ -37,12 +36,13 @@ class Reference(ABC):
 
     @property
     @abstractmethod
-    def resource_type(self) -> int:
-        ...
+    def resource_type(self) -> int: ...
 
 
 @dataclass
 class TaskReference(Reference):
+    """A reference object containing metadata that points to a remote task."""
+
     @property
     def resource_type(self) -> int:
         return _identifier_model.ResourceType.TASK
@@ -50,6 +50,8 @@ class TaskReference(Reference):
 
 @dataclass
 class LaunchPlanReference(Reference):
+    """A reference object containing metadata that points to a remote launch plan."""
+
     @property
     def resource_type(self) -> int:
         return _identifier_model.ResourceType.LAUNCH_PLAN
@@ -57,6 +59,8 @@ class LaunchPlanReference(Reference):
 
 @dataclass
 class WorkflowReference(Reference):
+    """A reference object containing metadata that points to a remote workflow."""
+
     @property
     def resource_type(self) -> int:
         return _identifier_model.ResourceType.WORKFLOW
@@ -66,7 +70,7 @@ class ReferenceEntity(object):
     def __init__(
         self,
         reference: Union[WorkflowReference, TaskReference, LaunchPlanReference],
-        inputs: Optional[Dict[str, Union[Type[Any], Tuple[Type[Any], Any]]]],
+        inputs: Dict[str, Type],
         outputs: Dict[str, Type],
     ):
         if (
@@ -74,20 +78,20 @@ class ReferenceEntity(object):
             and not isinstance(reference, TaskReference)
             and not isinstance(reference, LaunchPlanReference)
         ):
-            raise Exception("Must be one of task, workflow, or launch plan")
+            raise ValueError(f"Must be one of task, workflow, or launch plan, but got {type(reference)}")
         self._reference = reference
         self._native_interface = Interface(inputs=inputs, outputs=outputs)
         self._interface = transform_interface_to_typed_interface(self._native_interface)
 
     def execute(self, **kwargs) -> Any:
-        raise Exception("Remote reference entities cannot be run locally. You must mock this out.")
+        raise NotImplementedError("Remote reference entities cannot be run locally. You must mock this out.")
 
     @property
-    def python_interface(self) -> Optional[Interface]:
+    def python_interface(self) -> Interface:
         return self._native_interface
 
     @property
-    def interface(self) -> Optional[_interface_models.TypedInterface]:
+    def interface(self) -> _interface_models.TypedInterface:
         return self._interface
 
     @property
@@ -104,7 +108,7 @@ class ReferenceEntity(object):
 
     def unwrap_literal_map_and_execute(
         self, ctx: FlyteContext, input_literal_map: _literal_models.LiteralMap
-    ) -> Union[VoidPromise, _literal_models.LiteralMap, _dynamic_job.DynamicJobSpec]:
+    ) -> _literal_models.LiteralMap:
         """
         Please see the implementation of the dispatch_execute function in the real task.
         """
@@ -119,7 +123,6 @@ class ReferenceEntity(object):
         except Exception as e:
             logger.exception(f"Exception when executing {e}")
             raise e
-        logger.info(f"Task executed successfully in user level, outputs: {native_outputs}")
 
         expected_output_names = list(self.python_interface.outputs.keys())
         if len(expected_output_names) == 1:
@@ -142,7 +145,7 @@ class ReferenceEntity(object):
         # After the execute has been successfully completed
         return outputs_literal_map
 
-    def local_execute(self, ctx: FlyteContext, **kwargs) -> Union[Tuple[Promise], Promise, VoidPromise]:
+    def local_execute(self, ctx: FlyteContext, **kwargs) -> Optional[Union[Tuple[Promise], Promise, VoidPromise]]:
         """
         Please see the local_execute comments in the main task.
         """
@@ -176,8 +179,11 @@ class ReferenceEntity(object):
         vals = [Promise(var, outputs_literals[var]) for var in output_names]
         return create_task_output(vals, self.python_interface)
 
+    def local_execution_mode(self):
+        return ExecutionState.Mode.LOCAL_TASK_EXECUTION
+
     def construct_node_metadata(self) -> _workflow_model.NodeMetadata:
-        return _workflow_model.NodeMetadata(name=f"{self.__module__}.{self.name}")
+        return _workflow_model.NodeMetadata(name=extract_obj_name(self.name))
 
     def compile(self, ctx: FlyteContext, *args, **kwargs):
         return create_and_link_node(ctx, entity=self, **kwargs)
@@ -193,17 +199,25 @@ class ReferenceEntity(object):
         #     nothing. Subsequent tasks will have to know how to unwrap these. If by chance a non-Flyte task uses a
         #     task output as an input, things probably will fail pretty obviously.
         #     Since this is a reference entity, it still needs to be mocked otherwise an exception will be raised.
-        if len(args) > 0:
-            raise _user_exceptions.FlyteAssertion(
-                f"Cannot call reference entity with args - detected {len(args)} positional args {args}"
+
+        interface = self.python_interface
+
+        # Check if we have more arguments than expected
+        if len(args) > len(interface.inputs):
+            raise AssertionError(
+                f"Received more arguments than expected in function '{self.name}'. Expected {len(interface.inputs)} but got {len(args)}"
             )
 
+        # Convert args to kwargs
+        for arg, input_name in zip(args, interface.inputs.keys()):
+            if input_name in kwargs:
+                raise AssertionError(f"Got multiple values for argument '{input_name}' in function '{self.name}'")
+            kwargs[input_name] = arg
+
         ctx = FlyteContext.current_context()
         if ctx.compilation_state is not None and ctx.compilation_state.mode == 1:
             return self.compile(ctx, *args, **kwargs)
-        elif (
-            ctx.execution_state is not None and ctx.execution_state.mode == ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION
-        ):
+        elif ctx.execution_state and ctx.execution_state.is_local_execution():
             if ctx.execution_state.branch_eval_mode == BranchEvalMode.BRANCH_SKIPPED:
                 return
             return self.local_execute(ctx, **kwargs)
