@@ -1,29 +1,35 @@
 """
-.. autoclass:: flytekit.core.schedule.CronSchedule
-   :noindex:
-
+These classes provide functionality related to schedules.
 """
 
 import datetime
-import re as _re
+import re
+from typing import Optional, Protocol, Union
 
-import croniter as _croniter
+import croniter
+from flyteidl.admin import schedule_pb2
+from google.protobuf import message as google_message
 
 from flytekit.models import schedule as _schedule_models
 
 
+class LaunchPlanTriggerBase(Protocol):
+    def to_flyte_idl(self, *args, **kwargs) -> google_message.Message: ...
+
+
 # Duplicates flytekit.common.schedules.Schedule to avoid using the ExtendedSdkType metaclass.
 class CronSchedule(_schedule_models.Schedule):
     """
     Use this when you have a launch plan that you want to run on a cron expression.
-    This uses standard `cron format <https://docs.flyte.org/en/latest/concepts/schedules.html#cron-expression-table>`__
+    This uses standard [`cron format`](https://docs.flyte.org/en/latest/concepts/schedules.html#cron-expression-table)
     in case where you are using default native scheduler using the schedule attribute.
 
-    .. code-block::
+    ```
 
         CronSchedule(
             schedule="*/1 * * * *",  # Following schedule runs every min
         )
+    ```
 
     See the :std:ref:`User Guide <cookbook:cron schedules>` for further examples.
     """
@@ -49,15 +55,19 @@ class CronSchedule(_schedule_models.Schedule):
     ]
 
     # Not a perfect regex but good enough and simple to reason about
-    _OFFSET_PATTERN = _re.compile("([-+]?)P([-+0-9YMWD]+)?(T([-+0-9HMS.,]+)?)?")
+    _OFFSET_PATTERN = re.compile("([-+]?)P([-+0-9YMWD]+)?(T([-+0-9HMS.,]+)?)?")
 
     def __init__(
-        self, cron_expression: str = None, schedule: str = None, offset: str = None, kickoff_time_input_arg: str = None
+        self,
+        cron_expression: Optional[str] = None,
+        schedule: Optional[str] = None,
+        offset: Optional[str] = None,
+        kickoff_time_input_arg: Optional[str] = None,
     ):
         """
         :param str cron_expression: This should be a cron expression in AWS style.Shouldn't be used in case of native scheduler.
         :param str schedule: This takes a cron alias (see ``_VALID_CRON_ALIASES``) or a croniter parseable schedule.
-          Only one of this or ``cron_expression`` can be set, not both. This uses standard `cron format <https://docs.flyte.org/en/latest/concepts/schedules.html#cron-expression-table>`_
+          Only one of this or ``cron_expression`` can be set, not both. This uses standard [`cron format`](https://docs.flyte.org/en/latest/concepts/schedules.html#cron-expression)
           and is supported by native scheduler
         :param str offset:
         :param str kickoff_time_input_arg: This is a convenient argument to use when your code needs to know what time
@@ -73,14 +83,11 @@ class CronSchedule(_schedule_models.Schedule):
                 kickoff_time_input_arg="kickoff_time")
 
         """
-        if cron_expression is None and schedule is None:
-            raise AssertionError("Either `cron_expression` or `schedule` should be specified.")
-
-        if cron_expression is not None and offset is not None:
-            raise AssertionError("Only `schedule` is supported when specifying `offset`.")
-
-        if cron_expression is not None:
-            CronSchedule._validate_expression(cron_expression)
+        if cron_expression:
+            raise AssertionError(
+                "cron_expression is deprecated and should not be used. Use `schedule` instead. "
+                "See the documentation for more information."
+            )
 
         if schedule is not None:
             CronSchedule._validate_schedule(schedule)
@@ -124,7 +131,7 @@ class CronSchedule(_schedule_models.Schedule):
         try:
             # Cut to 5 fields and just assume year field is good because croniter treats the 6th field as seconds.
             # TODO: Parse this field ourselves and check
-            _croniter.croniter(" ".join(cron_expression.replace("?", "*").split()[:5]))
+            croniter.croniter(" ".join(cron_expression.replace("?", "*").split()[:5]))
         except Exception:
             raise ValueError(
                 "Scheduled string is invalid.  The cron expression was found to be invalid."
@@ -135,12 +142,12 @@ class CronSchedule(_schedule_models.Schedule):
     def _validate_schedule(schedule: str):
         if schedule.lower() not in CronSchedule._VALID_CRON_ALIASES:
             try:
-                _croniter.croniter(schedule)
-            except Exception:
-                raise ValueError(
-                    "Schedule is invalid. It must be set to either a cron alias or valid cron expression."
-                    f" Provided schedule: {schedule}"
-                )
+                # Validate the cron expression
+                cron = croniter.croniter(schedule)
+                # Try to get the next occurrence to validate the schedule
+                cron.get_next(datetime.datetime)
+            except Exception as e:
+                raise ValueError(f"Schedule is invalid. Provided schedule: {schedule} Error: {str(e)}")
 
     @staticmethod
     def _validate_offset(offset: str):
@@ -152,16 +159,16 @@ class FixedRate(_schedule_models.Schedule):
     """
     Use this class to schedule a fixed-rate interval for a launch plan.
 
-    .. code-block:: python
+    ```python
+    from datetime import timedelta
 
-        from datetime import timedelta
-
-        FixedRate(duration=timedelta(minutes=10))
+    FixedRate(duration=timedelta(minutes=10))
+    ```
 
     See the :std:ref:`fixed rate intervals` chapter in the cookbook for additional usage examples.
     """
 
-    def __init__(self, duration: datetime.timedelta, kickoff_time_input_arg: str = None):
+    def __init__(self, duration: datetime.timedelta, kickoff_time_input_arg: Optional[str] = None):
         """
         :param datetime.timedelta duration:
         :param str kickoff_time_input_arg:
@@ -197,3 +204,14 @@ class FixedRate(_schedule_models.Schedule):
                 int(duration.total_seconds() / _SECONDS_TO_MINUTES),
                 _schedule_models.Schedule.FixedRateUnit.MINUTE,
             )
+
+
+class OnSchedule(LaunchPlanTriggerBase):
+    def __init__(self, schedule: Union[CronSchedule, FixedRate]):
+        """
+        :param Union[CronSchedule, FixedRate] schedule: Either a cron or a fixed rate
+        """
+        self._schedule = schedule
+
+    def to_flyte_idl(self) -> schedule_pb2.Schedule:
+        return self._schedule.to_flyte_idl()
