@@ -6,15 +6,17 @@ import tempfile
 import typing
 from dataclasses import dataclass
 
-import pandas as pd
-
-from flytekit import FlyteContext, kwtypes
+from flytekit import FlyteContext, kwtypes, lazy_module
 from flytekit.configuration import DefaultImages, SerializationSettings
 from flytekit.core.base_sql_task import SQLTask
 from flytekit.core.python_customized_container_task import PythonCustomizedContainerTask
 from flytekit.core.shim_task import ShimTaskExecutor
 from flytekit.models import task as task_models
-from flytekit.types.schema import FlyteSchema
+
+if typing.TYPE_CHECKING:
+    import pandas as pd
+else:
+    pd = lazy_module("pandas")
 
 
 def unarchive_file(local_path: str, to_dir: str):
@@ -44,7 +46,7 @@ class SQLite3Config(object):
     Args:
         uri: default FlyteFile that will be downloaded on execute
         compressed: Boolean that indicates if the given file is a compressed archive. Supported file types are
-                    [zip, tar, gztar, bztar, xztar]
+            [zip, tar, gztar, bztar, xztar]
     """
 
     uri: str
@@ -55,19 +57,33 @@ class SQLite3Task(PythonCustomizedContainerTask[SQLite3Config], SQLTask[SQLite3C
     """
     Run client side SQLite3 queries that optionally return a FlyteSchema object.
 
-    .. note::
-
-       This is a pre-built container task. That is, your user container will not be used at task execution time.
+    > [!NOTE]
+    > This is a pre-built container task. That is, your user container will not be used at task execution time.
        Instead the image defined in this task definition will be used instead.
 
+    <!--
     .. literalinclude:: ../../../tests/flytekit/unit/extras/sqlite3/test_task.py
        :start-after: # sqlite3_start
        :end-before: # sqlite3_end
        :language: python
        :dedent: 4
+    -->
+    ```python
+    sql_task = SQLite3Task(
+            "test",
+            query_template="select TrackId, Name from tracks limit {{.inputs.limit}}",
+            inputs=kwtypes(limit=int),
+            output_schema_type=FlyteSchema[kwtypes(TrackId=int, Name=str)],
+            task_config=SQLite3Config(
+                uri=EXAMPLE_DB,
+                compressed=True,
+            ),
+    )
+    ```
 
-    See the :std:ref:`cookbook <extend_sql_sqlite3>` for additional usage examples and
-    the base class :py:class:`flytekit.extend.PythonCustomizedContainerTask` as well.
+
+    See the :ref:`integrations guide <cookbook:integrations_sql_sqlite3>` for additional usage examples and
+    the base class {{< py_class_ref flytekit.extend.PythonCustomizedContainerTask >}} as well.
     """
 
     _SQLITE_TASK_TYPE = "sqlite"
@@ -78,12 +94,14 @@ class SQLite3Task(PythonCustomizedContainerTask[SQLite3Config], SQLTask[SQLite3C
         query_template: str,
         inputs: typing.Optional[typing.Dict[str, typing.Type]] = None,
         task_config: typing.Optional[SQLite3Config] = None,
-        output_schema_type: typing.Optional[typing.Type[FlyteSchema]] = None,
+        output_schema_type: typing.Optional[typing.Type["FlyteSchema"]] = None,  # type: ignore
         container_image: typing.Optional[str] = None,
         **kwargs,
     ):
         if task_config is None or task_config.uri is None:
             raise ValueError("SQLite DB uri is required.")
+        from flytekit.types.schema import FlyteSchema
+
         outputs = kwtypes(results=output_schema_type if output_schema_type else FlyteSchema)
         super().__init__(
             name=name,
