@@ -1,20 +1,38 @@
 import datetime
 import os
+import platform
 import string
 import subprocess
 import typing
 from dataclasses import dataclass
+from typing import List
 
 import flytekit
 from flytekit.core.context_manager import ExecutionParameters
 from flytekit.core.interface import Interface
 from flytekit.core.python_function_task import PythonInstanceTask
 from flytekit.core.task import TaskPlugins
+from flytekit.exceptions.user import FlyteRecoverableException
 from flytekit.loggers import logger
 from flytekit.types.directory import FlyteDirectory
 from flytekit.types.file import FlyteFile
 
 
+@dataclass
+class ProcessResult:
+    """Stores a process return code, standard output and standard error.
+
+    Args:
+        returncode: int The sub-process return code
+        output: str The sub-process standard output string
+        error: str The sub-process standard error string
+    """
+
+    returncode: int
+    output: str
+    error: str
+
+
 @dataclass
 class OutputLocation:
     """
@@ -31,6 +49,68 @@ class OutputLocation:
     location: typing.Union[os.PathLike, str]
 
 
+def subproc_execute(command: typing.Union[List[str], str], **kwargs) -> ProcessResult:
+    """
+    Execute a command and capture its stdout and stderr. Useful for executing
+    shell commands from within a python task.
+
+    Args:
+        command (List[str]): The command to be executed as a list of strings.
+
+    Returns:
+        ProcessResult: Structure containing output of the command.
+
+    Raises:
+        Exception: If the command execution fails, this exception is raised with
+            details about the command, return code, and stderr output.
+        Exception: If the executable is not found, this exception is raised with
+            guidance on specifying a container image in the task definition when
+            using custom dependencies.
+    """
+    defaults = {
+        "stdout": subprocess.PIPE,
+        "stderr": subprocess.PIPE,
+        "text": True,
+        "check": True,
+    }
+
+    kwargs = {**defaults, **kwargs}
+
+    if kwargs.get("shell"):
+        if "|" in command:
+            logger.warning(
+                """Found a pipe in the command and shell=True.
+                This can lead to silent failures if subsequent commands
+                succeed despite previous failures."""
+            )
+        if type(command) == list:
+            logger.warning(
+                """Found `command` formatted as a list instead of a string with shell=True.
+                With this configuration, the first member of the list will be
+                executed and the remaining arguments will be passed as arguments
+                to the shell instead of to the binary being called. This may not
+                be intended behavior and may lead to confusing failures."""
+            )
+
+    try:
+        # Execute the command and capture stdout and stderr
+        result = subprocess.run(command, **kwargs)
+        result.check_returncode()
+
+        # Access the stdout and stderr output
+        return ProcessResult(result.returncode, result.stdout, result.stderr)
+
+    except subprocess.CalledProcessError as e:
+        raise RuntimeError(f"Command: {e.cmd}\nFailed with return code {e.returncode}:\n{e.stderr}")
+
+    except FileNotFoundError as e:
+        raise RuntimeError(
+            f"""Process failed because the executable could not be found.
+            Did you specify a container image in the task definition if using
+            custom dependencies?\n{e}"""
+        )
+
+
 def _dummy_task_func():
     """
     A Fake function to satisfy the inner PythonTask requirements
@@ -97,6 +177,40 @@ class _PythonFStringInterpolizer:
 T = typing.TypeVar("T")
 
 
+def _run_script(script: str, shell: str) -> ProcessResult:
+    """
+    Run script as a subprocess and return the returncode, stdout, and stderr.
+
+    While executing the su process, stdout of the subprocess will be printed
+    to the current process stdout so that the subprocess execution will not appear unresponsive
+
+    :param script: script to be executed
+    :type script: str
+    :param shell: shell to use to run the script
+    :type shell: str
+    :return: structure containing the process returncode, stdout (stripped from carriage returns), and stderr
+    :rtype: ProcessResult
+    """
+    process = subprocess.Popen(
+        script,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        bufsize=0,
+        shell=True,
+        text=True,
+        executable=shell,
+    )
+
+    process_stdout, process_stderr = process.communicate()
+    out = ""
+    for line in process_stdout.splitlines():
+        print(line)
+        out += line
+
+    code = process.wait()
+    return ProcessResult(code, out, process_stderr)
+
+
 class ShellTask(PythonInstanceTask[T]):
     """ """
 
@@ -107,6 +221,7 @@ class ShellTask(PythonInstanceTask[T]):
         script: typing.Optional[str] = None,
         script_file: typing.Optional[str] = None,
         task_config: T = None,
+        shell: str = "/bin/sh",
         inputs: typing.Optional[typing.Dict[str, typing.Type]] = None,
         output_locs: typing.Optional[typing.List[OutputLocation]] = None,
         **kwargs,
@@ -117,10 +232,12 @@ class ShellTask(PythonInstanceTask[T]):
             debug: bool Print the generated script and other debugging information
             script: The actual script specified as a string
             script_file: A path to the file that contains the script (Only script or script_file) can be provided
-            task_config: T Configuration for the task, can be either a Pod (or coming soon, BatchJob) config
+            task_config: Configuration for the task, can be either a Pod (or coming soon, BatchJob) config
+            shell: Shell to use to run the script
             inputs: A Dictionary of input names to types
-            output_locs: A list of :py:class:`OutputLocations`
-            **kwargs: Other arguments that can be passed to :ref:class:`PythonInstanceTask`
+            output_locs: A list of {{< py_class_ref OutputLocations >}}
+            **kwargs: Other arguments that can be passed to
+                {{< py_class_ref flytekit.core.python_function_task.PythonInstanceTask >}}
         """
         if script and script_file:
             raise ValueError("Only either of script or script_file can be provided")
@@ -150,9 +267,11 @@ class ShellTask(PythonInstanceTask[T]):
         self._script = script
         self._script_file = script_file
         self._debug = debug
+        self._shell = shell
         self._output_locs = output_locs if output_locs else []
         self._interpolizer = _PythonFStringInterpolizer()
         outputs = self._validate_output_locs()
+        self._process_result: typing.Optional[ProcessResult] = None
         super().__init__(
             name,
             task_config,
@@ -177,6 +296,10 @@ class ShellTask(PythonInstanceTask[T]):
             outputs[v.var] = v.var_type
         return outputs
 
+    @property
+    def result(self) -> typing.Optional[ProcessResult]:
+        return self._process_result
+
     @property
     def script(self) -> typing.Optional[str]:
         return self._script
@@ -211,18 +334,25 @@ class ShellTask(PythonInstanceTask[T]):
             print(gen_script)
             print("\n==============================================\n")
 
-        try:
-            subprocess.check_call(gen_script, shell=True)
-        except subprocess.CalledProcessError as e:
+        if platform.system() == "Windows":
+            if os.environ.get("ComSpec") is None:
+                # https://github.com/python/cpython/issues/101283
+                os.environ["ComSpec"] = "C:\\Windows\\System32\\cmd.exe"
+            self._shell = os.environ["ComSpec"]
+
+        self._process_result = _run_script(gen_script, self._shell)
+        if self._process_result.returncode != 0:
             files = os.listdir(".")
             fstr = "\n-".join(files)
-            logger.error(
-                f"Failed to Execute Script, return-code {e.returncode} \n"
-                f"StdErr: {e.stderr}\n"
-                f"StdOut: {e.stdout}\n"
-                f" Current directory contents: .\n-{fstr}"
+            error = (
+                f"Failed to Execute Script, return-code {self._process_result.returncode} \n"
+                f"Current directory contents: .\n-{fstr}\n"
+                f"StdOut: {self._process_result.output}\n"
+                f"StdErr: {self._process_result.error}\n"
             )
-            raise
+            logger.error(error)
+            # raise FlyteRecoverableException so that it's classified as user error and will be retried
+            raise FlyteRecoverableException(error)
 
         final_outputs = []
         for v in self._output_locs:
@@ -264,12 +394,12 @@ class RawShellTask(ShellTask):
         template. The template itself will export the desired environment variables, and subsequently execute the
         desired "raw" script with the specified arguments.
 
-        .. note::
-            This means that within your workflow, you can dynamically control the env variables, arguments, and even the
+        > [!NOTE]
+        > This means that within your workflow, you can dynamically control the env variables, arguments, and even the
             actual script you want to run.
 
-        .. note::
-            The downside is that a dynamic workflow will be required. The "raw" script passed in at execution time must
+        > [!NOTE]
+        > The downside is that a dynamic workflow will be required. The "raw" script passed in at execution time must
             be at the specified location.
 
         These args are forwarded directly to the parent `ShellTask` constructor as behavior does not diverge
@@ -355,7 +485,6 @@ class RawShellTask(ShellTask):
 # This utility function allows for the specification of env variables, arguments, and the actual script within the
 # workflow definition rather than at `RawShellTask` instantiation
 def get_raw_shell_task(name: str) -> RawShellTask:
-
     return RawShellTask(
         name=name,
         debug=True,
