@@ -1,12 +1,5 @@
 """
-.. currentmodule:: flytekit.extras.tensorflow
-
-.. autosummary::
-   :template: custom.rst
-   :toctree: generated/
-
-   TensorFlowRecordFileTransformer
-   TensorFlowRecordsDirTransformer
+This module provides functionality related to Tensorflow
 """
 
 from flytekit.loggers import logger
@@ -26,9 +19,10 @@ except (ImportError, OSError):
 
 
 if _tensorflow_installed:
+    from .model import TensorFlowModelTransformer
     from .record import TensorFlowRecordFileTransformer, TensorFlowRecordsDirTransformer
 else:
     logger.info(
-        "We won't register TensorFlowRecordFileTransformer and TensorFlowRecordsDirTransformer "
+        "We won't register TensorFlowRecordFileTransformer, TensorFlowRecordsDirTransformer and TensorFlowModelTransformer"
         "because tensorflow is not installed."
     )
