@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+import typing
 
 from flyteidl.admin import workflow_pb2 as _admin_workflow
 
@@ -6,10 +6,16 @@ from flytekit.models import common as _common
 from flytekit.models.core import compiler as _compiler_models
 from flytekit.models.core import identifier as _identifier
 from flytekit.models.core import workflow as _core_workflow
+from flytekit.models.documentation import Documentation
 
 
 class WorkflowSpec(_common.FlyteIdlEntity):
-    def __init__(self, template, sub_workflows):
+    def __init__(
+        self,
+        template: _core_workflow.WorkflowTemplate,
+        sub_workflows: typing.List[_core_workflow.WorkflowTemplate],
+        docs: typing.Optional[Documentation] = None,
+    ):
         """
         This object fully encapsulates the specification of a workflow
         :param flytekit.models.core.workflow.WorkflowTemplate template:
@@ -17,27 +23,39 @@ class WorkflowSpec(_common.FlyteIdlEntity):
         """
         self._template = template
         self._sub_workflows = sub_workflows
+        self._docs = docs
 
     @property
     def template(self):
         """
-        :rtype: flytekit.models.core.workflow.WorkflowTemplate.WorkflowTemplate
+        :rtype: flytekit.models.core.workflow.WorkflowTemplate
         """
         return self._template
 
     @property
     def sub_workflows(self):
         """
-        :rtype: list[flytekit.models.core.workflow.WorkflowTemplate.WorkflowTemplate]
+        :rtype: list[flytekit.models.core.workflow.WorkflowTemplate]
         """
         return self._sub_workflows
 
+    @property
+    def docs(self):
+        """
+        :rtype: Description entity for the workflow
+        """
+        return self._docs
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.workflow_pb2.WorkflowSpec
         """
+        subwfs = sorted(self._sub_workflows, key=lambda x: x.id.name)
+
         return _admin_workflow.WorkflowSpec(
-            template=self._template.to_flyte_idl(), sub_workflows=[s.to_flyte_idl() for s in self._sub_workflows],
+            template=self._template.to_flyte_idl(),
+            sub_workflows=[s.to_flyte_idl() for s in subwfs],
+            description=self._docs.to_flyte_idl() if self._docs else None,
         )
 
     @classmethod
@@ -49,17 +67,19 @@ class WorkflowSpec(_common.FlyteIdlEntity):
         return cls(
             _core_workflow.WorkflowTemplate.from_flyte_idl(pb2_object.template),
             [_core_workflow.WorkflowTemplate.from_flyte_idl(s) for s in pb2_object.sub_workflows],
+            Documentation.from_flyte_idl(pb2_object.description) if pb2_object.description else None,
         )
 
 
 class Workflow(_common.FlyteIdlEntity):
-    def __init__(self, id, closure):
+    def __init__(self, id, closure, short_description=None):
         """
         :param flytekit.models.core.identifier.Identifier id:
         :param WorkflowClosure closure:
         """
         self._id = id
         self._closure = closure
+        self._short_description = short_description
 
     @property
     def id(self):
@@ -75,11 +95,22 @@ class Workflow(_common.FlyteIdlEntity):
         """
         return self._closure
 
+    @property
+    def short_description(self):
+        """
+        :rtype: str
+        """
+        return self._short_description
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.workflow_pb2.Workflow
         """
-        return _admin_workflow.Workflow(id=self.id.to_flyte_idl(), closure=self.closure.to_flyte_idl())
+        return _admin_workflow.Workflow(
+            id=self.id.to_flyte_idl(),
+            closure=self.closure.to_flyte_idl(),
+            short_description=self.short_description,
+        )
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -90,6 +121,7 @@ class Workflow(_common.FlyteIdlEntity):
         return cls(
             id=_identifier.Identifier.from_flyte_idl(pb2_object.id),
             closure=WorkflowClosure.from_flyte_idl(pb2_object.closure),
+            short_description=pb2_object.short_description,
         )
 
 
