@@ -1,12 +1,18 @@
-import abc as _abc
-import json as _json
+import abc
+import json
+import os
+from contextlib import closing
+from io import StringIO
+from textwrap import shorten
+from typing import Dict
 
 from flyteidl.admin import common_pb2 as _common_pb2
+from flyteidl.core import literals_pb2 as _literals_pb2
 from google.protobuf import json_format as _json_format
 from google.protobuf import struct_pb2 as _struct
 
 
-class FlyteABCMeta(_abc.ABCMeta):
+class FlyteABCMeta(abc.ABCMeta):
     def __instancecheck__(cls, instance):
         if cls in type(instance).__mro__:
             return True
@@ -32,11 +38,34 @@ class FlyteType(FlyteABCMeta):
         """
         return cls.short_class_string()
 
-    @_abc.abstractmethod
+    @abc.abstractmethod
     def from_flyte_idl(cls, idl_object):
         pass
 
 
+def _repr_idl_yaml_like(idl, indent=0) -> str:
+    """Formats an IDL into a YAML-like representation."""
+    if not hasattr(idl, "ListFields"):
+        return str(idl)
+
+    with closing(StringIO()) as out:
+        for descriptor, field in idl.ListFields():
+            try:
+                inner_fields = field.ListFields()
+                # if inner_fields is empty, then we do not render the descriptor,
+                # because it is empty
+                if inner_fields:
+                    out.write(" " * indent + descriptor.name + ":" + os.linesep)
+                    out.write(_repr_idl_yaml_like(field, indent + 2))
+            except AttributeError:
+                # No ListFields -> Must be a scalar
+                str_repr = shorten(str(field).strip(), width=80)
+                if str_repr:
+                    out.write(" " * indent + descriptor.name + ": " + str_repr + os.linesep)
+
+        return out.getvalue()
+
+
 class FlyteIdlEntity(object, metaclass=FlyteType):
     def __eq__(self, other):
         return isinstance(other, FlyteIdlEntity) and other.to_flyte_idl() == self.to_flyte_idl()
@@ -57,7 +86,9 @@ class FlyteIdlEntity(object, metaclass=FlyteType):
         """
         :rtype: Text
         """
-        return str(self.to_flyte_idl())
+        str_repr = _repr_idl_yaml_like(self.to_flyte_idl(), indent=2).rstrip(os.linesep)
+        type_str = type(self).__name__
+        return f"Flyte Serialized object ({type_str}):" + os.linesep + str_repr
 
     def verbose_string(self):
         """
@@ -68,11 +99,19 @@ class FlyteIdlEntity(object, metaclass=FlyteType):
     def serialize_to_string(self) -> str:
         return self.to_flyte_idl().SerializeToString()
 
+    def _repr_html_(self) -> str:
+        """HTML repr for object."""
+        # `_repr_html_` is used by Jupyter to render objects
+        type_str = type(self).__name__
+        idl = self.to_flyte_idl()
+        str_repr = _repr_idl_yaml_like(idl).rstrip(os.linesep)
+        return f"<h4>{type_str}</h4><pre>{str_repr}</pre>"
+
     @property
     def is_empty(self):
         return len(self.to_flyte_idl().SerializeToString()) == 0
 
-    @_abc.abstractmethod
+    @abc.abstractmethod
     def to_flyte_idl(self):
         pass
 
@@ -88,13 +127,13 @@ class FlyteCustomIdlEntity(FlyteIdlEntity):
         return cls.from_dict(idl_dict=_json_format.MessageToDict(idl_object))
 
     def to_flyte_idl(self):
-        return _json_format.Parse(_json.dumps(self.to_dict()), _struct.Struct())
+        return _json_format.Parse(json.dumps(self.to_dict()), _struct.Struct())
 
-    @_abc.abstractmethod
+    @abc.abstractmethod
     def from_dict(self, idl_dict):
         pass
 
-    @_abc.abstractmethod
+    @abc.abstractmethod
     def to_dict(self):
         """
         Converts self to a dictionary.
@@ -414,8 +453,10 @@ class UrlBlob(FlyteIdlEntity):
 
 class AuthRole(FlyteIdlEntity):
     def __init__(self, assumable_iam_role=None, kubernetes_service_account=None):
-        """
+        """Auth configuration for IAM or K8s service account.
+
         Either one or both of the assumable IAM role and/or the K8s service account can be set.
+
         :param Text assumable_iam_role: IAM identity with set permissions policies.
         :param Text kubernetes_service_account: Provides an identity for workflow execution resources.
           Flyte deployment administrators are responsible for handling permissions as they
@@ -481,3 +522,19 @@ class RawOutputDataConfig(FlyteIdlEntity):
     @classmethod
     def from_flyte_idl(cls, pb2):
         return cls(output_location_prefix=pb2.output_location_prefix)
+
+
+class Envs(FlyteIdlEntity):
+    def __init__(self, envs: Dict[str, str]):
+        self._envs = envs
+
+    @property
+    def envs(self) -> Dict[str, str]:
+        return self._envs
+
+    def to_flyte_idl(self) -> _common_pb2.Envs:
+        return _common_pb2.Envs(values=[_literals_pb2.KeyValuePair(key=k, value=v) for k, v in self.envs.items()])
+
+    @classmethod
+    def from_flyte_idl(cls, pb2: _common_pb2.Envs) -> _common_pb2.Envs:
+        return cls(envs={kv.key: kv.value for kv in pb2.values})
