@@ -1,22 +1,29 @@
 import datetime
+import json
 import typing
 
+from flyteidl.core import tasks_pb2
 from flyteidl.core import workflow_pb2 as _core_workflow
+from google.protobuf import json_format, struct_pb2
+from google.protobuf.wrappers_pb2 import BoolValue
 
+from flytekit.core.pod_template import PodTemplate
 from flytekit.models import common as _common
 from flytekit.models import interface as _interface
 from flytekit.models import types as type_models
 from flytekit.models.core import condition as _condition
+from flytekit.models.core import identifier
 from flytekit.models.core import identifier as _identifier
 from flytekit.models.literals import Binding as _Binding
 from flytekit.models.literals import RetryStrategy as _RetryStrategy
-from flytekit.models.task import Resources
+from flytekit.models.task import K8sObjectMetadata, Resources
 
 
 class IfBlock(_common.FlyteIdlEntity):
     def __init__(self, condition, then_node):
         """
         Defines a condition and the execution unit that should be executed if the condition is satisfied.
+
         :param flytekit.models.core.condition.BooleanExpression condition:
         :param Node then_node:
         """
@@ -72,6 +79,7 @@ class IfElseBlock(_common.FlyteIdlEntity):
     def case(self):
         """
         First condition to evaluate.
+
         :rtype: IfBlock
         """
 
@@ -81,6 +89,7 @@ class IfElseBlock(_common.FlyteIdlEntity):
     def other(self):
         """
         Additional branches to evaluate.
+
         :rtype: list[IfBlock]
         """
 
@@ -90,6 +99,7 @@ class IfElseBlock(_common.FlyteIdlEntity):
     def else_node(self):
         """
         The node to execute in case none of the branches were taken.
+
         :rtype: Node
         """
 
@@ -99,6 +109,7 @@ class IfElseBlock(_common.FlyteIdlEntity):
     def error(self):
         """
         An error to throw in case none of the branches were taken.
+
         :rtype: flytekit.models.types.Error
         """
 
@@ -129,7 +140,8 @@ class BranchNode(_common.FlyteIdlEntity):
     def __init__(self, if_else: IfElseBlock):
         """
         BranchNode is a special node that alter the flow of the workflow graph. It allows the control flow to branch at
-        runtime based on a series of conditions that get evaluated on various parameters (e.g. inputs, primtives).
+        runtime based on a series of conditions that get evaluated on various parameters (e.g. inputs, primitives).
+
         :param IfElseBlock if_else:
         """
 
@@ -155,19 +167,34 @@ class BranchNode(_common.FlyteIdlEntity):
 
 
 class NodeMetadata(_common.FlyteIdlEntity):
-    def __init__(self, name, timeout=None, retries=None, interruptible=None):
+    def __init__(
+        self,
+        name,
+        timeout=None,
+        retries=None,
+        interruptible: typing.Optional[bool] = None,
+        cacheable: typing.Optional[bool] = None,
+        cache_version: typing.Optional[str] = None,
+        cache_serializable: typing.Optional[bool] = None,
+    ):
         """
         Defines extra information about the Node.
 
         :param Text name: Friendly name for the Node.
         :param datetime.timedelta timeout: [Optional] Overall timeout for a task.
         :param flytekit.models.literals.RetryStrategy retries: [Optional] Number of retries per task.
-        :param bool interruptible: [Optional] Can be safely interrupted during execution.
+        :param bool interruptible: Can be safely interrupted during execution.
+        :param cacheable: Indicates that this nodes outputs should be cached.
+        :param cache_version: The version of the cached data.
+        :param cacheable: Indicates that cache operations on this node should be serialized.
         """
         self._name = name
         self._timeout = timeout if timeout is not None else datetime.timedelta()
         self._retries = retries if retries is not None else _RetryStrategy(0)
         self._interruptible = interruptible
+        self._cacheable = cacheable
+        self._cache_version = cache_version
+        self._cache_serializable = cache_serializable
 
     @property
     def name(self):
@@ -191,12 +218,21 @@ class NodeMetadata(_common.FlyteIdlEntity):
         return self._retries
 
     @property
-    def interruptible(self):
-        """
-        :rtype: flytekit.models.
-        """
+    def interruptible(self) -> typing.Optional[bool]:
         return self._interruptible
 
+    @property
+    def cacheable(self) -> typing.Optional[bool]:
+        return self._cacheable
+
+    @property
+    def cache_version(self) -> typing.Optional[str]:
+        return self._cache_version
+
+    @property
+    def cache_serializable(self) -> typing.Optional[bool]:
+        return self._cache_serializable
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.workflow_pb2.NodeMetadata
@@ -205,6 +241,9 @@ class NodeMetadata(_common.FlyteIdlEntity):
             name=self.name,
             retries=self.retries.to_flyte_idl(),
             interruptible=self.interruptible,
+            cacheable=self.cacheable,
+            cache_version=self.cache_version,
+            cache_serializable=self.cache_serializable,
         )
         if self.timeout:
             node_metadata.timeout.FromTimedelta(self.timeout)
@@ -216,6 +255,10 @@ class NodeMetadata(_common.FlyteIdlEntity):
             pb2_object.name,
             pb2_object.timeout.ToTimedelta(),
             _RetryStrategy.from_flyte_idl(pb2_object.retries),
+            pb2_object.interruptible if pb2_object.HasField("interruptible") else None,
+            pb2_object.cacheable if pb2_object.HasField("cacheable") else None,
+            pb2_object.cache_version if pb2_object.HasField("cache_version") else None,
+            pb2_object.cache_serializable if pb2_object.HasField("cache_serializable") else None,
         )
 
 
@@ -223,6 +266,7 @@ class SignalCondition(_common.FlyteIdlEntity):
     def __init__(self, signal_id: str, type: type_models.LiteralType, output_variable_name: str):
         """
         Represents a dependency on an signal from a user.
+
         :param signal_id: The node id of the signal, also the signal name.
         :param type:
         """
@@ -260,6 +304,7 @@ class ApproveCondition(_common.FlyteIdlEntity):
     def __init__(self, signal_id: str):
         """
         Represents a dependency on an signal from a user.
+
         :param signal_id: The node id of the signal, also the signal name.
         """
         self._signal_id = signal_id
@@ -340,6 +385,57 @@ class GateNode(_common.FlyteIdlEntity):
         )
 
 
+class ArrayNode(_common.FlyteIdlEntity):
+    def __init__(
+        self,
+        node: "Node",
+        parallelism=None,
+        min_successes=None,
+        min_success_ratio=None,
+        execution_mode=None,
+        is_original_sub_node_interface=False,
+        data_mode=None,
+        bound_inputs=None,
+    ) -> None:
+        """
+        TODO: docstring
+        """
+        self._node = node
+        self._parallelism = parallelism
+        # TODO either min_successes or min_success_ratio should be set
+        self._min_successes = min_successes
+        self._min_success_ratio = min_success_ratio
+        self._execution_mode = execution_mode
+        self._is_original_sub_node_interface = is_original_sub_node_interface
+        self._data_mode = data_mode
+        self._bound_inputs = bound_inputs
+
+    @property
+    def node(self) -> "Node":
+        return self._node
+
+    def to_flyte_idl(self) -> _core_workflow.ArrayNode:
+        return _core_workflow.ArrayNode(
+            node=self._node.to_flyte_idl() if self._node is not None else None,
+            parallelism=self._parallelism,
+            min_successes=self._min_successes,
+            min_success_ratio=self._min_success_ratio,
+            execution_mode=self._execution_mode,
+            is_original_sub_node_interface=BoolValue(value=self._is_original_sub_node_interface),
+            data_mode=self._data_mode,
+            bound_inputs=sorted(self._bound_inputs) if self._bound_inputs else None,
+        )
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object) -> "ArrayNode":
+        return cls(
+            Node.from_flyte_idl(pb2_object.node),
+            pb2_object.parallelism,
+            pb2_object.min_successes,
+            pb2_object.min_success_ratio,
+        )
+
+
 class Node(_common.FlyteIdlEntity):
     def __init__(
         self,
@@ -352,6 +448,7 @@ class Node(_common.FlyteIdlEntity):
         workflow_node=None,
         branch_node=None,
         gate_node: typing.Optional[GateNode] = None,
+        array_node: typing.Optional[ArrayNode] = None,
     ):
         """
         A Workflow graph Node. One unit of execution in the graph. Each node can be linked to a Task,
@@ -383,12 +480,14 @@ class Node(_common.FlyteIdlEntity):
         self._workflow_node = workflow_node
         self._branch_node = branch_node
         self._gate_node = gate_node
+        self._array_node = array_node
 
     @property
     def id(self):
         """
         A workflow-level unique identifier that identifies this node in the workflow. "inputs" and
         "outputs" are reserved node ids that cannot be used by other nodes.
+
         :rtype: Text
         """
         return self._id
@@ -397,6 +496,7 @@ class Node(_common.FlyteIdlEntity):
     def metadata(self):
         """
         Extra metadata about the node.
+
         :rtype: NodeMetadata
         """
         return self._metadata
@@ -406,6 +506,7 @@ class Node(_common.FlyteIdlEntity):
         """
         Specifies how to bind the underlying interface's inputs.  All required inputs specified
         in the underlying interface must be fulfilled.
+
         :rtype: list[flytekit.models.literals.Binding]
         """
         return self._inputs
@@ -416,6 +517,7 @@ class Node(_common.FlyteIdlEntity):
         [Optional] Specifies execution dependency for this node ensuring it will
         only get scheduled to run after all its upstream nodes have completed. This node will have
         an implicit dependency on any node that appears in inputs field.
+
         :rtype: list[Text]
         """
         return self._upstream_node_ids
@@ -426,6 +528,7 @@ class Node(_common.FlyteIdlEntity):
         [Optional] A node can define aliases for a subset of its outputs. This
         is particularly useful if different nodes need to conform to the same interface (e.g. all branches in
         a branch node). Downstream nodes must refer to this node's outputs using the alias if one is specified.
+
         :rtype: list[Alias]
         """
         return self._output_aliases
@@ -434,6 +537,7 @@ class Node(_common.FlyteIdlEntity):
     def task_node(self):
         """
         [Optional] Information about the Task to execute in this node.
+
         :rtype: TaskNode
         """
         return self._task_node
@@ -442,6 +546,7 @@ class Node(_common.FlyteIdlEntity):
     def workflow_node(self):
         """
         [Optional] Information about the Workflow to execute in this mode.
+
         :rtype: WorkflowNode
         """
         return self._workflow_node
@@ -450,6 +555,7 @@ class Node(_common.FlyteIdlEntity):
     def branch_node(self):
         """
         [Optional] Information about the branch node to evaluate in this node.
+
         :rtype: BranchNode
         """
         return self._branch_node
@@ -458,6 +564,10 @@ class Node(_common.FlyteIdlEntity):
     def gate_node(self) -> typing.Optional[GateNode]:
         return self._gate_node
 
+    @property
+    def array_node(self) -> typing.Optional[ArrayNode]:
+        return self._array_node
+
     @property
     def target(self):
         """
@@ -479,6 +589,7 @@ class Node(_common.FlyteIdlEntity):
             workflow_node=self.workflow_node.to_flyte_idl() if self.workflow_node is not None else None,
             branch_node=self.branch_node.to_flyte_idl() if self.branch_node is not None else None,
             gate_node=self.gate_node.to_flyte_idl() if self.gate_node else None,
+            array_node=self.array_node.to_flyte_idl() if self.array_node else None,
         )
 
     @classmethod
@@ -501,39 +612,87 @@ class Node(_common.FlyteIdlEntity):
             if pb2_object.HasField("branch_node")
             else None,
             gate_node=GateNode.from_flyte_idl(pb2_object.gate_node) if pb2_object.HasField("gate_node") else None,
+            array_node=ArrayNode.from_flyte_idl(pb2_object.array_node) if pb2_object.HasField("array_node") else None,
         )
 
 
 class TaskNodeOverrides(_common.FlyteIdlEntity):
-    def __init__(self, resources: typing.Optional[Resources] = None):
+    def __init__(
+        self,
+        resources: typing.Optional[Resources],
+        extended_resources: typing.Optional[tasks_pb2.ExtendedResources],
+        container_image: typing.Optional[str] = None,
+        pod_template: typing.Optional[PodTemplate] = None,
+    ):
         self._resources = resources
+        self._extended_resources = extended_resources
+        self._container_image = container_image
+        self._pod_template = pod_template
 
     @property
     def resources(self) -> Resources:
         return self._resources
 
+    @property
+    def extended_resources(self) -> tasks_pb2.ExtendedResources:
+        return self._extended_resources
+
+    @property
+    def container_image(self) -> typing.Optional[str]:
+        return self._container_image
+
+    @property
+    def pod_template(self) -> typing.Optional[PodTemplate]:
+        return self._pod_template
+
     def to_flyte_idl(self):
+        pod_template_override = None
+        if self.pod_template is not None:
+            pod_template_override = tasks_pb2.K8sPod(
+                metadata=K8sObjectMetadata(
+                    labels=self.pod_template.labels,
+                    annotations=self.pod_template.annotations,
+                ).to_flyte_idl(),
+                pod_spec=json_format.Parse(json.dumps(self.pod_template.pod_spec), struct_pb2.Struct()),
+                primary_container_name=self.pod_template.primary_container_name,
+            )
         return _core_workflow.TaskNodeOverrides(
             resources=self.resources.to_flyte_idl() if self.resources is not None else None,
+            extended_resources=self.extended_resources,
+            container_image=self.container_image,
+            pod_template=pod_template_override,
         )
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
         resources = Resources.from_flyte_idl(pb2_object.resources)
+        extended_resources = pb2_object.extended_resources if pb2_object.HasField("extended_resources") else None
+        container_image = pb2_object.container_image if len(pb2_object.container_image) > 0 else None
+        pod_template = pb2_object.pod_template if pb2_object.HasField("pod_template") else None
         if bool(resources.requests) or bool(resources.limits):
-            return cls(resources=resources)
-        return cls(resources=None)
+            return cls(
+                resources=resources,
+                extended_resources=extended_resources,
+                container_image=container_image,
+                pod_template=pod_template,
+            )
+        return cls(
+            resources=None,
+            extended_resources=extended_resources,
+            container_image=container_image,
+            pod_template=pod_template,
+        )
 
 
 class TaskNode(_common.FlyteIdlEntity):
     def __init__(self, reference_id, overrides: typing.Optional[TaskNodeOverrides] = None):
         """
         Refers to the task that the Node is to execute.
-        NB: This is currently a oneof in protobuf, but there's only one option currently.  This code should be updated
-            when more options are available.
+        This is currently a oneof in protobuf, but there's only one option currently.
+        This code should be updated when more options are available.
 
         :param flytekit.models.core.identifier.Identifier reference_id: A globally unique identifier for the task.
-        :param flyteidl.core.workflow_pb2.TaskNodeOverrides
+        :param flyteidl.core.workflow_pb2.TaskNodeOverrides:
         """
         self._reference_id = reference_id
         self._overrides = overrides
@@ -541,7 +700,8 @@ class TaskNode(_common.FlyteIdlEntity):
     @property
     def reference_id(self):
         """
-        A globally unique identifier for the task.  This should map to the identifier in Flyte Admin.
+        A globally unique identifier for the task. This should map to the identifier in Flyte Admin.
+
         :rtype: flytekit.models.core.identifier.Identifier
         """
         return self._reference_id
@@ -577,10 +737,10 @@ class TaskNode(_common.FlyteIdlEntity):
 class WorkflowNode(_common.FlyteIdlEntity):
     def __init__(self, launchplan_ref=None, sub_workflow_ref=None):
         """
-        Refers to a the workflow the node is to execute.  One of the references must be supplied.
+        Refers to a the workflow the node is to execute. One of the references must be supplied.
 
         :param flytekit.models.core.identifier.Identifier launchplan_ref: [Optional] A globally unique identifier for
-            the launch plan.  Should map to Admin.
+            the launch plan. Should map to Admin.
         :param flytekit.models.core.identifier.Identifier sub_workflow_ref: [Optional] Reference to a subworkflow,
             that should be defined with the compiler context.
         """
@@ -588,9 +748,10 @@ class WorkflowNode(_common.FlyteIdlEntity):
         self._sub_workflow_ref = sub_workflow_ref
 
     @property
-    def launchplan_ref(self):
+    def launchplan_ref(self) -> identifier.Identifier:
         """
         [Optional] A globally unique identifier for the launch plan.  Should map to Admin.
+
         :rtype: flytekit.models.core.identifier.Identifier
         """
         return self._launchplan_ref
@@ -599,6 +760,7 @@ class WorkflowNode(_common.FlyteIdlEntity):
     def sub_workflow_ref(self):
         """
         [Optional] Reference to a subworkflow, that should be defined with the compiler context.
+
         :rtype: flytekit.models.core.identifier.Identifier
         """
         return self._sub_workflow_ref
@@ -623,6 +785,7 @@ class WorkflowNode(_common.FlyteIdlEntity):
     def from_flyte_idl(cls, pb2_object):
         """
         :param flyteidl.core.workflow_pb2.WorkflowNode pb2_object:
+
         :rtype: WorkflowNode
         """
         if pb2_object.HasField("launchplan_ref"):
@@ -654,6 +817,7 @@ class WorkflowMetadata(_common.FlyteIdlEntity):
     def __init__(self, on_failure=None):
         """
         Metadata for the workflow.
+
         :param on_failure flytekit.models.core.workflow.WorkflowMetadata.OnFailurePolicy: [Optional] The execution policy when the workflow detects a failure.
         """
         self._on_failure = on_failure
@@ -678,6 +842,7 @@ class WorkflowMetadata(_common.FlyteIdlEntity):
     def from_flyte_idl(cls, pb2_object):
         """
         :param flyteidl.core.workflow_pb2.WorkflowMetadata pb2_object:
+
         :rtype: WorkflowMetadata
         """
         return cls(
@@ -708,6 +873,7 @@ class WorkflowMetadataDefaults(_common.FlyteIdlEntity):
     def from_flyte_idl(cls, pb2_object):
         """
         :param flyteidl.core.workflow_pb2.WorkflowMetadataDefaults pb2_object:
+
         :rtype: WorkflowMetadata
         """
         return cls(interruptible=pb2_object.interruptible)
@@ -758,6 +924,7 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
     def id(self):
         """
         This is an autogenerated id by the system. The id is globally unique across Flyte.
+
         :rtype: flytekit.models.core.identifier.Identifier
         """
         return self._id
@@ -766,6 +933,7 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
     def metadata(self):
         """
         This contains information on how to run the workflow.
+
         :rtype: WorkflowMetadata
         """
         return self._metadata
@@ -774,6 +942,7 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
     def metadata_defaults(self):
         """
         This contains information on how to run the workflow.
+
         :rtype: WorkflowMetadataDefaults
         """
         return self._metadata_defaults
@@ -783,6 +952,7 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
         """
         Defines a strongly typed interface for the Workflow (inputs, outputs). This can include some optional
         parameters.
+
         :rtype: flytekit.models.interface.TypedInterface
         """
         return self._interface
@@ -791,7 +961,8 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
     def nodes(self):
         """
         A list of nodes. In addition, "globals" is a special reserved node id that can be used to consume
-        workflow inputs
+        workflow inputs.
+
         :rtype: list[Node]
         """
         return self._nodes
@@ -803,6 +974,7 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
         pull node outputs or specify literals. All workflow outputs specified in the interface field must be bound
         in order for the workflow to be validated. A workflow has an implicit dependency on all of its nodes
         to execute successfully in order to bind final outputs.
+
         :rtype: list[flytekit.models.literals.Binding]
         """
         return self._outputs
@@ -813,6 +985,7 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
         Node failure_node: A catch-all node. This node is executed whenever the execution engine determines the
         workflow has failed. The interface of this node must match the Workflow interface with an additional input
         named "error" of type pb.lyft.flyte.core.Error.
+
         :rtype: Node
         """
         return self._failure_node
@@ -835,6 +1008,7 @@ class WorkflowTemplate(_common.FlyteIdlEntity):
     def from_flyte_idl(cls, pb2_object):
         """
         :param flyteidl.core.workflow_pb2.WorkflowTemplate pb2_object:
+
         :rtype: WorkflowTemplate
         """
         return cls(
@@ -863,6 +1037,7 @@ class Alias(_common.FlyteIdlEntity):
     def var(self):
         """
         Must match one of the output variable names on a node.
+
         :rtype: Text
         """
         return self._var
@@ -871,6 +1046,7 @@ class Alias(_common.FlyteIdlEntity):
     def alias(self):
         """
         A workflow-level unique alias that downstream nodes can refer to in their input.
+
         :rtype: Text
         """
         return self._alias
@@ -885,6 +1061,7 @@ class Alias(_common.FlyteIdlEntity):
     def from_flyte_idl(cls, pb2_object):
         """
         :param flyteidl.core.workflow_pb2.Alias pb2_object:
+
         :return: Alias
         """
         return cls(pb2_object.var, pb2_object.alias)
