@@ -2,12 +2,15 @@ from __future__ import annotations
 
 import datetime
 import typing
+from datetime import timezone as _timezone
+from typing import Optional
 
 import flyteidl
+import flyteidl.admin.cluster_assignment_pb2 as _cluster_assignment_pb2
 import flyteidl.admin.execution_pb2 as _execution_pb2
 import flyteidl.admin.node_execution_pb2 as _node_execution_pb2
 import flyteidl.admin.task_execution_pb2 as _task_execution_pb2
-import pytz as _pytz
+from google.protobuf.wrappers_pb2 import BoolValue
 
 import flytekit
 from flytekit.models import common as _common_models
@@ -15,6 +18,7 @@ from flytekit.models import literals as _literals_models
 from flytekit.models import security
 from flytekit.models.core import execution as _core_execution
 from flytekit.models.core import identifier as _identifier
+from flytekit.models.matchable_resource import ExecutionClusterLabel
 from flytekit.models.node_execution import DynamicWorkflowNodeMetadata
 
 
@@ -47,10 +51,10 @@ class ExecutionMetadata(_common_models.FlyteIdlEntity):
         mode: int,
         principal: str,
         nesting: int,
-        scheduled_at: typing.Optional[datetime.datetime] = None,
-        parent_node_execution: typing.Optional[_identifier.NodeExecutionIdentifier] = None,
-        reference_execution: typing.Optional[_identifier.WorkflowExecutionIdentifier] = None,
-        system_metadata: typing.Optional[SystemMetadata] = None,
+        scheduled_at: Optional[datetime.datetime] = None,
+        parent_node_execution: Optional[_identifier.NodeExecutionIdentifier] = None,
+        reference_execution: Optional[_identifier.WorkflowExecutionIdentifier] = None,
+        system_metadata: Optional[SystemMetadata] = None,
     ):
         """
         :param mode: An enum value from ExecutionMetadata.ExecutionMode which specifies how the job started.
@@ -173,9 +177,14 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         annotations=None,
         auth_role=None,
         raw_output_data_config=None,
-        max_parallelism=None,
-        security_context: typing.Optional[security.SecurityContext] = None,
-        overwrite_cache: bool = None,
+        max_parallelism: Optional[int] = None,
+        security_context: Optional[security.SecurityContext] = None,
+        overwrite_cache: Optional[bool] = None,
+        interruptible: Optional[bool] = None,
+        envs: Optional[_common_models.Envs] = None,
+        tags: Optional[typing.List[str]] = None,
+        cluster_assignment: Optional[ClusterAssignment] = None,
+        execution_cluster_label: Optional[ExecutionClusterLabel] = None,
     ):
         """
         :param flytekit.models.core.identifier.Identifier launch_plan: Launch plan unique identifier to execute
@@ -189,7 +198,12 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         :param max_parallelism int: Controls the maximum number of tasknodes that can be run in parallel for the entire
             workflow. This is useful to achieve fairness. Note: MapTasks are regarded as one unit, and
             parallelism/concurrency of MapTasks is independent from this.
-
+        :param security_context: Optional security context to use for this execution.
+        :param overwrite_cache: Optional flag to overwrite the cache for this execution.
+        :param interruptible: Optional flag to override the default interruptible flag of the executed entity.
+        :param envs: flytekit.models.common.Envs environment variables to set for this execution.
+        :param tags: Optional list of tags to apply to the execution.
+        :param execution_cluster_label: Optional execution cluster label to use for this execution.
         """
         self._launch_plan = launch_plan
         self._metadata = metadata
@@ -201,7 +215,12 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         self._raw_output_data_config = raw_output_data_config
         self._max_parallelism = max_parallelism
         self._security_context = security_context
-        self.overwrite_cache = overwrite_cache
+        self._overwrite_cache = overwrite_cache
+        self._interruptible = interruptible
+        self._envs = envs
+        self._tags = tags
+        self._cluster_assignment = cluster_assignment
+        self._execution_cluster_label = execution_cluster_label
 
     @property
     def launch_plan(self):
@@ -268,6 +287,30 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
     def security_context(self) -> typing.Optional[security.SecurityContext]:
         return self._security_context
 
+    @property
+    def overwrite_cache(self) -> Optional[bool]:
+        return self._overwrite_cache
+
+    @property
+    def interruptible(self) -> Optional[bool]:
+        return self._interruptible
+
+    @property
+    def envs(self) -> Optional[_common_models.Envs]:
+        return self._envs
+
+    @property
+    def tags(self) -> Optional[typing.List[str]]:
+        return self._tags
+
+    @property
+    def cluster_assignment(self) -> Optional[ClusterAssignment]:
+        return self._cluster_assignment
+
+    @property
+    def execution_cluster_label(self) -> Optional[ExecutionClusterLabel]:
+        return self._execution_cluster_label
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.execution_pb2.ExecutionSpec
@@ -276,7 +319,7 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
             launch_plan=self.launch_plan.to_flyte_idl(),
             metadata=self.metadata.to_flyte_idl(),
             notifications=self.notifications.to_flyte_idl() if self.notifications else None,
-            disable_all=self.disable_all,
+            disable_all=self.disable_all,  # type: ignore
             labels=self.labels.to_flyte_idl(),
             annotations=self.annotations.to_flyte_idl(),
             auth_role=self._auth_role.to_flyte_idl() if self.auth_role else None,
@@ -286,6 +329,13 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
             max_parallelism=self.max_parallelism,
             security_context=self.security_context.to_flyte_idl() if self.security_context else None,
             overwrite_cache=self.overwrite_cache,
+            interruptible=BoolValue(value=self.interruptible) if self.interruptible is not None else None,
+            envs=self.envs.to_flyte_idl() if self.envs else None,
+            tags=self.tags,
+            cluster_assignment=self._cluster_assignment.to_flyte_idl() if self._cluster_assignment else None,
+            execution_cluster_label=self._execution_cluster_label.to_flyte_idl()
+            if self._execution_cluster_label
+            else None,
         )
 
     @classmethod
@@ -310,9 +360,49 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
             if p.security_context
             else None,
             overwrite_cache=p.overwrite_cache,
+            interruptible=p.interruptible.value if p.HasField("interruptible") else None,
+            envs=_common_models.Envs.from_flyte_idl(p.envs) if p.HasField("envs") else None,
+            tags=p.tags,
+            cluster_assignment=ClusterAssignment.from_flyte_idl(p.cluster_assignment)
+            if p.HasField("cluster_assignment")
+            else None,
+            execution_cluster_label=ExecutionClusterLabel.from_flyte_idl(p.execution_cluster_label)
+            if p.HasField("execution_cluster_label")
+            else None,
         )
 
 
+class ClusterAssignment(_common_models.FlyteIdlEntity):
+    def __init__(self, cluster_pool=None):
+        """
+        :param Text cluster_pool:
+        """
+        self._cluster_pool = cluster_pool
+
+    @property
+    def cluster_pool(self):
+        """
+        :rtype: Text
+        """
+        return self._cluster_pool
+
+    def to_flyte_idl(self):
+        """
+        :rtype: flyteidl.admin._cluster_assignment_pb2.ClusterAssignment
+        """
+        return _cluster_assignment_pb2.ClusterAssignment(
+            cluster_pool_name=self._cluster_pool,
+        )
+
+    @classmethod
+    def from_flyte_idl(cls, p):
+        """
+        :param flyteidl.admin._cluster_assignment_pb2.ClusterAssignment p:
+        :rtype: flyteidl.admin.ClusterAssignment
+        """
+        return cls(cluster_pool=p.cluster_pool_name)
+
+
 class LiteralMapBlob(_common_models.FlyteIdlEntity):
     def __init__(self, values=None, uri=None):
         """
@@ -510,12 +600,12 @@ class ExecutionClosure(_common_models.FlyteIdlEntity):
             outputs=self.outputs.to_flyte_idl() if self.outputs is not None else None,
             abort_metadata=self.abort_metadata.to_flyte_idl() if self.abort_metadata is not None else None,
         )
-        obj.started_at.FromDatetime(self.started_at.astimezone(_pytz.UTC).replace(tzinfo=None))
+        obj.started_at.FromDatetime(self.started_at.astimezone(_timezone.utc).replace(tzinfo=None))
         obj.duration.FromTimedelta(self.duration)
         if self.created_at:
-            obj.created_at.FromDatetime(self.created_at.astimezone(_pytz.UTC).replace(tzinfo=None))
+            obj.created_at.FromDatetime(self.created_at.astimezone(_timezone.utc).replace(tzinfo=None))
         if self.updated_at:
-            obj.updated_at.FromDatetime(self.updated_at.astimezone(_pytz.UTC).replace(tzinfo=None))
+            obj.updated_at.FromDatetime(self.updated_at.astimezone(_timezone.utc).replace(tzinfo=None))
         return obj
 
     @classmethod
@@ -537,13 +627,13 @@ class ExecutionClosure(_common_models.FlyteIdlEntity):
             error=error,
             outputs=outputs,
             phase=pb2_object.phase,
-            started_at=pb2_object.started_at.ToDatetime().replace(tzinfo=_pytz.UTC),
+            started_at=pb2_object.started_at.ToDatetime().replace(tzinfo=_timezone.utc),
             duration=pb2_object.duration.ToTimedelta(),
             abort_metadata=abort_metadata,
-            created_at=pb2_object.created_at.ToDatetime().replace(tzinfo=_pytz.UTC)
+            created_at=pb2_object.created_at.ToDatetime().replace(tzinfo=_timezone.utc)
             if pb2_object.HasField("created_at")
             else None,
-            updated_at=pb2_object.updated_at.ToDatetime().replace(tzinfo=_pytz.UTC)
+            updated_at=pb2_object.updated_at.ToDatetime().replace(tzinfo=_timezone.utc)
             if pb2_object.HasField("updated_at")
             else None,
         )
