@@ -1,22 +1,28 @@
+import typing
+
 from flyteidl.admin import launch_plan_pb2 as _launch_plan
+from google.protobuf.any_pb2 import Any
 
 from flytekit.models import common as _common
 from flytekit.models import interface as _interface
 from flytekit.models import literals as _literals
 from flytekit.models import schedule as _schedule
+from flytekit.models import security
 from flytekit.models.core import identifier as _identifier
 
 
 class LaunchPlanMetadata(_common.FlyteIdlEntity):
-    def __init__(self, schedule, notifications):
+    def __init__(self, schedule, notifications, launch_conditions=None):
         """
 
         :param flytekit.models.schedule.Schedule schedule: Schedule to execute the Launch Plan
         :param list[flytekit.models.common.Notification] notifications: List of notifications based on
             execution status transitions
+        :param launch_conditions: Additional metadata for launching
         """
         self._schedule = schedule
         self._notifications = notifications
+        self._launch_conditions = launch_conditions
 
     @property
     def schedule(self):
@@ -34,14 +40,24 @@ class LaunchPlanMetadata(_common.FlyteIdlEntity):
         """
         return self._notifications
 
+    @property
+    def launch_conditions(self):
+        return self._launch_conditions
+
     def to_flyte_idl(self):
         """
         List of notifications based on Execution status transitions
         :rtype: flyteidl.admin.launch_plan_pb2.LaunchPlanMetadata
         """
+        if self.launch_conditions:
+            a = Any()
+            a.Pack(self.launch_conditions)
+        else:
+            a = None
         return _launch_plan.LaunchPlanMetadata(
             schedule=self.schedule.to_flyte_idl() if self.schedule is not None else None,
             notifications=[n.to_flyte_idl() for n in self.notifications],
+            launch_conditions=a,
         )
 
     @classmethod
@@ -55,6 +71,7 @@ class LaunchPlanMetadata(_common.FlyteIdlEntity):
             if pb2_object.HasField("schedule")
             else None,
             notifications=[_common.Notification.from_flyte_idl(n) for n in pb2_object.notifications],
+            launch_conditions=pb2_object.launch_conditions if pb2_object.HasField("launch_conditions") else None,
         )
 
 
@@ -114,11 +131,13 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
         entity_metadata,
         default_inputs,
         fixed_inputs,
-        labels,
-        annotations,
-        auth_role,
-        raw_output_data_config,
-        max_parallelism=None,
+        labels: _common.Labels,
+        annotations: _common.Annotations,
+        auth_role: _common.AuthRole,
+        raw_output_data_config: _common.RawOutputDataConfig,
+        max_parallelism: typing.Optional[int] = None,
+        security_context: typing.Optional[security.SecurityContext] = None,
+        overwrite_cache: typing.Optional[bool] = None,
     ):
         """
         The spec for a Launch Plan.
@@ -127,16 +146,18 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
         :param LaunchPlanMetadata entity_metadata: Metadata
         :param flytekit.models.interface.ParameterMap default_inputs: Input values to be passed for the execution
         :param flytekit.models.literals.LiteralMap fixed_inputs: Fixed, non-overridable inputs for the Launch Plan
-        :param flyteidl.admin.common_pb2.Labels:
+        :param flytekit.models.common.Labels:
             Any custom kubernetes labels to apply to workflows executed by this launch plan.
-        :param flyteidl.admin.common_pb2.Annotations annotations:
+        :param flytekit.models.common.Annotations annotations:
             Any custom kubernetes annotations to apply to workflows executed by this launch plan.
-        :param flytekit.models.common.Auth auth_role: The auth method with which to execute the workflow.
+        :param flytekit.models.common.AuthRole auth_role: The auth method with which to execute the workflow.
         :param flytekit.models.common.RawOutputDataConfig raw_output_data_config: Value for where to store offloaded
             data like Blobs and Schemas.
         :param max_parallelism int: Controls the maximum number of tasknodes that can be run in parallel for the entire
             workflow. This is useful to achieve fairness. Note: MapTasks are regarded as one unit, and
             parallelism/concurrency of MapTasks is independent from this.
+        :param security_context: This can be used to add security information to a LaunchPlan, which will be used by
+                                 every execution
         """
         self._workflow_id = workflow_id
         self._entity_metadata = entity_metadata
@@ -147,6 +168,8 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
         self._auth_role = auth_role
         self._raw_output_data_config = raw_output_data_config
         self._max_parallelism = max_parallelism
+        self._security_context = security_context
+        self._overwrite_cache = overwrite_cache
 
     @property
     def workflow_id(self):
@@ -180,7 +203,7 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
         return self._fixed_inputs
 
     @property
-    def labels(self):
+    def labels(self) -> _common.Labels:
         """
         The labels to execute the workflow with
         :rtype: flytekit.models.common.Labels
@@ -188,7 +211,7 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
         return self._labels
 
     @property
-    def annotations(self):
+    def annotations(self) -> _common.Annotations:
         """
         The annotations to execute the workflow with
         :rtype: flytekit.models.common.Annotations
@@ -199,7 +222,7 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
     def auth_role(self):
         """
         The authorization method with which to execute the workflow.
-        :rtype: flytekit.models.common.Auth
+        :rtype: flytekit.models.common.AuthRole
         """
         return self._auth_role
 
@@ -212,9 +235,17 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
         return self._raw_output_data_config
 
     @property
-    def max_parallelism(self) -> int:
+    def max_parallelism(self) -> typing.Optional[int]:
         return self._max_parallelism
 
+    @property
+    def security_context(self) -> typing.Optional[security.SecurityContext]:
+        return self._security_context
+
+    @property
+    def overwrite_cache(self) -> typing.Optional[bool]:
+        return self._overwrite_cache
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.launch_plan_pb2.LaunchPlanSpec
@@ -226,9 +257,11 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
             fixed_inputs=self.fixed_inputs.to_flyte_idl(),
             labels=self.labels.to_flyte_idl(),
             annotations=self.annotations.to_flyte_idl(),
-            auth_role=self.auth_role.to_flyte_idl(),
+            auth_role=self.auth_role.to_flyte_idl() if self.auth_role else None,
             raw_output_data_config=self.raw_output_data_config.to_flyte_idl(),
             max_parallelism=self.max_parallelism,
+            security_context=self.security_context.to_flyte_idl() if self.security_context else None,
+            overwrite_cache=self.overwrite_cache if self.overwrite_cache else None,
         )
 
     @classmethod
@@ -258,6 +291,10 @@ class LaunchPlanSpec(_common.FlyteIdlEntity):
             auth_role=auth_role,
             raw_output_data_config=_common.RawOutputDataConfig.from_flyte_idl(pb2.raw_output_data_config),
             max_parallelism=pb2.max_parallelism,
+            security_context=security.SecurityContext.from_flyte_idl(pb2.security_context)
+            if pb2.security_context
+            else None,
+            overwrite_cache=pb2.overwrite_cache if pb2.overwrite_cache else None,
         )
 
 
@@ -336,15 +373,24 @@ class LaunchPlanClosure(_common.FlyteIdlEntity):
 
 
 class LaunchPlan(_common.FlyteIdlEntity):
-    def __init__(self, id, spec, closure):
+    # TODO shall we add auto_activate here? this is client side only and breaks the contract
+    def __init__(self, id, spec, closure, auto_activate=False):
         """
         :param flytekit.models.core.identifier.Identifier id:
         :param LaunchPlanSpec spec:
         :param LaunchPlanClosure closure:
+        :param bool auto_activate: Whether to automatically activate the launch plan, this is a client side only
+            parameter and does not affect the actual launch plan. Also cannot be used from flytectl, only available
+            from the python SDK or pyflyte register.
         """
         self._id = id
         self._spec = spec
         self._closure = closure
+        self._auto_activate = auto_activate
+
+    @property
+    def should_auto_activate(self) -> bool:
+        return self._auto_activate
 
     @property
     def id(self):
