@@ -1,21 +1,22 @@
 from datetime import datetime as _datetime
-from typing import Optional
+from datetime import timedelta as _timedelta
+from datetime import timezone as _timezone
+from typing import Dict, Optional
 
-import pytz as _pytz
 from flyteidl.core import literals_pb2 as _literals_pb2
 from google.protobuf.struct_pb2 import Struct
 
 from flytekit.exceptions import user as _user_exceptions
 from flytekit.models import common as _common
 from flytekit.models.core import types as _core_types
+from flytekit.models.types import Error, LiteralType, StructuredDatasetType
 from flytekit.models.types import LiteralType as _LiteralType
 from flytekit.models.types import OutputReference as _OutputReference
 from flytekit.models.types import SchemaType as _SchemaType
-from flytekit.models.types import StructuredDatasetType
 
 
 class RetryStrategy(_common.FlyteIdlEntity):
-    def __init__(self, retries):
+    def __init__(self, retries: int):
         """
         :param int retries: Number of retries to attempt on recoverable failures.  If retries is 0, then
             only one attempt will be made.
@@ -48,12 +49,12 @@ class RetryStrategy(_common.FlyteIdlEntity):
 class Primitive(_common.FlyteIdlEntity):
     def __init__(
         self,
-        integer=None,
-        float_value=None,
-        string_value=None,
-        boolean=None,
-        datetime=None,
-        duration=None,
+        integer: Optional[int] = None,
+        float_value: Optional[float] = None,
+        string_value: Optional[str] = None,
+        boolean: Optional[bool] = None,
+        datetime: Optional[_datetime] = None,
+        duration: Optional[_timedelta] = None,
     ):
         """
         This object proxies the primitives supported by the Flyte IDL system.  Only one value can be set.
@@ -77,44 +78,44 @@ class Primitive(_common.FlyteIdlEntity):
         self._duration = duration
 
     @property
-    def integer(self):
+    def integer(self) -> Optional[int]:
         """
         :rtype: int
         """
         return self._integer
 
     @property
-    def float_value(self):
+    def float_value(self) -> Optional[float]:
         """
         :rtype: float
         """
         return self._float_value
 
     @property
-    def string_value(self):
+    def string_value(self) -> Optional[str]:
         """
         :rtype: Text
         """
         return self._string_value
 
     @property
-    def boolean(self):
+    def boolean(self) -> Optional[bool]:
         """
         :rtype: bool
         """
         return self._boolean
 
     @property
-    def datetime(self):
+    def datetime(self) -> Optional[_datetime]:
         """
         :rtype: datetime.datetime
         """
         if self._datetime is None or self._datetime.tzinfo is not None:
             return self._datetime
-        return self._datetime.replace(tzinfo=_pytz.UTC)
+        return self._datetime.replace(tzinfo=_timezone.utc)
 
     @property
-    def duration(self):
+    def duration(self) -> Optional[_timedelta]:
         """
         :rtype: datetime.timedelta
         """
@@ -149,7 +150,7 @@ class Primitive(_common.FlyteIdlEntity):
         )
         if self.datetime is not None:
             # Convert to UTC and remove timezone so protobuf behaves.
-            primitive.datetime.FromDatetime(self.datetime.astimezone(_pytz.UTC).replace(tzinfo=None))
+            primitive.datetime.FromDatetime(self.datetime.astimezone(_timezone.utc).replace(tzinfo=None))
         if self.duration is not None:
             primitive.duration.FromTimedelta(self.duration)
         return primitive
@@ -165,7 +166,7 @@ class Primitive(_common.FlyteIdlEntity):
             float_value=proto.float_value if proto.HasField("float_value") else None,
             string_value=proto.string_value if proto.HasField("string_value") else None,
             boolean=proto.boolean if proto.HasField("boolean") else None,
-            datetime=proto.datetime.ToDatetime().replace(tzinfo=_pytz.UTC) if proto.HasField("datetime") else None,
+            datetime=proto.datetime.ToDatetime().replace(tzinfo=_timezone.utc) if proto.HasField("datetime") else None,
             duration=proto.duration.ToTimedelta() if proto.HasField("duration") else None,
         )
 
@@ -245,7 +246,7 @@ class Blob(_common.FlyteIdlEntity):
     def __init__(self, metadata, uri):
         """
         This literal model is used to represent binary data offloaded to some storage location which is
-        identifiable with a unique string. See :py:class:`flytekit.FlyteFile` as an example.
+        identifiable with a unique string. See {{< py_class_ref flytekit.FlyteFile >}} as an example.
 
         :param BlobMetadata metadata:
         :param Text uri: The location of this blob
@@ -628,7 +629,7 @@ class StructuredDataset(_common.FlyteIdlEntity):
         return self._uri
 
     @property
-    def metadata(self) -> StructuredDatasetMetadata:
+    def metadata(self) -> Optional[StructuredDatasetMetadata]:
         return self._metadata
 
     def to_flyte_idl(self) -> _literals_pb2.StructuredDataset:
@@ -703,15 +704,15 @@ class LiteralMap(_common.FlyteIdlEntity):
 class Scalar(_common.FlyteIdlEntity):
     def __init__(
         self,
-        primitive: Primitive = None,
-        blob: Blob = None,
-        binary: Binary = None,
-        schema: Schema = None,
-        union: Union = None,
-        none_type: Void = None,
-        error=None,
-        generic: Struct = None,
-        structured_dataset: StructuredDataset = None,
+        primitive: Optional[Primitive] = None,
+        blob: Optional[Blob] = None,
+        binary: Optional[Binary] = None,
+        schema: Optional[Schema] = None,
+        union: Optional[Union] = None,
+        none_type: Optional[Void] = None,
+        error: Optional[Error] = None,
+        generic: Optional[Struct] = None,
+        structured_dataset: Optional[StructuredDataset] = None,
     ):
         """
         Scalar wrapper around Flyte types.  Only one can be specified.
@@ -721,7 +722,7 @@ class Scalar(_common.FlyteIdlEntity):
         :param Binary binary:
         :param Schema schema:
         :param Void none_type:
-        :param error:
+        :param Error error:
         :param google.protobuf.struct_pb2.Struct generic:
         :param StructuredDataset structured_dataset:
         """
@@ -781,7 +782,7 @@ class Scalar(_common.FlyteIdlEntity):
     @property
     def error(self):
         """
-        :rtype: TODO
+        :rtype: Error
         """
         return self._error
 
@@ -825,7 +826,7 @@ class Scalar(_common.FlyteIdlEntity):
             schema=self.schema.to_flyte_idl() if self.schema is not None else None,
             union=self.union.to_flyte_idl() if self.union is not None else None,
             none_type=self.none_type.to_flyte_idl() if self.none_type is not None else None,
-            error=self.error if self.error is not None else None,
+            error=self.error.to_flyte_idl() if self.error is not None else None,
             generic=self.generic,
             structured_dataset=self.structured_dataset.to_flyte_idl() if self.structured_dataset is not None else None,
         )
@@ -852,9 +853,61 @@ class Scalar(_common.FlyteIdlEntity):
         )
 
 
+class LiteralOffloadedMetadata(_common.FlyteIdlEntity):
+    def __init__(
+        self,
+        uri: Optional[str] = None,
+        size_bytes: Optional[int] = None,
+        inferred_type: Optional[LiteralType] = None,
+    ):
+        """
+        :param Text uri: URI of the offloaded literal
+        :param int size_bytes: Size in bytes of the offloaded literal proto
+        :param LiteralType inferred_type: Inferred type of the offloaded literal
+        """
+        self._uri = uri
+        self._size_bytes = size_bytes
+        self._inferred_type = inferred_type
+
+    @property
+    def uri(self):
+        return self._uri
+
+    @property
+    def size_bytes(self):
+        return self._size_bytes
+
+    @property
+    def inferred_type(self):
+        return self._inferred_type
+
+    def to_flyte_idl(self):
+        return _literals_pb2.LiteralOffloadedMetadata(
+            uri=self.uri,
+            size_bytes=self.size_bytes,
+            inferred_type=self.inferred_type.to_flyte_idl() if self.inferred_type else None,
+        )
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object):
+        return cls(
+            uri=pb2_object.uri,
+            size_bytes=pb2_object.size_bytes,
+            inferred_type=_LiteralType.from_flyte_idl(pb2_object.inferred_type)
+            if pb2_object.HasField("inferred_type")
+            else None,
+        )
+
+
 class Literal(_common.FlyteIdlEntity):
     def __init__(
-        self, scalar: Scalar = None, collection: LiteralCollection = None, map: LiteralMap = None, hash: str = None
+        self,
+        scalar: Optional[Scalar] = None,
+        collection: Optional[LiteralCollection] = None,
+        map: Optional[LiteralMap] = None,
+        hash: Optional[str] = None,
+        metadata: Optional[Dict[str, str]] = None,
+        offloaded_metadata: Optional[LiteralOffloadedMetadata] = None,
     ):
         """
         This IDL message represents a literal value in the Flyte ecosystem.
@@ -867,6 +920,8 @@ class Literal(_common.FlyteIdlEntity):
         self._collection = collection
         self._map = map
         self._hash = hash
+        self._metadata = metadata
+        self._offloaded_metadata = offloaded_metadata
 
     @property
     def scalar(self):
@@ -912,6 +967,28 @@ class Literal(_common.FlyteIdlEntity):
     def hash(self, value):
         self._hash = value
 
+    @property
+    def metadata(self) -> Optional[Dict[str, str]]:
+        """
+        This value holds metadata about the literal.
+        """
+        return self._metadata
+
+    @property
+    def offloaded_metadata(self) -> Optional[LiteralOffloadedMetadata]:
+        """
+        This value holds metadata about the offloaded literal.
+        """
+        # The following check might seem non-sensical, since `_offloaded_metadata` is set in the constructor.
+        # This is here to support backwards compatibility caused by the local cache implementation. Let me explain.
+        # The local cache pickles values and unpickles them. When unpickling, the constructor is not called, so there
+        # are cases where the `_offloaded_metadata` is not set (for example if you cache a value using flytekit<=1.13.6
+        # and you load that value later using flytekit>1.13.6).
+        # In other words, this is a workaround to support backwards compatibility with the local cache.
+        if hasattr(self, "_offloaded_metadata"):
+            return self._offloaded_metadata
+        return None
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.literals_pb2.Literal
@@ -921,10 +998,12 @@ class Literal(_common.FlyteIdlEntity):
             collection=self.collection.to_flyte_idl() if self.collection is not None else None,
             map=self.map.to_flyte_idl() if self.map is not None else None,
             hash=self.hash,
+            metadata=self.metadata,
+            offloaded_metadata=self.offloaded_metadata.to_flyte_idl() if self.offloaded_metadata is not None else None,
         )
 
     @classmethod
-    def from_flyte_idl(cls, pb2_object):
+    def from_flyte_idl(cls, pb2_object: _literals_pb2.Literal):
         """
         :param flyteidl.core.literals_pb2.Literal pb2_object:
         :rtype: Literal
@@ -938,4 +1017,15 @@ class Literal(_common.FlyteIdlEntity):
             collection=collection,
             map=LiteralMap.from_flyte_idl(pb2_object.map) if pb2_object.HasField("map") else None,
             hash=pb2_object.hash if pb2_object.hash else None,
+            metadata={k: v for k, v in pb2_object.metadata.items()} if pb2_object.metadata else None,
+            offloaded_metadata=LiteralOffloadedMetadata.from_flyte_idl(pb2_object.offloaded_metadata)
+            if pb2_object.HasField("offloaded_metadata")
+            else None,
         )
+
+    def set_metadata(self, metadata: Dict[str, str]):
+        """
+        Note: This is a mutation on the literal
+        :param Dict[str, str] metadata: Metadata to be added
+        """
+        self._metadata = metadata
