@@ -1,12 +1,14 @@
 import json as _json
 import typing
 
+from flyteidl.admin import agent_pb2 as _admin_agent
 from flyteidl.admin import task_pb2 as _admin_task
 from flyteidl.core import compiler_pb2 as _compiler
 from flyteidl.core import literals_pb2 as _literals_pb2
 from flyteidl.core import tasks_pb2 as _core_task
 from google.protobuf import json_format as _json_format
 from google.protobuf import struct_pb2 as _struct
+from google.protobuf.wrappers_pb2 import BoolValue
 
 from flytekit.models import common as _common
 from flytekit.models import interface as _interface
@@ -15,6 +17,9 @@ from flytekit.models import security as _sec
 from flytekit.models.core import identifier as _identifier
 from flytekit.models.documentation import Documentation
 
+if typing.TYPE_CHECKING:
+    from flytekit import PodTemplate
+
 
 class Resources(_common.FlyteIdlEntity):
     class ResourceName(object):
@@ -22,7 +27,6 @@ class Resources(_common.FlyteIdlEntity):
         CPU = _core_task.Resources.CPU
         GPU = _core_task.Resources.GPU
         MEMORY = _core_task.Resources.MEMORY
-        STORAGE = _core_task.Resources.STORAGE
         EPHEMERAL_STORAGE = _core_task.Resources.EPHEMERAL_STORAGE
 
     class ResourceEntry(_common.FlyteIdlEntity):
@@ -178,6 +182,10 @@ class TaskMetadata(_common.FlyteIdlEntity):
         deprecated_error_message,
         cache_serializable,
         pod_template_name,
+        cache_ignore_input_vars,
+        is_eager: bool = False,
+        generates_deck: bool = False,
+        k8s_object_metadata: typing.Optional["K8sObjectMetadata"] = None,
     ):
         """
         Information needed at runtime to determine behavior such as whether or not outputs are discoverable, timeouts,
@@ -197,7 +205,11 @@ class TaskMetadata(_common.FlyteIdlEntity):
             receive deprecation warnings.
         :param bool cache_serializable: Whether or not caching operations are executed in serial. This means only a
             single instance over identical inputs is executed, other concurrent executions wait for the cached results.
+        :param bool generates_deck: Whether the task will generate a Deck URI.
         :param pod_template_name: The name of the existing PodTemplate resource which will be used in this task.
+        :param cache_ignore_input_vars: Input variables that should not be included when calculating hash for cache.
+        :param is_eager:
+        :param metadata: Kubernetes metadata for the task.
         """
         self._discoverable = discoverable
         self._runtime = runtime
@@ -208,6 +220,14 @@ class TaskMetadata(_common.FlyteIdlEntity):
         self._deprecated_error_message = deprecated_error_message
         self._cache_serializable = cache_serializable
         self._pod_template_name = pod_template_name
+        self._cache_ignore_input_vars = cache_ignore_input_vars
+        self._is_eager = is_eager
+        self._generates_deck = generates_deck
+        self._k8s_object_metadata = k8s_object_metadata
+
+    @property
+    def is_eager(self):
+        return self._is_eager
 
     @property
     def discoverable(self):
@@ -285,6 +305,30 @@ class TaskMetadata(_common.FlyteIdlEntity):
         """
         return self._pod_template_name
 
+    @property
+    def generates_deck(self) -> bool:
+        """
+        Whether the task will generate a Deck.
+        :rtype: bool
+        """
+        return self._generates_deck
+
+    @property
+    def cache_ignore_input_vars(self):
+        """
+        Input variables that should not be included when calculating hash for cache.
+        :rtype: tuple[Text]
+        """
+        return self._cache_ignore_input_vars
+
+    @property
+    def k8s_object_metadata(self) -> typing.Optional["K8sObjectMetadata"]:
+        """
+        Kubernetes metadata for the task.
+        :rtype: K8sObjectMetadata
+        """
+        return self._k8s_object_metadata
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.task_pb2.TaskMetadata
@@ -298,13 +342,17 @@ class TaskMetadata(_common.FlyteIdlEntity):
             deprecated_error_message=self.deprecated_error_message,
             cache_serializable=self.cache_serializable,
             pod_template_name=self.pod_template_name,
+            cache_ignore_input_vars=self.cache_ignore_input_vars,
+            is_eager=self.is_eager,
+            generates_deck=BoolValue(value=self.generates_deck),
+            metadata=self.k8s_object_metadata.to_flyte_idl() if self.k8s_object_metadata else None,
         )
         if self.timeout:
             tm.timeout.FromTimedelta(self.timeout)
         return tm
 
     @classmethod
-    def from_flyte_idl(cls, pb2_object):
+    def from_flyte_idl(cls, pb2_object: _core_task.TaskMetadata):
         """
         :param flyteidl.core.task_pb2.TaskMetadata pb2_object:
         :rtype: TaskMetadata
@@ -319,6 +367,12 @@ class TaskMetadata(_common.FlyteIdlEntity):
             deprecated_error_message=pb2_object.deprecated_error_message,
             cache_serializable=pb2_object.cache_serializable,
             pod_template_name=pb2_object.pod_template_name,
+            cache_ignore_input_vars=pb2_object.cache_ignore_input_vars,
+            is_eager=pb2_object.is_eager,
+            generates_deck=pb2_object.generates_deck.value if pb2_object.HasField("generates_deck") else False,
+            k8s_object_metadata=K8sObjectMetadata.from_flyte_idl(pb2_object.metadata)
+            if pb2_object.HasField("metadata")
+            else None,
         )
 
 
@@ -336,6 +390,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
         config=None,
         k8s_pod=None,
         sql=None,
+        extended_resources=None,
     ):
         """
         A task template represents the full set of information necessary to perform a unit of work in the Flyte system.
@@ -359,6 +414,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
             in tandem with the custom.
         :param K8sPod k8s_pod: Alternative to the container used to execute this task.
         :param Sql sql: This is used to execute query in FlytePropeller instead of running container or k8s_pod.
+        :param flyteidl.core.tasks_pb2.ExtendedResources extended_resources: The extended resources to allocate to the task.
         """
         if (
             (container is not None and k8s_pod is not None)
@@ -377,6 +433,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
         self._security_context = security_context
         self._k8s_pod = k8s_pod
         self._sql = sql
+        self._extended_resources = extended_resources
 
     @property
     def id(self):
@@ -451,6 +508,14 @@ class TaskTemplate(_common.FlyteIdlEntity):
     def sql(self):
         return self._sql
 
+    @property
+    def extended_resources(self):
+        """
+        If not None, the extended resources to allocate to the task.
+        :rtype: flyteidl.core.tasks_pb2.ExtendedResources
+        """
+        return self._extended_resources
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.tasks_pb2.TaskTemplate
@@ -464,6 +529,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
             container=self.container.to_flyte_idl() if self.container else None,
             task_type_version=self.task_type_version,
             security_context=self.security_context.to_flyte_idl() if self.security_context else None,
+            extended_resources=self.extended_resources,
             config={k: v for k, v in self.config.items()} if self.config is not None else None,
             k8s_pod=self.k8s_pod.to_flyte_idl() if self.k8s_pod else None,
             sql=self.sql.to_flyte_idl() if self.sql else None,
@@ -487,12 +553,110 @@ class TaskTemplate(_common.FlyteIdlEntity):
             security_context=_sec.SecurityContext.from_flyte_idl(pb2_object.security_context)
             if pb2_object.security_context and pb2_object.security_context.ByteSize() > 0
             else None,
+            extended_resources=pb2_object.extended_resources if pb2_object.HasField("extended_resources") else None,
             config={k: v for k, v in pb2_object.config.items()} if pb2_object.config is not None else None,
             k8s_pod=K8sPod.from_flyte_idl(pb2_object.k8s_pod) if pb2_object.HasField("k8s_pod") else None,
             sql=Sql.from_flyte_idl(pb2_object.sql) if pb2_object.HasField("sql") else None,
         )
 
 
+class TaskExecutionMetadata(_common.FlyteIdlEntity):
+    def __init__(
+        self,
+        task_execution_id,
+        namespace,
+        labels,
+        annotations,
+        k8s_service_account,
+        environment_variables,
+        identity,
+    ):
+        """
+        Runtime task execution metadata.
+
+        :param flytekit.models.core.identifier.TaskExecutionIdentifier task_execution_id: This is generated by the system and uniquely identifies
+            this execution of the task.
+        :param Text namespace: This is the namespace the task is executing in.
+        :param dict[str, str] labels: Labels to use for the execution of this task.
+        :param dict[str, str] annotations: Annotations to use for the execution of this task.
+        :param Text k8s_service_account: Service account to use for execution of this task.
+        :param dict[str, str] environment_variables: Environment variables for this task.
+        :param flytekit.models.security.Identity identity: Identity of user executing this task
+        """
+        self._task_execution_id = task_execution_id
+        self._namespace = namespace
+        self._labels = labels
+        self._annotations = annotations
+        self._k8s_service_account = k8s_service_account
+        self._environment_variables = environment_variables
+        self._identity = identity
+
+    @property
+    def task_execution_id(self):
+        return self._task_execution_id
+
+    @property
+    def namespace(self):
+        return self._namespace
+
+    @property
+    def labels(self):
+        return self._labels
+
+    @property
+    def annotations(self):
+        return self._annotations
+
+    @property
+    def k8s_service_account(self):
+        return self._k8s_service_account
+
+    @property
+    def environment_variables(self):
+        return self._environment_variables
+
+    @property
+    def identity(self):
+        return self._identity
+
+    def to_flyte_idl(self):
+        """
+        :rtype: flyteidl.admin.agent_pb2.TaskExecutionMetadata
+        """
+        task_execution_metadata = _admin_agent.TaskExecutionMetadata(
+            task_execution_id=self.task_execution_id.to_flyte_idl(),
+            namespace=self.namespace,
+            labels={k: v for k, v in self.labels.items()} if self.labels is not None else None,
+            annotations={k: v for k, v in self.annotations.items()} if self.annotations is not None else None,
+            k8s_service_account=self.k8s_service_account,
+            environment_variables={k: v for k, v in self.environment_variables.items()}
+            if self.labels is not None
+            else None,
+            identity=self.identity.to_flyte_idl() if self.identity else None,
+        )
+        return task_execution_metadata
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object):
+        """
+        :param flyteidl.admin.agent_pb2.TaskExecutionMetadata pb2_object:
+        :rtype: TaskExecutionMetadata
+        """
+        return cls(
+            task_execution_id=_identifier.TaskExecutionIdentifier.from_flyte_idl(pb2_object.task_execution_id),
+            namespace=pb2_object.namespace,
+            labels={k: v for k, v in pb2_object.labels.items()} if pb2_object.labels is not None else None,
+            annotations={k: v for k, v in pb2_object.annotations.items()}
+            if pb2_object.annotations is not None
+            else None,
+            k8s_service_account=pb2_object.k8s_service_account,
+            environment_variables={k: v for k, v in pb2_object.environment_variables.items()}
+            if pb2_object.environment_variables is not None
+            else None,
+            identity=_sec.Identity.from_flyte_idl(pb2_object.identity) if pb2_object.identity else None,
+        )
+
+
 class TaskSpec(_common.FlyteIdlEntity):
     def __init__(self, template: TaskTemplate, docs: typing.Optional[Documentation] = None):
         """
@@ -537,13 +701,14 @@ class TaskSpec(_common.FlyteIdlEntity):
 
 
 class Task(_common.FlyteIdlEntity):
-    def __init__(self, id, closure):
+    def __init__(self, id, closure, short_description=None):
         """
         :param flytekit.models.core.identifier.Identifier id: The (project, domain, name) identifier for this task.
         :param TaskClosure closure: The closure for the underlying workload.
         """
         self._id = id
         self._closure = closure
+        self._short_description = short_description
 
     @property
     def id(self):
@@ -561,6 +726,14 @@ class Task(_common.FlyteIdlEntity):
         """
         return self._closure
 
+    @property
+    def short_description(self):
+        """
+        The short description of the task.
+        :rtype: str
+        """
+        return self._short_description
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.task_pb2.Task
@@ -568,6 +741,7 @@ class Task(_common.FlyteIdlEntity):
         return _admin_task.Task(
             closure=self.closure.to_flyte_idl(),
             id=self.id.to_flyte_idl(),
+            short_description=self.short_description,
         )
 
     @classmethod
@@ -579,6 +753,7 @@ class Task(_common.FlyteIdlEntity):
         return cls(
             closure=TaskClosure.from_flyte_idl(pb2_object.closure),
             id=_identifier.Identifier.from_flyte_idl(pb2_object.id),
+            short_description=pb2_object.short_description,
         )
 
 
@@ -775,7 +950,7 @@ class Container(_common.FlyteIdlEntity):
         return self._resources
 
     @property
-    def env(self):
+    def env(self) -> typing.Dict[str, str]:
         """
         A definition of key-value pairs for environment variables.  Currently, only str->str is
             supported.
@@ -825,7 +1000,7 @@ class Container(_common.FlyteIdlEntity):
         return cls(
             image=pb2_object.image,
             command=pb2_object.command,
-            args=pb2_object.args,
+            args=[arg for arg in pb2_object.args],
             resources=Resources.from_flyte_idl(pb2_object.resources),
             env={kv.key: kv.value for kv in pb2_object.env},
             config={kv.key: kv.value for kv in pb2_object.config},
@@ -836,7 +1011,11 @@ class Container(_common.FlyteIdlEntity):
 
 
 class K8sObjectMetadata(_common.FlyteIdlEntity):
-    def __init__(self, labels: typing.Dict[str, str] = None, annotations: typing.Dict[str, str] = None):
+    def __init__(
+        self,
+        labels: typing.Optional[typing.Dict[str, str]] = None,
+        annotations: typing.Optional[typing.Dict[str, str]] = None,
+    ):
         """
         This defines additional metadata for building a kubernetes pod.
         """
@@ -868,12 +1047,20 @@ class K8sObjectMetadata(_common.FlyteIdlEntity):
 
 
 class K8sPod(_common.FlyteIdlEntity):
-    def __init__(self, metadata: K8sObjectMetadata = None, pod_spec: typing.Dict[str, typing.Any] = None):
+    def __init__(
+        self,
+        metadata: K8sObjectMetadata = None,
+        pod_spec: typing.Dict[str, typing.Any] = None,
+        data_config: typing.Optional[DataLoadingConfig] = None,
+        primary_container_name: typing.Optional[str] = None,
+    ):
         """
         This defines a kubernetes pod target.  It will build the pod target during task execution
         """
         self._metadata = metadata
         self._pod_spec = pod_spec
+        self._data_config = data_config
+        self._primary_container_name = primary_container_name
 
     @property
     def metadata(self) -> K8sObjectMetadata:
@@ -883,10 +1070,20 @@ class K8sPod(_common.FlyteIdlEntity):
     def pod_spec(self) -> typing.Dict[str, typing.Any]:
         return self._pod_spec
 
+    @property
+    def data_config(self) -> typing.Optional[DataLoadingConfig]:
+        return self._data_config
+
+    @property
+    def primary_container_name(self) -> typing.Optional[str]:
+        return self._primary_container_name
+
     def to_flyte_idl(self) -> _core_task.K8sPod:
         return _core_task.K8sPod(
-            metadata=self._metadata.to_flyte_idl(),
+            metadata=self._metadata.to_flyte_idl() if self.metadata else None,
             pod_spec=_json_format.Parse(_json.dumps(self.pod_spec), _struct.Struct()) if self.pod_spec else None,
+            data_config=self.data_config.to_flyte_idl() if self.data_config else None,
+            primary_container_name=self.primary_container_name,
         )
 
     @classmethod
@@ -894,6 +1091,28 @@ class K8sPod(_common.FlyteIdlEntity):
         return cls(
             metadata=K8sObjectMetadata.from_flyte_idl(pb2_object.metadata),
             pod_spec=_json_format.MessageToDict(pb2_object.pod_spec) if pb2_object.HasField("pod_spec") else None,
+            data_config=DataLoadingConfig.from_flyte_idl(pb2_object.data_config)
+            if pb2_object.HasField("data_config")
+            else None,
+        )
+
+    def to_pod_template(self) -> "PodTemplate":
+        from flytekit import PodTemplate
+
+        return PodTemplate(
+            labels=self.metadata.labels,
+            annotations=self.metadata.annotations,
+            pod_spec=self.pod_spec,
+        )
+
+    @classmethod
+    def from_pod_template(cls, pod_template: "PodTemplate") -> "K8sPod":
+        from kubernetes.client import ApiClient
+
+        return cls(
+            metadata=K8sObjectMetadata(labels=pod_template.labels, annotations=pod_template.annotations),
+            pod_spec=ApiClient().sanitize_for_serialization(pod_template.pod_spec),
+            primary_container_name=pod_template.primary_container_name,
         )
 
 
