@@ -5,7 +5,7 @@ from datetime import datetime, timedelta
 from croniter import croniter
 
 from flytekit import LaunchPlan
-from flytekit.core.workflow import ImperativeWorkflow, WorkflowBase
+from flytekit.core.workflow import ImperativeWorkflow, WorkflowBase, WorkflowFailurePolicy
 from flytekit.remote.entities import FlyteLaunchPlan
 
 
@@ -16,6 +16,7 @@ def create_backfill_workflow(
     parallel: bool = False,
     per_node_timeout: timedelta = None,
     per_node_retries: int = 0,
+    failure_policy: typing.Optional[WorkflowFailurePolicy] = None,
 ) -> typing.Tuple[WorkflowBase, datetime, datetime]:
     """
     Generates a new imperative workflow for the launchplan that can be used to backfill the given launchplan.
@@ -23,29 +24,31 @@ def create_backfill_workflow(
 
     the Backfill plan is generated as (start_date - exclusive, end_date inclusive)
 
-    .. code-block:: python
-       :caption: Correct usage for dates example
+    > [!NOTE]
+    > Correct usage for dates example
 
-        lp = Launchplan.get_or_create(...)
-        start_date = datetime.datetime(2023, 1, 1)
-        end_date =  start_date + datetime.timedelta(days=10)
-        wf = create_backfill_workflow(start_date, end_date, for_lp=lp)
-
-
-    .. code-block:: python
-       :caption: Incorrect date example
-
-        wf = create_backfill_workflow(end_date, start_date, for_lp=lp) # end_date is before start_date
-        # OR
-        wf = create_backfill_workflow(start_date, start_date, for_lp=lp) # start and end date are same
+    ```python
+    lp = Launchplan.get_or_create(...)
+    start_date = datetime.datetime(2023, 1, 1)
+    end_date =  start_date + datetime.timedelta(days=10)
+    wf = create_backfill_workflow(start_date, end_date, for_lp=lp)
+    ```
+    > [!WARNING]
+    > Incorrect date example
 
+    ```python
+    wf = create_backfill_workflow(end_date, start_date, for_lp=lp) # end_date is before start_date
+    # OR
+    wf = create_backfill_workflow(start_date, start_date, for_lp=lp) # start and end date are same
+    ```
 
     :param start_date: datetime generate a backfill starting at this datetime (exclusive)
     :param end_date: datetime generate a backfill ending at this datetime (inclusive)
-    :param for_lp: typing.Union[LaunchPlan, FlyteLaunchPlan] the backfill is generatd for this launchplan
+    :param for_lp: typing.Union[LaunchPlan, FlyteLaunchPlan] the backfill is generated for this launchplan
     :param parallel: if the backfill should be run in parallel. False (default) will run each bacfill sequentially
     :param per_node_timeout: timedelta Timeout to use per node
     :param per_node_retries: int Retries to user per node
+    :param failure_policy: WorkflowFailurePolicy Failure policy to use for the backfill workflow
     :return: WorkflowBase, datetime datetime -> New generated workflow, datetime for first instance of backfill, datetime for last instance of backfill
     """
     if not for_lp:
@@ -66,8 +69,13 @@ def create_backfill_workflow(
     else:
         raise NotImplementedError("Currently backfilling only supports cron schedules.")
 
-    logging.info(f"Generating backfill from {start_date} -> {end_date}. Parallel?[{parallel}]")
-    wf = ImperativeWorkflow(name=f"backfill-{for_lp.name}")
+    logging.info(
+        f"Generating backfill from {start_date} -> {end_date}. "
+        f"Parallel?[{parallel}] FailurePolicy[{str(failure_policy)}]"
+    )
+    wf = ImperativeWorkflow(name=f"backfill-{for_lp.name}", failure_policy=failure_policy)
+
+    input_name = schedule.kickoff_time_input_arg
     date_iter = croniter(cron_schedule.schedule, start_time=start_date, ret_type=datetime)
     prev_node = None
     actual_start = None
@@ -79,7 +87,10 @@ def create_backfill_workflow(
         if next_start_date >= end_date:
             break
         actual_end = next_start_date
-        next_node = wf.add_launch_plan(for_lp, t=next_start_date)
+        inputs = {}
+        if input_name:
+            inputs[input_name] = next_start_date
+        next_node = wf.add_launch_plan(for_lp, **inputs)
         next_node = next_node.with_overrides(
             name=f"b-{next_start_date}", retries=per_node_retries, timeout=per_node_timeout
         )
