@@ -1,26 +1,29 @@
 from abc import ABC, abstractmethod
-from typing import Any, Dict, Optional, Tuple, Type, Union
+from typing import Any, Optional, Tuple, Union
 
 from flytekit.core.context_manager import BranchEvalMode, ExecutionState, FlyteContext
 from flytekit.core.promise import Promise, VoidPromise, create_and_link_node_from_remote, extract_obj_name
 from flytekit.exceptions import user as user_exceptions
-from flytekit.loggers import remote_logger as logger
+from flytekit.loggers import logger
+from flytekit.models.core.identifier import Identifier
 from flytekit.models.core.workflow import NodeMetadata
 
 
 class RemoteEntity(ABC):
     def __init__(self, *args, **kwargs):
-
         # In cases where we make a FlyteTask/Workflow/LaunchPlan from a locally created Python object (i.e. an @task
         # or an @workflow decorated function), we actually have the Python interface, so
-        self._python_interface: Optional[Dict[str, Type]] = None
+        self._python_interface: Optional["Interface"] = None
 
         super().__init__(*args, **kwargs)
 
     @property
     @abstractmethod
-    def name(self) -> str:
-        ...
+    def name(self) -> str: ...
+
+    @property
+    @abstractmethod
+    def id(self) -> Identifier: ...
 
     def construct_node_metadata(self) -> NodeMetadata:
         """
@@ -65,9 +68,16 @@ class RemoteEntity(ABC):
     def local_execute(self, ctx: FlyteContext, **kwargs) -> Optional[Union[Tuple[Promise], Promise, VoidPromise]]:
         return self.execute(**kwargs)
 
+    def local_execution_mode(self) -> ExecutionState.Mode:
+        return ExecutionState.Mode.LOCAL_TASK_EXECUTION
+
     def execute(self, **kwargs) -> Any:
         raise AssertionError(f"Remotely fetched entities cannot be run locally. Please mock the {self.name}.execute.")
 
     @property
-    def python_interface(self) -> Optional[Dict[str, Type]]:
+    def python_interface(self) -> Optional["Interface"]:
         return self._python_interface
+
+    @python_interface.setter
+    def python_interface(self, i: Optional["Interface"]):
+        self._python_interface = i
