@@ -1,5 +1,5 @@
 import math
-import os as _os
+import os
 import sys
 import typing
 from collections import OrderedDict
@@ -9,14 +9,14 @@ import click
 from flytekit import LaunchPlan
 from flytekit.core import context_manager as flyte_context
 from flytekit.core.base_task import PythonTask
+from flytekit.core.python_function_task import EagerAsyncPythonFunctionTask
 from flytekit.core.workflow import WorkflowBase
-from flytekit.exceptions.user import FlyteValidationException
 from flytekit.models import launch_plan as _launch_plan_models
 from flytekit.models import task as task_models
 from flytekit.models.admin import workflow as admin_workflow_models
 from flytekit.models.admin.workflow import WorkflowSpec
-from flytekit.models.core import identifier as _identifier
 from flytekit.models.task import TaskSpec
+from flytekit.remote.remote_callable import RemoteEntity
 from flytekit.tools.translator import FlyteControlPlaneEntity, Options, get_serializable
 
 
@@ -40,21 +40,7 @@ def _should_register_with_admin(entity) -> bool:
     """
     return isinstance(
         entity, (task_models.TaskSpec, _launch_plan_models.LaunchPlan, admin_workflow_models.WorkflowSpec)
-    )
-
-
-def _find_duplicate_tasks(tasks: typing.List[task_models.TaskSpec]) -> typing.Set[task_models.TaskSpec]:
-    """
-    Given a list of `TaskSpec`, this function returns a set containing the duplicated `TaskSpec` if any exists.
-    """
-    seen: typing.Set[_identifier.Identifier] = set()
-    duplicate_tasks: typing.Set[task_models.TaskSpec] = set()
-    for task in tasks:
-        if task.template.id not in seen:
-            seen.add(task.template.id)
-        else:
-            duplicate_tasks.add(task)
-    return duplicate_tasks
+    ) and not isinstance(entity, RemoteEntity)
 
 
 def get_registrable_entities(
@@ -75,21 +61,13 @@ def get_registrable_entities(
                 lp = LaunchPlan.get_default_launch_plan(ctx, entity)
                 get_serializable(new_api_serializable_entities, ctx.serialization_settings, lp, options)
 
+            if isinstance(entity, EagerAsyncPythonFunctionTask):
+                wf = entity.get_as_workflow()
+                lp = LaunchPlan.get_default_launch_plan(ctx, wf)
+                get_serializable(new_api_serializable_entities, ctx.serialization_settings, lp, options)
+
     new_api_model_values = list(new_api_serializable_entities.values())
     entities_to_be_serialized = list(filter(_should_register_with_admin, new_api_model_values))
-    serializable_tasks: typing.List[task_models.TaskSpec] = [
-        entity for entity in entities_to_be_serialized if isinstance(entity, task_models.TaskSpec)
-    ]
-    # Detect if any of the tasks is duplicated. Duplicate tasks are defined as having the same
-    # metadata identifiers (see :py:class:`flytekit.common.core.identifier.Identifier`). Duplicate
-    # tasks are considered invalid at registration
-    # time and usually indicate user error, so we catch this common mistake at serialization time.
-    duplicate_tasks = _find_duplicate_tasks(serializable_tasks)
-    if len(duplicate_tasks) > 0:
-        duplicate_task_names = [task.template.id.name for task in duplicate_tasks]
-        raise FlyteValidationException(
-            f"Multiple definitions of the following tasks were found: {duplicate_task_names}"
-        )
 
     return entities_to_be_serialized
 
@@ -123,6 +101,6 @@ def persist_registrable_entities(entities: typing.List[FlyteControlPlaneEntity],
             click.secho(f"Entity is incorrect formatted {entity} - type {type(entity)}", fg="red")
             sys.exit(-1)
         click.secho(f"  Packaging {name} -> {fname}", dim=True)
-        fname = _os.path.join(folder, fname)
+        fname = os.path.join(folder, fname)
         with open(fname, "wb") as writer:
             writer.write(entity.serialize_to_string())
