@@ -1,36 +1,43 @@
 import sys
 import typing
 from collections import OrderedDict
-from dataclasses import dataclass
 from typing import Callable, Dict, List, Optional, Tuple, Union
 
-from flytekit import PythonFunctionTask, SourceCode
-from flytekit.configuration import SerializationSettings
+from flyteidl.admin import schedule_pb2
+
+from flytekit import ImageSpec, PodTemplate, PythonFunctionTask, SourceCode
+from flytekit.configuration import Image, ImageConfig, SerializationSettings
 from flytekit.core import constants as _common_constants
+from flytekit.core import context_manager
+from flytekit.core.array_node import ArrayNode
+from flytekit.core.array_node_map_task import ArrayNodeMapTask
 from flytekit.core.base_task import PythonTask
 from flytekit.core.condition import BranchNode
+from flytekit.core.container_task import ContainerTask
 from flytekit.core.gate import Gate
 from flytekit.core.launch_plan import LaunchPlan, ReferenceLaunchPlan
-from flytekit.core.map_task import MapPythonTask
+from flytekit.core.legacy_map_task import MapPythonTask
 from flytekit.core.node import Node
-from flytekit.core.python_auto_container import PythonAutoContainerTask
+from flytekit.core.options import Options
+from flytekit.core.python_auto_container import (
+    PythonAutoContainerTask,
+)
+from flytekit.core.python_function_task import EagerAsyncPythonFunctionTask
 from flytekit.core.reference_entity import ReferenceEntity, ReferenceSpec, ReferenceTemplate
 from flytekit.core.task import ReferenceTask
-from flytekit.core.utils import _dnsify
+from flytekit.core.utils import ClassDecorator, _dnsify, _serialize_pod_spec
 from flytekit.core.workflow import ReferenceWorkflow, WorkflowBase
 from flytekit.models import common as _common_models
-from flytekit.models import common as common_models
 from flytekit.models import interface as interface_models
 from flytekit.models import launch_plan as _launch_plan_models
-from flytekit.models import security
 from flytekit.models.admin import workflow as admin_workflow_models
 from flytekit.models.admin.workflow import WorkflowSpec
 from flytekit.models.core import identifier as _identifier_model
 from flytekit.models.core import workflow as _core_wf
 from flytekit.models.core import workflow as workflow_model
-from flytekit.models.core.workflow import ApproveCondition
+from flytekit.models.core.workflow import ApproveCondition, GateNode, SignalCondition, SleepCondition, TaskNodeOverrides
+from flytekit.models.core.workflow import ArrayNode as ArrayNodeModel
 from flytekit.models.core.workflow import BranchNode as BranchNodeModel
-from flytekit.models.core.workflow import GateNode, SignalCondition, SleepCondition, TaskNodeOverrides
 from flytekit.models.task import TaskSpec, TaskTemplate
 
 FlyteLocalEntity = Union[
@@ -43,6 +50,7 @@ FlyteLocalEntity = Union[
     ReferenceTask,
     ReferenceLaunchPlan,
     ReferenceEntity,
+    ArrayNode,
 ]
 FlyteControlPlaneEntity = Union[
     TaskSpec,
@@ -50,52 +58,10 @@ FlyteControlPlaneEntity = Union[
     admin_workflow_models.WorkflowSpec,
     workflow_model.Node,
     BranchNodeModel,
+    ArrayNodeModel,
 ]
 
 
-@dataclass
-class Options(object):
-    """
-    These are options that can be configured for a launchplan during registration or overridden during an execution.
-    For instance two people may want to run the same workflow but have the offloaded data stored in two different
-    buckets. Or you may want labels or annotations to be different. This object is used when launching an execution
-    in a Flyte backend, and also when registering launch plans.
-
-    Args:
-        labels: Custom labels to be applied to the execution resource
-        annotations: Custom annotations to be applied to the execution resource
-        security_context: Indicates security context for permissions triggered with this launch plan
-        raw_output_data_config: Optional location of offloaded data for things like S3, etc.
-                remote prefix for storage location of the form ``s3://<bucket>/key...`` or
-           ``gcs://...`` or ``file://...``. If not specified will use the platform configured default. This is where
-           the data for offloaded types is stored.
-        max_parallelism: Controls the maximum number of tasknodes that can be run in parallel for the entire workflow.
-        notifications: List of notifications for this execution
-        disable_notifications: This should be set to true if all notifications are intended to be disabled for this execution.
-    """
-
-    labels: typing.Optional[common_models.Labels] = None
-    annotations: typing.Optional[common_models.Annotations] = None
-    raw_output_data_config: typing.Optional[common_models.RawOutputDataConfig] = None
-    security_context: typing.Optional[security.SecurityContext] = None
-    max_parallelism: typing.Optional[int] = None
-    notifications: typing.Optional[typing.List[common_models.Notification]] = None
-    disable_notifications: typing.Optional[bool] = None
-
-    @classmethod
-    def default_from(
-        cls, k8s_service_account: typing.Optional[str] = None, raw_data_prefix: typing.Optional[str] = None
-    ) -> "Options":
-        return cls(
-            security_context=security.SecurityContext(run_as=security.Identity(k8s_service_account=k8s_service_account))
-            if k8s_service_account
-            else None,
-            raw_output_data_config=common_models.RawOutputDataConfig(output_location_prefix=raw_data_prefix)
-            if raw_data_prefix
-            else None,
-        )
-
-
 def to_serializable_case(
     entity_mapping: OrderedDict,
     settings: SerializationSettings,
@@ -156,8 +122,10 @@ def _fast_serialize_command_fn(
 
 
 def get_serializable_task(
+    entity_mapping: OrderedDict,
     settings: SerializationSettings,
     entity: FlyteLocalEntity,
+    options: Optional[Options] = None,
 ) -> TaskSpec:
     task_id = _identifier_model.Identifier(
         _identifier_model.ResourceType.TASK,
@@ -168,18 +136,36 @@ def get_serializable_task(
     )
 
     if isinstance(entity, PythonFunctionTask) and entity.execution_mode == PythonFunctionTask.ExecutionBehavior.DYNAMIC:
+        for e in context_manager.FlyteEntities.entities:
+            if isinstance(e, PythonAutoContainerTask):
+                # 1. Build the ImageSpec for all the entities that are inside the current context,
+                # 2. Add images to the serialization context, so the dynamic task can look it up at runtime.
+                if isinstance(e.container_image, ImageSpec):
+                    if settings.image_config.images is None:
+                        settings.image_config = ImageConfig.create_from(settings.image_config.default_image)
+                    img = Image.look_up_image_info(e.container_image.id, e.get_image(settings))
+                    if img not in settings.image_config.images:
+                        settings.image_config.images.append(img)
+
         # In case of Dynamic tasks, we want to pass the serialization context, so that they can reconstruct the state
         # from the serialization context. This is passed through an environment variable, that is read from
         # during dynamic serialization
         settings = settings.with_serialized_context()
 
+        if entity.node_dependency_hints is not None:
+            for entity_hint in entity.node_dependency_hints:
+                get_serializable(entity_mapping, settings, entity_hint, options)
+
+    if isinstance(entity, EagerAsyncPythonFunctionTask):
+        settings = settings.with_serialized_context()
+
     container = entity.get_container(settings)
     # This pod will be incorrect when doing fast serialize
     pod = entity.get_k8s_pod(settings)
 
     if settings.should_fast_serialize():
         # This handles container tasks.
-        if container and isinstance(entity, (PythonAutoContainerTask, MapPythonTask)):
+        if container and isinstance(entity, (PythonAutoContainerTask, MapPythonTask, ArrayNodeMapTask)):
             # For fast registration, we'll need to muck with the command, but on
             # ly for certain kinds of tasks. Specifically,
             # tasks that rely on user code defined in the container. This should be encapsulated by the auto container
@@ -189,8 +175,8 @@ def get_serializable_task(
         # If the pod spec is not None, we have to get it again, because the one we retrieved above will be incorrect.
         # The reason we have to call get_k8s_pod again, instead of just modifying the command in this file, is because
         # the pod spec is a K8s library object, and we shouldn't be messing around with it in this file.
-        elif pod:
-            if isinstance(entity, MapPythonTask):
+        elif pod and not isinstance(entity, ContainerTask):
+            if isinstance(entity, (MapPythonTask, ArrayNodeMapTask)):
                 entity.set_command_prefix(get_command_prefix_for_fast_execute(settings))
                 pod = entity.get_k8s_pod(settings)
             else:
@@ -198,6 +184,17 @@ def get_serializable_task(
                 pod = entity.get_k8s_pod(settings)
                 entity.reset_command_fn()
 
+    entity_config = entity.get_config(settings) or {}
+    extra_config = {}
+
+    if hasattr(entity, "task_function"):
+        if isinstance(entity.task_function, ClassDecorator):
+            extra_config = entity.task_function.get_extra_config()
+    if entity.enable_deck:
+        entity.metadata.generates_deck = True
+
+    merged_config = {**entity_config, **extra_config}
+
     tt = TaskTemplate(
         id=task_id,
         type=entity.task_type,
@@ -207,9 +204,10 @@ def get_serializable_task(
         container=container,
         task_type_version=entity.task_type_version,
         security_context=entity.security_context,
-        config=entity.get_config(settings),
+        config=merged_config,
         k8s_pod=pod,
         sql=entity.get_sql(settings),
+        extended_resources=entity.get_extended_resources(settings),
     )
     if settings.should_fast_serialize() and isinstance(entity, PythonAutoContainerTask):
         entity.reset_command_fn()
@@ -231,6 +229,10 @@ def get_serializable_workflow(
         if n.id == _common_constants.GLOBAL_INPUT_NODE_ID:
             continue
 
+        # Ensure no node is named the failure node id
+        if n.id == _common_constants.DEFAULT_FAILURE_NODE_ID:
+            raise ValueError(f"Node {n.id} is reserved for the failure node")
+
         # Recursively serialize the node
         serialized_nodes.append(get_serializable(entity_mapping, settings, n, options))
 
@@ -241,7 +243,7 @@ def get_serializable_workflow(
             # require a network call to flyteadmin to populate the WorkflowTemplate
             # object
             if isinstance(n.flyte_entity, ReferenceWorkflow):
-                raise Exception(
+                raise ValueError(
                     "Reference sub-workflows are currently unsupported. Use reference launch plans instead."
                 )
             sub_wf_spec = get_serializable(entity_mapping, settings, n.flyte_entity, options)
@@ -283,6 +285,14 @@ def get_serializable_workflow(
                     sub_wfs.append(leaf_node.flyte_entity)
                     sub_wfs.extend([s for s in leaf_node.flyte_entity.sub_workflows.values()])
 
+    serialized_failure_node = None
+    if entity.failure_node:
+        serialized_failure_node = get_serializable(entity_mapping, settings, entity.failure_node, options)
+        if isinstance(entity.failure_node.flyte_entity, WorkflowBase):
+            sub_wf_spec = get_serializable(entity_mapping, settings, entity.failure_node.flyte_entity, options)
+            sub_wfs.append(sub_wf_spec.template)
+            sub_wfs.extend(sub_wf_spec.sub_workflows)
+
     wf_id = _identifier_model.Identifier(
         resource_type=_identifier_model.ResourceType.WORKFLOW,
         project=settings.project,
@@ -290,6 +300,7 @@ def get_serializable_workflow(
         name=entity.name,
         version=settings.version,
     )
+
     wf_t = workflow_model.WorkflowTemplate(
         id=wf_id,
         metadata=entity.workflow_metadata.to_flyte_model(),
@@ -297,6 +308,7 @@ def get_serializable_workflow(
         interface=entity.interface,
         nodes=serialized_nodes,
         outputs=entity.output_bindings,
+        failure_node=serialized_failure_node,
     )
 
     return admin_workflow_models.WorkflowSpec(
@@ -339,11 +351,19 @@ def get_serializable_launch_plan(
     else:
         raw_prefix_config = entity.raw_output_data_config or _common_models.RawOutputDataConfig("")
 
+    if entity.trigger:
+        lc = entity.trigger.to_flyte_idl(entity)
+        if isinstance(lc, schedule_pb2.Schedule):
+            raise ValueError("Please continue to use the schedule arg, the trigger arg is not implemented yet")
+    else:
+        lc = None
+
     lps = _launch_plan_models.LaunchPlanSpec(
         workflow_id=wf_id,
         entity_metadata=_launch_plan_models.LaunchPlanMetadata(
             schedule=entity.schedule,
             notifications=options.notifications or entity.notifications,
+            launch_conditions=lc,
         ),
         default_inputs=entity.parameters,
         fixed_inputs=entity.fixed_inputs,
@@ -353,6 +373,7 @@ def get_serializable_launch_plan(
         raw_output_data_config=raw_prefix_config,
         max_parallelism=options.max_parallelism or entity.max_parallelism,
         security_context=options.security_context or entity.security_context,
+        overwrite_cache=options.overwrite_cache or entity.overwrite_cache,
     )
 
     lp_id = _identifier_model.Identifier(
@@ -370,6 +391,7 @@ def get_serializable_launch_plan(
             expected_inputs=interface_models.ParameterMap({}),
             expected_outputs=interface_models.VariableMap({}),
         ),
+        auto_activate=entity.should_auto_activate,
     )
 
     return lp_model
@@ -382,7 +404,7 @@ def get_serializable_node(
     options: Optional[Options] = None,
 ) -> workflow_model.Node:
     if entity.flyte_entity is None:
-        raise Exception(f"Node {entity.id} has no flyte entity")
+        raise ValueError(f"Node {entity.id} has no flyte entity")
 
     upstream_nodes = [
         get_serializable(entity_mapping, settings, n, options=options)
@@ -408,14 +430,43 @@ def get_serializable_node(
         elif ref_template.resource_type == _identifier_model.ResourceType.LAUNCH_PLAN:
             node_model._workflow_node = workflow_model.WorkflowNode(launchplan_ref=ref_template.id)
         else:
-            raise Exception(
+            raise TypeError(
                 f"Unexpected resource type for reference entity {entity.flyte_entity}: {ref_template.resource_type}"
             )
         return node_model
 
     from flytekit.remote import FlyteLaunchPlan, FlyteTask, FlyteWorkflow
 
-    if isinstance(entity.flyte_entity, PythonTask):
+    if isinstance(entity.flyte_entity, ArrayNode):
+        node_model = workflow_model.Node(
+            id=_dnsify(entity.id),
+            metadata=entity.metadata,
+            inputs=entity.bindings,
+            upstream_node_ids=[n.id for n in upstream_nodes],
+            output_aliases=[],
+            array_node=get_serializable_array_node(entity_mapping, settings, entity, options=options),
+        )
+    elif isinstance(entity.flyte_entity, ArrayNodeMapTask):
+        node_model = workflow_model.Node(
+            id=_dnsify(entity.id),
+            metadata=entity.metadata,
+            inputs=entity.bindings,
+            upstream_node_ids=[n.id for n in upstream_nodes],
+            output_aliases=[],
+            array_node=get_serializable_array_node_map_task(entity_mapping, settings, entity, options=options),
+        )
+        # TODO: do I need this?
+        # if entity._aliases:
+        #     node_model._output_aliases = entity._aliases
+    elif isinstance(entity.flyte_entity, PythonTask):
+        # handle pod template overrides
+        override_pod_spec = {}
+        if entity._pod_template is not None:
+            if settings.should_fast_serialize():
+                entity.flyte_entity.set_command_fn(_fast_serialize_command_fn(settings, entity.flyte_entity))
+            override_pod_spec = _serialize_pod_spec(
+                entity._pod_template, entity.flyte_entity._get_container(settings), settings
+            )
         task_spec = get_serializable(entity_mapping, settings, entity.flyte_entity, options=options)
         node_model = workflow_model.Node(
             id=_dnsify(entity.id),
@@ -424,7 +475,22 @@ def get_serializable_node(
             upstream_node_ids=[n.id for n in upstream_nodes],
             output_aliases=[],
             task_node=workflow_model.TaskNode(
-                reference_id=task_spec.template.id, overrides=TaskNodeOverrides(resources=entity._resources)
+                reference_id=task_spec.template.id,
+                overrides=TaskNodeOverrides(
+                    resources=entity._resources,
+                    extended_resources=entity._extended_resources,
+                    container_image=entity._container_image,
+                    pod_template=PodTemplate(
+                        pod_spec=override_pod_spec,
+                        labels=entity._pod_template.labels if entity._pod_template.labels else None,
+                        annotations=entity._pod_template.annotations if entity._pod_template.annotations else None,
+                        primary_container_name=entity._pod_template.primary_container_name
+                        if entity._pod_template.primary_container_name
+                        else None,
+                    )
+                    if entity._pod_template
+                    else None,
+                ),
             ),
         )
         if entity._aliases:
@@ -500,7 +566,12 @@ def get_serializable_node(
             upstream_node_ids=[n.id for n in upstream_nodes],
             output_aliases=[],
             task_node=workflow_model.TaskNode(
-                reference_id=entity.flyte_entity.id, overrides=TaskNodeOverrides(resources=entity._resources)
+                reference_id=entity.flyte_entity.id,
+                overrides=TaskNodeOverrides(
+                    resources=entity._resources,
+                    extended_resources=entity._extended_resources,
+                    container_image=entity._container_image,
+                ),
             ),
         )
     elif isinstance(entity.flyte_entity, FlyteWorkflow):
@@ -533,11 +604,68 @@ def get_serializable_node(
             workflow_node=workflow_model.WorkflowNode(launchplan_ref=entity.flyte_entity.id),
         )
     else:
-        raise Exception(f"Node contained non-serializable entity {entity._flyte_entity}")
+        raise ValueError(f"Node contained non-serializable entity {entity._flyte_entity}")
 
     return node_model
 
 
+def get_serializable_array_node(
+    entity_mapping: OrderedDict,
+    settings: SerializationSettings,
+    node: FlyteLocalEntity,
+    options: Optional[Options] = None,
+) -> ArrayNodeModel:
+    array_node = node.flyte_entity
+    # pass in parent node metadata to be set for subnode
+    array_node.metadata = node.metadata
+    return ArrayNodeModel(
+        node=get_serializable_node(entity_mapping, settings, array_node, options=options),
+        parallelism=array_node.concurrency,
+        min_successes=array_node.min_successes,
+        min_success_ratio=array_node.min_success_ratio,
+        execution_mode=array_node.execution_mode,
+        is_original_sub_node_interface=array_node.is_original_sub_node_interface,
+        data_mode=array_node.data_mode,
+        bound_inputs=array_node.bound_inputs,
+    )
+
+
+def get_serializable_array_node_map_task(
+    entity_mapping: OrderedDict,
+    settings: SerializationSettings,
+    node: Node,
+    options: Optional[Options] = None,
+) -> ArrayNodeModel:
+    # TODO Add support for other flyte entities
+    entity = node.flyte_entity
+    task_spec = get_serializable(entity_mapping, settings, entity, options)
+    task_node = workflow_model.TaskNode(
+        reference_id=task_spec.template.id,
+        overrides=TaskNodeOverrides(
+            resources=node._resources,
+            extended_resources=node._extended_resources,
+            container_image=node._container_image,
+        ),
+    )
+    node = workflow_model.Node(
+        id=entity.name,
+        metadata=entity.sub_node_metadata,
+        inputs=node.bindings,
+        upstream_node_ids=[],
+        output_aliases=[],
+        task_node=task_node,
+    )
+    return ArrayNodeModel(
+        node=node,
+        parallelism=entity.concurrency,
+        min_successes=entity.min_successes,
+        min_success_ratio=entity.min_success_ratio,
+        execution_mode=entity.execution_mode,
+        is_original_sub_node_interface=entity.is_original_sub_node_interface,
+        bound_inputs=entity.bound_inputs,
+    )
+
+
 def get_serializable_branch_node(
     entity_mapping: OrderedDict,
     settings: SerializationSettings,
@@ -648,7 +776,7 @@ def get_serializable(
         cp_entity = get_reference_spec(entity_mapping, settings, entity)
 
     elif isinstance(entity, PythonTask):
-        cp_entity = get_serializable_task(settings, entity)
+        cp_entity = get_serializable_task(entity_mapping, settings, entity)
 
     elif isinstance(entity, WorkflowBase):
         cp_entity = get_serializable_workflow(entity_mapping, settings, entity, options)
@@ -662,11 +790,6 @@ def get_serializable(
     elif isinstance(entity, BranchNode):
         cp_entity = get_serializable_branch_node(entity_mapping, settings, entity, options)
 
-    elif isinstance(entity, GateNode):
-        import ipdb
-
-        ipdb.set_trace()
-
     elif isinstance(entity, FlyteTask) or isinstance(entity, FlyteWorkflow):
         if entity.should_register:
             if isinstance(entity, FlyteTask):
@@ -684,8 +807,11 @@ def get_serializable(
     elif isinstance(entity, FlyteLaunchPlan):
         cp_entity = entity
 
+    elif isinstance(entity, ArrayNode):
+        cp_entity = get_serializable_array_node(entity_mapping, settings, entity, options)
+
     else:
-        raise Exception(f"Non serializable type found {type(entity)} Entity {entity}")
+        raise ValueError(f"Non serializable type found {type(entity)} Entity {entity}")
 
     if isinstance(entity, TaskSpec) or isinstance(entity, WorkflowSpec):
         # 1. Check if the size of long description exceeds 16KB
@@ -713,7 +839,7 @@ def gather_dependent_entities(
     The ``get_serializable`` function above takes in an ``OrderedDict`` that helps keep track of dependent entities.
     For example, when serializing a workflow, all its tasks are also serialized. The ordered dict will also contain
     serialized entities that aren't as useful though, like nodes and branches. This is just a small helper function
-    that will pull out the serialzed tasks, workflows, and launch plans. This function is primarily used for testing.
+    that will pull out the serialized tasks, workflows, and launch plans. This function is primarily used for testing.
 
     :param serialized: This should be the filled in OrderedDict used in the get_serializable function above.
     :return:
