@@ -1,22 +1,17 @@
 """
-Flytekit Directory Type
-==========================================================
-.. currentmodule:: flytekit.types.directory
+Similar to {{< py_class_ref flytekit.types.file.FlyteFile >}} there are some 'preformatted' directory types.
 
-Similar to :py:class:`flytekit.types.file.FlyteFile` there are some 'preformatted' directory types.
+| Class | Description |
+|-------| ---- |
+| {{< py_class_ref FlyteDirectory >}} | {{< py_class_docsum FlyteDirectory >}} |
+| {{< py_class_ref TensorboardLogs >}} | {{< py_class_docsum TensorboardLogs >}} |
+| {{< py_class_ref TFRecordsDirectory >}} | {{< py_class_docsum TFRecordsDirectory >}} |
 
-.. autosummary::
-   :toctree: generated/
-   :template: file_types.rst
-
-   FlyteDirectory
-   TensorboardLogs
-   TFRecordsDirectory
 """
 
 import typing
 
-from .types import FlyteDirectory
+from .types import FlyteDirectory, FlyteDirToMultipartBlobTransformer
 
 # The following section provides some predefined aliases for commonly used FlyteDirectory formats.
 
@@ -28,7 +23,7 @@ TensorboardLogs = FlyteDirectory[tensorboard]
     TensorBoard.
 """
 
-tfrecords_dir = typing.TypeVar("tfrecord")
+tfrecords_dir = typing.TypeVar("tfrecords_dir")
 TFRecordsDirectory = FlyteDirectory[tfrecords_dir]
 """
     This type can be used to denote that the output is a folder that contains tensorflow record files.
