@@ -7,24 +7,41 @@ import numpy as np
 from typing_extensions import Annotated, get_args, get_origin
 
 from flytekit.core.context_manager import FlyteContext
-from flytekit.core.type_engine import TypeEngine, TypeTransformer, TypeTransformerFailedError
+from flytekit.core.hash import HashMethod
+from flytekit.core.type_engine import (
+    AsyncTypeTransformer,
+    TypeEngine,
+    TypeTransformerFailedError,
+)
 from flytekit.models.core import types as _core_types
 from flytekit.models.literals import Blob, BlobMetadata, Literal, Scalar
 from flytekit.models.types import LiteralType
 
 
 def extract_metadata(t: Type[np.ndarray]) -> Tuple[Type[np.ndarray], Dict[str, bool]]:
-    metadata = {}
+    metadata: dict = {}
+    metadata_set = False
+
     if get_origin(t) is Annotated:
-        base_type, metadata = get_args(t)
-        if isinstance(metadata, OrderedDict):
-            return base_type, metadata
-        else:
-            raise TypeTransformerFailedError(f"{t}'s metadata needs to be of type kwtypes.")
+        base_type, *annotate_args = get_args(t)
+
+        for aa in annotate_args:
+            if isinstance(aa, OrderedDict):
+                if metadata_set:
+                    raise TypeTransformerFailedError(f"Metadata {metadata} is already specified, cannot use {aa}.")
+                metadata = aa
+                metadata_set = True
+            elif isinstance(aa, HashMethod):
+                continue
+            else:
+                raise TypeTransformerFailedError(f"The metadata for {t} must be of type kwtypes or HashMethod.")
+        return base_type, metadata
+
+    # Return the type itself if no metadata was found.
     return t, metadata
 
 
-class NumpyArrayTransformer(TypeTransformer[np.ndarray]):
+class NumpyArrayTransformer(AsyncTypeTransformer[np.ndarray]):
     """
     TypeTransformer that supports np.ndarray as a native type.
     """
@@ -37,18 +54,24 @@ class NumpyArrayTransformer(TypeTransformer[np.ndarray]):
     def get_literal_type(self, t: Type[np.ndarray]) -> LiteralType:
         return LiteralType(
             blob=_core_types.BlobType(
-                format=self.NUMPY_ARRAY_FORMAT, dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE
+                format=self.NUMPY_ARRAY_FORMAT,
+                dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE,
             )
         )
 
-    def to_literal(
-        self, ctx: FlyteContext, python_val: np.ndarray, python_type: Type[np.ndarray], expected: LiteralType
+    async def async_to_literal(
+        self,
+        ctx: FlyteContext,
+        python_val: np.ndarray,
+        python_type: Type[np.ndarray],
+        expected: LiteralType,
     ) -> Literal:
         python_type, metadata = extract_metadata(python_type)
 
         meta = BlobMetadata(
             type=_core_types.BlobType(
-                format=self.NUMPY_ARRAY_FORMAT, dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE
+                format=self.NUMPY_ARRAY_FORMAT,
+                dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE,
             )
         )
 
@@ -56,13 +79,17 @@ class NumpyArrayTransformer(TypeTransformer[np.ndarray]):
         pathlib.Path(local_path).parent.mkdir(parents=True, exist_ok=True)
 
         # save numpy array to file
-        np.save(file=local_path, arr=python_val, allow_pickle=metadata.get("allow_pickle", False))
-
-        remote_path = ctx.file_access.get_random_remote_path(local_path)
-        ctx.file_access.put_data(local_path, remote_path, is_multipart=False)
+        np.save(
+            file=local_path,
+            arr=python_val,
+            allow_pickle=metadata.get("allow_pickle", False),
+        )
+        remote_path = await ctx.file_access.async_put_raw_data(local_path)
         return Literal(scalar=Scalar(blob=Blob(metadata=meta, uri=remote_path)))
 
-    def to_python_value(self, ctx: FlyteContext, lv: Literal, expected_python_type: Type[np.ndarray]) -> np.ndarray:
+    async def async_to_python_value(
+        self, ctx: FlyteContext, lv: Literal, expected_python_type: Type[np.ndarray]
+    ) -> np.ndarray:
         try:
             uri = lv.scalar.blob.uri
         except AttributeError:
@@ -71,13 +98,13 @@ class NumpyArrayTransformer(TypeTransformer[np.ndarray]):
         expected_python_type, metadata = extract_metadata(expected_python_type)
 
         local_path = ctx.file_access.get_random_local_path()
-        ctx.file_access.get_data(uri, local_path, is_multipart=False)
+        await ctx.file_access.async_get_data(uri, local_path, is_multipart=False)
 
         # load numpy array from a file
         return np.load(
             file=local_path,
             allow_pickle=metadata.get("allow_pickle", False),
-            mmap_mode=metadata.get("mmap_mode"),
+            mmap_mode=metadata.get("mmap_mode"),  # type: ignore
         )
 
     def guess_python_type(self, literal_type: LiteralType) -> typing.Type[np.ndarray]:
