@@ -1,23 +1,29 @@
 from __future__ import annotations
 
-import datetime as _datetime
+import datetime
+import json
 import os
 import typing
 from abc import abstractmethod
 from dataclasses import dataclass, field
 from enum import Enum
 from pathlib import Path
-from typing import Type
+from typing import Dict, Optional, Type
 
-import numpy as _np
-import pandas
-from dataclasses_json import config, dataclass_json
+import msgpack
+from dataclasses_json import config
+from google.protobuf import json_format as _json_format
+from google.protobuf.struct_pb2 import Struct
 from marshmallow import fields
+from mashumaro.mixins.json import DataClassJSONMixin
+from mashumaro.types import SerializableType
 
+from flytekit.core.constants import MESSAGEPACK
 from flytekit.core.context_manager import FlyteContext, FlyteContextManager
-from flytekit.core.type_engine import TypeEngine, TypeTransformer, TypeTransformerFailedError
+from flytekit.core.type_engine import AsyncTypeTransformer, TypeEngine, TypeTransformerFailedError
+from flytekit.extras.pydantic_transformer.decorator import model_serializer, model_validator
 from flytekit.loggers import logger
-from flytekit.models.literals import Literal, Scalar, Schema
+from flytekit.models.literals import Binary, Literal, Scalar, Schema
 from flytekit.models.types import LiteralType, SchemaType
 
 T = typing.TypeVar("T")
@@ -68,12 +74,10 @@ class SchemaReader(typing.Generic[T]):
         return None
 
     @abstractmethod
-    def iter(self, **kwargs) -> typing.Generator[T, None, None]:
-        ...
+    def iter(self, **kwargs) -> typing.Generator[T, None, None]: ...
 
     @abstractmethod
-    def all(self, **kwargs) -> T:
-        ...
+    def all(self, **kwargs) -> T: ...
 
 
 class SchemaWriter(typing.Generic[T]):
@@ -95,37 +99,42 @@ class SchemaWriter(typing.Generic[T]):
         return None
 
     @abstractmethod
-    def write(self, *dfs, **kwargs):
-        ...
+    def write(self, *dfs, **kwargs): ...
 
 
 class LocalIOSchemaReader(SchemaReader[T]):
-    def __init__(self, from_path: os.PathLike, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
-        super().__init__(str(from_path), cols, fmt)
+    def __init__(self, from_path: str, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
+        super().__init__(from_path, cols, fmt)
 
     @abstractmethod
     def _read(self, *path: os.PathLike, **kwargs) -> T:
         pass
 
     def iter(self, **kwargs) -> typing.Generator[T, None, None]:
-        with os.scandir(self._from_path) as it:
+        with os.scandir(self._from_path) as it:  # type: ignore
             for entry in it:
-                if not entry.name.startswith(".") and entry.is_file():
-                    yield self._read(Path(entry.path), **kwargs)
+                if (
+                    not typing.cast(os.DirEntry, entry).name.startswith(".")
+                    and typing.cast(os.DirEntry, entry).is_file()
+                ):
+                    yield self._read(Path(typing.cast(os.DirEntry, entry).path), **kwargs)
 
     def all(self, **kwargs) -> T:
         files: typing.List[os.PathLike] = []
-        with os.scandir(self._from_path) as it:
+        with os.scandir(self._from_path) as it:  # type: ignore
             for entry in it:
-                if not entry.name.startswith(".") and entry.is_file():
-                    files.append(Path(entry.path))
+                if (
+                    not typing.cast(os.DirEntry, entry).name.startswith(".")
+                    and typing.cast(os.DirEntry, entry).is_file()
+                ):
+                    files.append(Path(typing.cast(os.DirEntry, entry).path))
 
         return self._read(*files, **kwargs)
 
 
 class LocalIOSchemaWriter(SchemaWriter[T]):
-    def __init__(self, to_local_path: os.PathLike, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
-        super().__init__(str(to_local_path), cols, fmt)
+    def __init__(self, to_local_path: str, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
+        super().__init__(to_local_path, cols, fmt)
 
     @abstractmethod
     def _write(self, df: T, path: os.PathLike, **kwargs):
@@ -173,14 +182,48 @@ class SchemaEngine(object):
         return cls._SCHEMA_HANDLERS[t]
 
 
-@dataclass_json
 @dataclass
-class FlyteSchema(object):
-    remote_path: typing.Optional[os.PathLike] = field(default=None, metadata=config(mm_field=fields.String()))
+class FlyteSchema(SerializableType, DataClassJSONMixin):
+    remote_path: typing.Optional[str] = field(default=None, metadata=config(mm_field=fields.String()))
     """
     This is the main schema class that users should use.
     """
 
+    def _serialize(self) -> Dict[str, Optional[str]]:
+        FlyteSchemaTransformer().to_literal(FlyteContextManager.current_context(), self, type(self), None)
+        return {"remote_path": self.remote_path}
+
+    @classmethod
+    def _deserialize(cls, value) -> "FlyteSchema":
+        remote_path = value.get("remote_path", None)
+
+        if remote_path is None:
+            raise ValueError("FlyteSchema's path should not be None")
+
+        t = FlyteSchemaTransformer()
+        return t.to_python_value(
+            FlyteContextManager.current_context(),
+            Literal(scalar=Scalar(schema=Schema(remote_path, t._get_schema_type(cls)))),
+            cls,
+        )
+
+    @model_serializer
+    def serialize_flyte_schema(self) -> Dict[str, Optional[str]]:
+        FlyteSchemaTransformer().to_literal(FlyteContextManager.current_context(), self, type(self), None)
+        return {"remote_path": self.remote_path}
+
+    @model_validator(mode="after")
+    def deserialize_flyte_schema(self, info) -> FlyteSchema:
+        if info.context is None or info.context.get("deserialize") is not True:
+            return self
+
+        t = FlyteSchemaTransformer()
+        return t.to_python_value(
+            FlyteContextManager.current_context(),
+            Literal(scalar=Scalar(schema=Schema(self.remote_path, t._get_schema_type(type(self))))),
+            type(self),
+        )
+
     @classmethod
     def columns(cls) -> typing.Dict[str, typing.Type]:
         return {}
@@ -196,6 +239,7 @@ class FlyteSchema(object):
     def __class_getitem__(
         cls, columns: typing.Dict[str, typing.Type], fmt: SchemaFormat = SchemaFormat.PARQUET
     ) -> Type[FlyteSchema]:
+        logger.warning("FlyteSchema is deprecated, use Structured Dataset instead.")
         if columns is None:
             return FlyteSchema
 
@@ -216,6 +260,18 @@ class FlyteSchema(object):
             # Get the type engine to see this as kind of a generic
             __origin__ = FlyteSchema
 
+            class AttributeHider:
+                def __get__(self, instance, owner):
+                    raise AttributeError(
+                        """We have to return false in hasattr(cls, "__class_getitem__") to make mashumaro deserialize FlyteSchema correctly."""
+                    )
+
+            # Set __class_getitem__ to AttributeHider to make mashumaro deserialize FlyteSchema correctly
+            # https://stackoverflow.com/questions/6057130/python-deleting-a-class-attribute-in-a-subclass/6057409
+            # Since mashumaro will use the method __class_getitem__ and __origin__ to construct the dataclass back
+            # https://github.com/Fatal1ty/mashumaro/blob/e945ee4319db49da9f7b8ede614e988cc8c8956b/mashumaro/core/meta/helpers.py#L300-L303
+            __class_getitem__ = AttributeHider()  # type: ignore
+
             @classmethod
             def columns(cls) -> typing.Dict[str, typing.Type]:
                 return columns
@@ -228,12 +284,12 @@ class FlyteSchema(object):
 
     def __init__(
         self,
-        local_path: os.PathLike = None,
-        remote_path: os.PathLike = None,
+        local_path: typing.Optional[str] = None,
+        remote_path: typing.Optional[str] = None,
         supported_mode: SchemaOpenMode = SchemaOpenMode.WRITE,
-        downloader: typing.Callable[[str, os.PathLike], None] = None,
+        downloader: typing.Optional[typing.Callable] = None,
     ):
-
+        logger.warning("FlyteSchema is deprecated, use Structured Dataset instead.")
         if supported_mode == SchemaOpenMode.READ and remote_path is None:
             raise ValueError("To create a FlyteSchema in read mode, remote_path is required")
         if (
@@ -247,14 +303,15 @@ class FlyteSchema(object):
         self._local_path = local_path
         # Make this field public, so that the dataclass transformer can set a value for it
         # https://github.com/flyteorg/flytekit/blob/bcc8541bd6227b532f8462563fe8aac902242b21/flytekit/core/type_engine.py#L298
-        self.remote_path = remote_path or FlyteContextManager.current_context().file_access.get_random_remote_path()
+        fp = FlyteContextManager.current_context().file_access
+        self.remote_path = remote_path or fp.join(fp.raw_output_prefix, fp.get_random_string())
         self._supported_mode = supported_mode
         # This is a special attribute that indicates if the data was either downloaded or uploaded
         self._downloaded = False
         self._downloader = downloader
 
     @property
-    def local_path(self) -> os.PathLike:
+    def local_path(self) -> str:
         return self._local_path
 
     @property
@@ -262,7 +319,7 @@ class FlyteSchema(object):
         return self._supported_mode
 
     def open(
-        self, dataframe_fmt: type = pandas.DataFrame, override_mode: SchemaOpenMode = None
+        self, dataframe_fmt: typing.Optional[type] = None, override_mode: typing.Optional[SchemaOpenMode] = None
     ) -> typing.Union[SchemaReader, SchemaWriter]:
         """
         Returns a reader or writer depending on the mode of the object when created. This mode can be
@@ -279,6 +336,9 @@ class FlyteSchema(object):
             raise AssertionError("Readonly schema cannot be opened in write mode!")
 
         mode = override_mode if override_mode else self._supported_mode
+        import pandas as pd
+
+        dataframe_fmt = dataframe_fmt if dataframe_fmt else pd.DataFrame
         h = SchemaEngine.get_handler(dataframe_fmt)
         if not h.handles_remote_io:
             # The Schema Handler does not manage its own IO, and this it will expect the files are on local file-system
@@ -290,13 +350,13 @@ class FlyteSchema(object):
                 self._downloader(self.remote_path, self.local_path)
                 self._downloaded = True
             if mode == SchemaOpenMode.WRITE:
-                return h.writer(typing.cast(str, self.local_path), self.columns(), self.format())
-            return h.reader(typing.cast(str, self.local_path), self.columns(), self.format())
+                return h.writer(self.local_path, self.columns(), self.format())
+            return h.reader(self.local_path, self.columns(), self.format())
 
         # Remote IO is handled. So we will just pass the remote reference to the object
         if mode == SchemaOpenMode.WRITE:
-            return h.writer(self.remote_path, self.columns(), self.format())
-        return h.reader(self.remote_path, self.columns(), self.format())
+            return h.writer(typing.cast(str, self.remote_path), self.columns(), self.format())
+        return h.reader(typing.cast(str, self.remote_path), self.columns(), self.format())
 
     def as_readonly(self) -> FlyteSchema:
         if self._supported_mode == SchemaOpenMode.READ:
@@ -304,34 +364,46 @@ class FlyteSchema(object):
         s = FlyteSchema.__class_getitem__(self.columns(), self.format())(
             local_path=self.local_path,
             # Dummy path is ok, as we will assume data is already downloaded and will not download again
-            remote_path=self.remote_path if self.remote_path else "",
+            remote_path=typing.cast(str, self.remote_path) if self.remote_path else "",
             supported_mode=SchemaOpenMode.READ,
         )
         s._downloaded = True
         return s
 
 
-class FlyteSchemaTransformer(TypeTransformer[FlyteSchema]):
+def _get_numpy_type_mappings() -> typing.Dict[Type, SchemaType.SchemaColumn.SchemaColumnType]:
+    try:
+        import numpy as _np
+
+        return {
+            _np.int32: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
+            _np.int64: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
+            _np.uint32: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
+            _np.uint64: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
+            _np.float32: SchemaType.SchemaColumn.SchemaColumnType.FLOAT,
+            _np.float64: SchemaType.SchemaColumn.SchemaColumnType.FLOAT,
+            _np.bool_: SchemaType.SchemaColumn.SchemaColumnType.BOOLEAN,  # type: ignore
+            _np.datetime64: SchemaType.SchemaColumn.SchemaColumnType.DATETIME,
+            _np.timedelta64: SchemaType.SchemaColumn.SchemaColumnType.DURATION,
+            _np.bytes_: SchemaType.SchemaColumn.SchemaColumnType.STRING,
+            _np.str_: SchemaType.SchemaColumn.SchemaColumnType.STRING,
+            _np.object_: SchemaType.SchemaColumn.SchemaColumnType.STRING,
+        }
+    except ImportError as e:
+        logger.debug("Numpy not found, skipping numpy type mappings, error: %s", e)
+        return {}
+
+
+class FlyteSchemaTransformer(AsyncTypeTransformer[FlyteSchema]):
     _SUPPORTED_TYPES: typing.Dict[Type, SchemaType.SchemaColumn.SchemaColumnType] = {
-        _np.int32: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
-        _np.int64: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
-        _np.uint32: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
-        _np.uint64: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
-        int: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
-        _np.float32: SchemaType.SchemaColumn.SchemaColumnType.FLOAT,
-        _np.float64: SchemaType.SchemaColumn.SchemaColumnType.FLOAT,
         float: SchemaType.SchemaColumn.SchemaColumnType.FLOAT,
-        _np.bool: SchemaType.SchemaColumn.SchemaColumnType.BOOLEAN,  # type: ignore
+        int: SchemaType.SchemaColumn.SchemaColumnType.INTEGER,
         bool: SchemaType.SchemaColumn.SchemaColumnType.BOOLEAN,
-        _np.datetime64: SchemaType.SchemaColumn.SchemaColumnType.DATETIME,
-        _datetime.datetime: SchemaType.SchemaColumn.SchemaColumnType.DATETIME,
-        _np.timedelta64: SchemaType.SchemaColumn.SchemaColumnType.DURATION,
-        _datetime.timedelta: SchemaType.SchemaColumn.SchemaColumnType.DURATION,
-        _np.string_: SchemaType.SchemaColumn.SchemaColumnType.STRING,
-        _np.str_: SchemaType.SchemaColumn.SchemaColumnType.STRING,
-        _np.object_: SchemaType.SchemaColumn.SchemaColumnType.STRING,
         str: SchemaType.SchemaColumn.SchemaColumnType.STRING,
+        datetime.datetime: SchemaType.SchemaColumn.SchemaColumnType.DATETIME,
+        datetime.timedelta: SchemaType.SchemaColumn.SchemaColumnType.DURATION,
     }
+    _SUPPORTED_TYPES.update(_get_numpy_type_mappings())
 
     def __init__(self):
         super().__init__("FlyteSchema Transformer", FlyteSchema)
@@ -355,23 +427,30 @@ class FlyteSchemaTransformer(TypeTransformer[FlyteSchema]):
     def get_literal_type(self, t: Type[FlyteSchema]) -> LiteralType:
         return LiteralType(schema=self._get_schema_type(t))
 
-    def to_literal(
+    async def async_to_literal(
         self, ctx: FlyteContext, python_val: FlyteSchema, python_type: Type[FlyteSchema], expected: LiteralType
     ) -> Literal:
         if isinstance(python_val, FlyteSchema):
             remote_path = python_val.remote_path
             if remote_path is None or remote_path == "":
-                remote_path = ctx.file_access.get_random_remote_path()
+                remote_path = ctx.file_access.join(
+                    ctx.file_access.raw_output_prefix,
+                    ctx.file_access.get_random_string(),
+                    ctx.file_access.get_file_tail(python_val.local_path),
+                )
             if python_val.supported_mode == SchemaOpenMode.READ and not python_val._downloaded:
                 # This means the local path is empty. Don't try to overwrite the remote data
                 logger.debug(f"Skipping upload for {python_val} because it was never downloaded.")
             else:
-                ctx.file_access.put_data(python_val.local_path, remote_path, is_multipart=True)
+                remote_path = await ctx.file_access.async_put_data(
+                    python_val.local_path, remote_path, is_multipart=True
+                )
             return Literal(scalar=Scalar(schema=Schema(remote_path, self._get_schema_type(python_type))))
 
+        remote_path = ctx.file_access.join(ctx.file_access.raw_output_prefix, ctx.file_access.get_random_string())
         schema = python_type(
             local_path=ctx.file_access.get_random_local_directory(),
-            remote_path=ctx.file_access.get_random_remote_directory(),
+            remote_path=remote_path,
         )
         try:
             h = SchemaEngine.get_handler(type(python_val))
@@ -382,10 +461,95 @@ class FlyteSchemaTransformer(TypeTransformer[FlyteSchema]):
         writer = schema.open(type(python_val))
         writer.write(python_val)
         if not h.handles_remote_io:
-            ctx.file_access.put_data(schema.local_path, schema.remote_path, is_multipart=True)
+            schema.remote_path = await ctx.file_access.async_put_data(
+                schema.local_path, schema.remote_path, is_multipart=True
+            )
         return Literal(scalar=Scalar(schema=Schema(schema.remote_path, self._get_schema_type(python_type))))
 
-    def to_python_value(self, ctx: FlyteContext, lv: Literal, expected_python_type: Type[FlyteSchema]) -> FlyteSchema:
+    def dict_to_flyte_schema(
+        self, dict_obj: typing.Dict[str, str], expected_python_type: Type[FlyteSchema]
+    ) -> FlyteSchema:
+        remote_path = dict_obj.get("remote_path", None)
+
+        if remote_path is None:
+            raise ValueError("FlyteSchema's path should not be None")
+
+        t = FlyteSchemaTransformer()
+        return t.to_python_value(
+            FlyteContextManager.current_context(),
+            Literal(scalar=Scalar(schema=Schema(remote_path, t._get_schema_type(expected_python_type)))),
+            expected_python_type,
+        )
+
+    def from_binary_idl(self, binary_idl_object: Binary, expected_python_type: Type[FlyteSchema]) -> FlyteSchema:
+        """
+        If the input is from flytekit, the Life Cycle will be as follows:
+
+        Life Cycle:
+        binary IDL                 -> resolved binary         -> bytes                   -> expected Python object
+        (flytekit customized          (propeller processing)     (flytekit binary IDL)      (flytekit customized
+        serialization)                                                                       deserialization)
+
+        Example Code:
+        @dataclass
+        class DC:
+            fs: FlyteSchema
+
+        @workflow
+        def wf(dc: DC):
+            t_fs(dc.fs)
+
+        Note:
+        - The deserialization is the same as put a flyte schema in a dataclass, which will deserialize by the mashumaro's API.
+
+        Related PR:
+        - Title: Override Dataclass Serialization/Deserialization Behavior for FlyteTypes via Mashumaro
+        - Link: https://github.com/flyteorg/flytekit/pull/2554
+        """
+        if binary_idl_object.tag == MESSAGEPACK:
+            python_val = msgpack.loads(binary_idl_object.value)
+            return self.dict_to_flyte_schema(dict_obj=python_val, expected_python_type=expected_python_type)
+        else:
+            raise TypeTransformerFailedError(f"Unsupported binary format: `{binary_idl_object.tag}`")
+
+    def from_generic_idl(self, generic: Struct, expected_python_type: Type[FlyteSchema]) -> FlyteSchema:
+        """
+        If the input is from Flyte Console, the Life Cycle will be as follows:
+
+        Life Cycle:
+        json str            -> protobuf struct         -> resolved protobuf struct   -> expected Python object
+        (console user input)   (console output)           (propeller)                   (flytekit customized deserialization)
+
+        Example Code:
+        @dataclass
+        class DC:
+            fs: FlyteSchema
+
+        @workflow
+        def wf(dc: DC):
+            t_fs(dc.fs)
+
+        Note:
+        - The deserialization is the same as put a flyte schema in a dataclass, which will deserialize by the mashumaro's API.
+
+        Related PR:
+        - Title: Override Dataclass Serialization/Deserialization Behavior for FlyteTypes via Mashumaro
+        - Link: https://github.com/flyteorg/flytekit/pull/2554
+        """
+        json_str = _json_format.MessageToJson(generic)
+        python_val = json.loads(json_str)
+        return self.dict_to_flyte_schema(dict_obj=python_val, expected_python_type=expected_python_type)
+
+    async def async_to_python_value(
+        self, ctx: FlyteContext, lv: Literal, expected_python_type: Type[FlyteSchema]
+    ) -> FlyteSchema:
+        # Handle dataclass attribute access
+        if lv.scalar:
+            if lv.scalar.binary:
+                return self.from_binary_idl(lv.scalar.binary, expected_python_type)
+            if lv.scalar.generic:
+                return self.from_generic_idl(lv.scalar.generic, expected_python_type)
+
         def downloader(x, y):
             ctx.file_access.get_data(x, y, is_multipart=True)
 
@@ -418,9 +582,9 @@ class FlyteSchemaTransformer(TypeTransformer[FlyteSchema]):
             elif literal_column.type == SchemaType.SchemaColumn.SchemaColumnType.STRING:
                 columns[literal_column.name] = str
             elif literal_column.type == SchemaType.SchemaColumn.SchemaColumnType.DATETIME:
-                columns[literal_column.name] = _datetime.datetime
+                columns[literal_column.name] = datetime.datetime
             elif literal_column.type == SchemaType.SchemaColumn.SchemaColumnType.DURATION:
-                columns[literal_column.name] = _datetime.timedelta
+                columns[literal_column.name] = datetime.timedelta
             elif literal_column.type == SchemaType.SchemaColumn.SchemaColumnType.BOOLEAN:
                 columns[literal_column.name] = bool
             else:
