@@ -5,7 +5,7 @@ from typing import Type
 import pandas
 
 from flytekit import FlyteContext
-from flytekit.core.type_engine import T, TypeEngine, TypeTransformer
+from flytekit.core.type_engine import AsyncTypeTransformer, T, TypeEngine
 from flytekit.models.literals import Literal, Scalar, Schema
 from flytekit.models.types import LiteralType, SchemaType
 from flytekit.types.schema import LocalIOSchemaReader, LocalIOSchemaWriter, SchemaEngine, SchemaFormat, SchemaHandler
@@ -17,7 +17,9 @@ class ParquetIO(object):
     def _read(self, chunk: os.PathLike, columns: typing.Optional[typing.List[str]], **kwargs) -> pandas.DataFrame:
         return pandas.read_parquet(chunk, columns=columns, engine=self.PARQUET_ENGINE, **kwargs)
 
-    def read(self, *files: os.PathLike, columns: typing.List[str] = None, **kwargs) -> pandas.DataFrame:
+    def read(
+        self, *files: os.PathLike, columns: typing.Optional[typing.List[str]] = None, **kwargs
+    ) -> pandas.DataFrame:
         frames = [self._read(chunk=f, columns=columns, **kwargs) for f in files if os.path.getsize(f) > 0]
         if len(frames) == 1:
             return frames[0]
@@ -56,7 +58,7 @@ class ParquetIO(object):
 
 
 class PandasSchemaReader(LocalIOSchemaReader[pandas.DataFrame]):
-    def __init__(self, local_dir: os.PathLike, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
+    def __init__(self, local_dir: str, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
         super().__init__(local_dir, cols, fmt)
         self._parquet_engine = ParquetIO()
 
@@ -65,7 +67,7 @@ class PandasSchemaReader(LocalIOSchemaReader[pandas.DataFrame]):
 
 
 class PandasSchemaWriter(LocalIOSchemaWriter[pandas.DataFrame]):
-    def __init__(self, local_dir: os.PathLike, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
+    def __init__(self, local_dir: str, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
         super().__init__(local_dir, cols, fmt)
         self._parquet_engine = ParquetIO()
 
@@ -73,7 +75,7 @@ class PandasSchemaWriter(LocalIOSchemaWriter[pandas.DataFrame]):
         return self._parquet_engine.write(df, to_file=path, **kwargs)
 
 
-class PandasDataFrameTransformer(TypeTransformer[pandas.DataFrame]):
+class PandasDataFrameTransformer(AsyncTypeTransformer[pandas.DataFrame]):
     """
     Transforms a pd.DataFrame to Schema without column types.
     """
@@ -89,7 +91,7 @@ class PandasDataFrameTransformer(TypeTransformer[pandas.DataFrame]):
     def get_literal_type(self, t: Type[pandas.DataFrame]) -> LiteralType:
         return LiteralType(schema=self._get_schema_type())
 
-    def to_literal(
+    async def async_to_literal(
         self,
         ctx: FlyteContext,
         python_val: pandas.DataFrame,
@@ -99,17 +101,20 @@ class PandasDataFrameTransformer(TypeTransformer[pandas.DataFrame]):
         local_dir = ctx.file_access.get_random_local_directory()
         w = PandasSchemaWriter(local_dir=local_dir, cols=None, fmt=SchemaFormat.PARQUET)
         w.write(python_val)
-        remote_path = ctx.file_access.get_random_remote_directory()
-        ctx.file_access.put_data(local_dir, remote_path, is_multipart=True)
+        remote_path = ctx.file_access.join(
+            ctx.file_access.raw_output_prefix,
+            ctx.file_access.get_random_string(),
+        )
+        remote_path = await ctx.file_access.async_put_data(local_dir, remote_path, is_multipart=True)
         return Literal(scalar=Scalar(schema=Schema(remote_path, self._get_schema_type())))
 
-    def to_python_value(
+    async def async_to_python_value(
         self, ctx: FlyteContext, lv: Literal, expected_python_type: Type[pandas.DataFrame]
     ) -> pandas.DataFrame:
         if not (lv and lv.scalar and lv.scalar.schema):
             return pandas.DataFrame()
         local_dir = ctx.file_access.get_random_local_directory()
-        ctx.file_access.get_data(lv.scalar.schema.uri, local_dir, is_multipart=True)
+        await ctx.file_access.async_get_data(lv.scalar.schema.uri, local_dir, is_multipart=True)
         r = PandasSchemaReader(local_dir=local_dir, cols=None, fmt=SchemaFormat.PARQUET)
         return r.all()
 
